@@ -32,17 +32,15 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: arg.c,v 1.7 97/10/10 08:26:20 abe Exp $";
+static char *rcsid = "$Id: arg.c,v 1.12 98/02/16 15:55:38 abe Exp $";
 #endif
 
 
 #include "lsof.h"
-#include "version.h"
 
 
 _PROTOTYPE(static int ckfd_range,(char *first, char *dash, char *last, int *lo, int *hi));
 _PROTOTYPE(static int enter_nwad,(struct nwad *n, int sp, int ep, char *s, struct hostent *he));
-_PROTOTYPE(static char *isnullstr,(char *s));
 
 
 /*
@@ -70,7 +68,7 @@ ckfd_range(first, dash, last, lo, hi)
  * Assemble and check the high and low values.
  */
 	for (cp = first, *lo = 0; *cp && cp < dash; cp++) {
-	    if (!isdigit(*cp)) {
+	    if (!isdigit((unsigned char)*cp)) {
 
 FD_range_nondigit:
 
@@ -81,7 +79,7 @@ FD_range_nondigit:
 	    *lo = (*lo * 10) + (int)(*cp - '0');
 	}
 	for (cp = dash+1, *hi = 0; *cp && cp < last; cp++) {
-	    if (!isdigit(*cp))
+	    if (!isdigit((unsigned char)*cp))
 		goto FD_range_nondigit;
 	    *hi = (*hi * 10) + (int)(*cp - '0');
 	}
@@ -94,6 +92,318 @@ FD_range_nondigit:
 }
 
 
+/*
+ * ck_file_arg() - check file arguments
+ */
+
+int
+ck_file_arg(i, ac, av)
+	int i;			/* first file argument index */
+	int ac;			/* argument count */
+	char *av[];		/* argument vector */
+{
+	unsigned char ad, an;
+	short err = 0;
+	char *fnm, *fsnm, *path;
+	int fsm, ftype, j;
+	MALLOC_S l;
+	struct mounts *mp;
+	static struct mounts **mmp = (struct mounts **)NULL;
+	int mx, nm;
+	static int nma = 0;
+	struct stat sb;
+	struct sfile *sfp;
+
+#if	defined(CKFA_EXPDEV)
+	dev_t dev, rdev;
+#endif	/* defined(CKFA_EXPDEV) */
+
+#if	defined(HASPROCFS)
+	short pfsnl = -1;
+	pid_t pid;
+	struct procfsid *pfi;
+#endif	/* defined(HASPROCFS) */
+
+/*
+ * Loop through arguments.
+ */
+	for (; i < ac; i++) {
+	    if (!(path = Readlink(av[i]))) {
+		err = 1;
+		continue;
+	    }
+	/*
+	 * Remove extra terminating `/'.
+	 *
+	 * Check for file system argument.
+	 */
+	    if ((j = strlen(path)) > 1 && path[j-1] == '/')
+		path[j-1] = '\0';
+	    for (ftype = 1, mp = Mtab, nm = 0;
+		 (Ffilesys != 1) && mp;
+		 mp = mp->next)
+	    {
+		fsm = 0;
+		if (strcmp(mp->dir, path) == 0)
+		    fsm++;
+		else if (Ffilesys == 2 || (mp->fs_mode & S_IFMT) == S_IFBLK) {
+		    if (mp->fsnmres && strcmp(mp->fsnmres, path) == 0)
+			fsm++;
+		}
+		if (!fsm)
+		    continue;
+		ftype = 0;
+	    /*
+	     * Skip duplicates.
+	     */
+		for (mx = 0; mx < nm; mx++) {
+		    if (strcmp(mp->dir, mmp[mx]->dir) == 0
+		    &&  mp->dev == mmp[mx]->dev
+		    &&  mp->inode == mmp[mx]->inode)
+			break;
+		}
+		if (mx < nm)
+		    continue;
+	    /*
+	     * Allocate space for and save another mount point match and
+	     * the type of match -- directory name (mounted) or file system
+	     * name (mounted-on).
+	     */
+		if (nm >= nma) {
+		    nma += 5;
+		    l = (MALLOC_S)(nma * sizeof(struct mounts *));
+		    if (mmp)
+			mmp = (struct mounts **)realloc((MALLOC_P *)mmp, l);
+		    else
+			mmp = (struct mounts **)malloc(l);
+		    if (!mmp) {
+			(void) fprintf(stderr,
+			    "%s: no space for mount pointers\n", Pn);
+			Exit(1);
+		    }
+		}
+		mmp[nm++] = mp;
+	    }
+	    if (Ffilesys == 2 && nm == 0) {
+		(void) fprintf(stderr, "%s: not a file system: %s\n",
+		    Pn, av[i]);
+		err = 1;
+		continue;
+	    }
+	/*
+	 * Loop through the file system matches.  If there were none, make one
+	 * pass through the loop, using simply the path name.
+	 */
+	    mx = 0;
+	    do {
+
+	    /*
+	     * Allocate an sfile structure and fill in the type and link.
+	     */
+		if (!(sfp = (struct sfile *)malloc(sizeof(struct sfile)))) {
+		    (void) fprintf(stderr, "%s: no space for files\n", Pn);
+		    Exit(1);
+		}
+		sfp->next = Sfile;
+		Sfile = sfp;
+		sfp->f = 0;
+		if ((sfp->type = ftype)) {
+
+		/*
+		 * For a non-file system path, use the path as the file name
+		 * and set a NULL file system name.
+		 */
+		    fnm = path;
+		    fsnm = (char *)NULL;
+		/*
+		 * Stat the path to obtain its characteristics.
+		 */
+		    if (statsafely(fnm, &sb) != 0) {
+			(void) fprintf(stderr, "%s: status error on %s: %s\n",
+			    Pn, fnm, strerror(errno));
+			err = 1;
+			break;
+		    }
+		    sfp->i = sb.st_ino;
+		    sfp->mode = sb.st_mode & S_IFMT;
+		
+#if	defined(CKFA_EXPDEV)
+		/*
+		 * Expand device numbers before saving, so that they match the
+		 * already-expanded Mtab device numbers.  (This is an EP/IX
+		 * 2.1.1 and above artifact.)
+		 */
+		    dev = expdev(sb.st_dev);
+		    rdev = expdev(sb.st_rdev);
+#endif	/* defined(CKFA_EXPDEV) */
+
+		    if (sfp->mode == S_IFBLK || sfp->mode == S_IFCHR
+
+#if	defined(CKFA_XDEVTST)
+		    ||  CKFA_XDEVTST
+#endif	/* defined(CKFA_XDEVTST) */
+
+		    )
+
+#if	defined(CKFA_EXPDEV)
+			sfp->dev = rdev;
+		    else
+			sfp->dev = dev;
+#else	/* !defined(CKFA_EXPDEV) */
+			sfp->dev = sb.st_rdev;
+		    else
+			sfp->dev = sb.st_dev;
+#endif	/* defined(CKFA_EXPDEV) */
+
+#if	defined(CKFA_MPXCHAN)
+		/*
+		 * Save a (possible) multiplexed channel number.  (This is an
+		 * AIX artifact.)
+		 */
+		    sfp->ch = getchan(path);
+#endif	/* defined(CKFA_MPXCHAN) */
+
+		} else {
+		    mp = mmp[mx++];
+
+#if	defined(HASPROCFS)
+		/*
+		 * If this is a /proc file system, set the search flag and
+		 * abandon the sfile entry.
+		 */
+		    if (mp == Mtprocfs) {
+			Sfile = sfp->next;
+			(void) free((FREE_P *)sfp);
+			Procsrch = 1;
+			continue;
+		    }
+#endif	/* defined(HASPROCFS) */
+
+		/*
+		 * Derive file name and file system name for a mount point.
+		 *
+		 * Save the device number, inode number, and modes.
+		 */
+		    fnm = mp->dir;
+		    fsnm = mp->fsname;
+		    sfp->dev = mp->dev;
+		    sfp->i = mp->inode;
+		    sfp->mode = mp->mode & S_IFMT;
+		}
+	    /*
+	     * Store the file name and file system name pointers in the sfile
+	     * structure, allocating space as necessary.
+	     */
+		if (!fnm || fnm == path) {
+		    sfp->name = fnm;
+		    an = 0;
+		} else {
+		    if (!(sfp->name=(char *)malloc((MALLOC_S)(strlen(fnm)+1))))
+		    {
+			(void) fprintf(stderr,
+			    "%s: no space for file name %s\n", Pn, fnm);
+			Exit(1);
+		    }
+		    (void) strcpy(sfp->name, fnm);
+		    an = 1;
+		}
+		if (!fsnm || fsnm == path) {
+		    sfp->devnm = fsnm;
+		    ad = 0;
+		} else {
+		    if (!(sfp->devnm=(char *)malloc((MALLOC_S)(strlen(fsnm)+1)))
+		    ) {
+			(void) fprintf(stderr,
+			    "%s: no space for file system name %s\n", Pn, fsnm);
+			Exit(1);
+		    }
+		    (void) strcpy(sfp->devnm, fsnm);
+		    ad = 1;
+		}
+		if (!(sfp->aname=(char *)malloc((MALLOC_S)(strlen(av[i])+1)))) {
+		    (void) fprintf(stderr,
+			"%s: no space for argument file name %s\n", Pn, av[i]);
+		    Exit(1);
+		}
+		(void) strcpy(sfp->aname, av[i]);
+
+#if	defined(HASPROCFS)
+	    /*
+	     * See if this is an individual member of a proc file system.
+	     */
+		if (!Mtprocfs || Procsrch)
+		    continue;
+
+# if	defined(HASFSTYPE)
+		if (strcmp(sb.st_fstype, HASPROCFS) != 0)
+		    continue;
+# endif	/* defined(HASFSTYPE) */
+
+		if (pfsnl == -1)
+		    pfsnl = strlen(Mtprocfs->dir);
+		if (!pfsnl)
+		    continue;
+		if (strncmp(Mtprocfs->dir, path, pfsnl) != 0)
+		    continue;
+		if (path[pfsnl] != '/')
+
+# if	defined(HASPINODEN)
+		    pid = 0;
+# else	/* !defined(HASPINODEN) */
+		    continue;
+# endif	/* defined(HASPINODEN) */
+
+		else {
+		    for (j = pfsnl+1; path[j]; j++) {
+			if (!isdigit(path[j]))
+			    break;
+		    }
+		    if (path[j] || (j - pfsnl - 1) < 1
+		    ||  (sfp->mode & S_IFMT) != S_IFREG)
+
+# if	defined(HASPINODEN)
+			pid = 0;
+# else	/* !defined(HASPINODEN) */
+			continue;
+# endif	/* defined(HASPINODEN) */
+
+		    else
+			pid = atoi(&path[pfsnl+1]);
+		}
+		if (!(pfi = (struct procfsid *)malloc((MALLOC_S)
+			     sizeof(struct procfsid))))
+		{
+		    (void) fprintf(stderr, "%s: no space for %s ID: %s\n",
+			Pn, Mtprocfs->dir, path);
+		    Exit(1);
+		}
+		pfi->pid = pid;
+		pfi->f = 0;
+		pfi->nm = sfp->aname;
+		pfi->next = Procfsid;
+		Procfsid = pfi;
+
+# if	defined(HASPINODEN)
+		pfi->inode = (unsigned long)sfp->i;
+# endif	/* defined(HASPINODEN) */
+
+	    /*
+	     * Abandon the Sfile entry, lest it be used in is_file_named().
+	     */
+		Sfile = sfp->next;
+		if (ad)
+		    (void) free((FREE_P *)sfp->devnm);
+		if (an)
+		    (void) free((FREE_P *)sfp->name);
+		(void) free((FREE_P *)sfp);
+#endif	/* defined(HASPROCFS) */
+
+	    } while (mx < nm);
+	}
+	return((int)err);
+}
+
+
 #if	defined(HASDCACHE)
 /*
  * ctrl_dcache() - enter device cache control
@@ -198,7 +508,7 @@ int
 enter_fd(f)
 	char *f;			/* file descriptor list pointer */
 {
-	char buf[32], c, *cp1, *cp2, *cp3, *dash;
+	char buf[32], c, *cp1, *cp2, *dash;
 	int err, hi, lo;
 	char *fc;
 	MALLOC_S len;
@@ -211,7 +521,7 @@ enter_fd(f)
 	}
 	if ((fc = (char *)malloc(len)) == (char *)NULL) {
 	    (void) fprintf(stderr, "%s: no space for %d byte fd string: %s\n",
-		Pn, len, f);
+		Pn, (int)len, f);
 	    Exit(1);
 	}
 	(void) strcpy(fc, f);
@@ -303,12 +613,12 @@ enter_id(ty, p)
 	/*
 	 * Assemble ID.
 	 */
-	    for (id = 0; *cp && *cp != ','; *cp++) {
+	    for (id = 0; *cp && *cp != ','; cp++) {
 
 #if	defined(__STDC__)
-		if ( ! isdigit(*cp))
+		if (!isdigit((unsigned char)*cp))
 #else
-		if ( ! isascii(*cp) || ! isdigit(*cp))
+		if (!isascii(*cp) || ! isdigit((unsigned char)*cp))
 #endif	/* __STDC__ */
 
 		{
@@ -374,15 +684,17 @@ enter_network_address(na)
 	char *na;			/* Internet address string pointer */
 {
 	unsigned char *ap;
-	int ep, pr, pt, pu, sp;
+	int ep, pr, sp;
 	struct hostent *he = (struct hostent *) NULL;
 	char *hn = NULL;
 	MALLOC_S l;
 	struct nwad n;
 	char *p, *wa;
+	int pt = 0;
+	int pu = 0;
 	struct servent *se, *se1;
 	char *sn = (char *)NULL;
-	int snl = 0;
+	MALLOC_S snl = 0;
 
 	if (na == NULL) {
 	    (void) fprintf(stderr, "%s: no network address specified\n", Pn);
@@ -589,7 +901,7 @@ unacc_port:
 			if (se && se1 && pt != pu) {
 			    (void) fprintf(stderr,
 				"%s: TCP=%d and UDP=%d %s ports conflict;\n",
-				Pn, pt, pu, sn, na);
+				Pn, pt, pu, sn);
 			    (void) fprintf(stderr,
 				"      specify \"tcp:%s\" or \"udp:%s\".\n",
 				sn, sn);
@@ -820,9 +1132,9 @@ enter_uid(u)
 		    continue;
 
 #if	defined(__STDC__)
-		if (isdigit(*s))
+		if (isdigit((unsigned char)*s))
 #else	/* !defined(__STDC__) */
-		if (isascii(*s) && isdigit(*s))
+		if (isascii(*s) && isdigit((unsigned char)*s))
 #endif	/* defined(__STDC__) */
 
 		    uid = (uid * 10) + *s - '0';
@@ -868,7 +1180,8 @@ enter_uid(u)
 		    continue;
 		}
 		(void) fprintf(stderr,
-		    "%s: UID %d has been included and excluded\n", Pn, uid);
+		    "%s: UID %d has been included and excluded\n",
+			Pn, (int)uid);
 		err = j = 1;
 		break;
 	    }
@@ -908,448 +1221,3 @@ enter_uid(u)
 	}
 	return(err);
 }
-
-
-/*
- * isnullstr() - is it a null string?
- */
-
-static char *
-isnullstr(s)
-	char *s;			/* string pointer */
-{
-	if (!s)
-		return((char *)NULL);
-	while (*s) {
-		if (*s != ' ')
-			return(s);
-		s++;
-	}
-	return((char *)NULL);
-}
-
-
-/*
- * usage() - display usage and exit
- */
-
-void
-usage(xv, fh, version)
-	int xv;				/* exit value */
-	int fh;				/* ``-F ?'' status */
-	int version;			/* ``-v'' status */
-{
-	char buf[MAXPATHLEN+1], *cp, *cp1, *cp2;
-	int dx = -2;
-	int  i;
-
-	if (Fhelp || xv) {
-	    (void) fprintf(stderr, "%s %s (latest revision at %s)\n",
-		Pn, LSOF_VERSION, LSOF_URL);
-	    (void) fprintf(stderr,
-		" usage: [-?ab%shlnNoOP%sstUvVw%s]",
-
-#if	defined(HASNCACHE)
-		"C",
-#else	/* !defined(HASNCACHE) */
-		"",
-#endif	/* defined(HASNCACHE) */
-
-#if	defined(HASPPID)
-		"R",
-#else	/* !defined(HASPPID) */
-		"",
-#endif	/* defined(HASPPID) */
-
-#if	defined(HASXOPT)
-# if	defined(HASXOPT_ROOT)
-		(Myuid == 0) ? "X" : ""
-# else	/* !defined(HASXOPT_ROOT) */
-		"X"
-# endif	/* defined(HASXOPT_ROOT) */
-#else	/* !defined(HASXOPT) */
-		""
-#endif	/* defined(HASXOPT) */
-
-	    );
-
-#if	defined(HAS_AFS) && defined(HASAOPT)
-	    (void) fprintf(stderr, " [-A A]");
-#endif	/* defined(HAS_AFS) && defined(HASAOPT) */
-
-	    (void) fprintf(stderr, " [-c c] [-d s]");
-
-#if	defined(HASDCACHE)
-	    (void) fprintf(stderr, " [-D D]");
-#endif	/* defined(HASDCACHE) */
-
-	    (void) fprintf(stderr, " [-F [f]] [-g [s]]\n [-i [i]]");
-
-#if	defined(HASKOPT)
-	    (void) fprintf(stderr, " [-k k]");
-#endif	/* defined(HASKOPT) */
-
-#if	defined(HASMOPT)
-	    (void) fprintf(stderr, " [-m m]");
-#endif	/* defined(HASMOPT) */
-
-	    (void) fprintf(stderr,
-	    " [-p s] [-r [t]] [-S [t]] [-T [t]] [-u s] [-U] [--]\n [names]\n");
-	}
-	if (xv && !Fhelp) {
-	    (void) fprintf(stderr,
-		"Use the ``-h'' option to get more help information.\n");
-	    if (!fh)
-    		Exit(xv);
-	}
-	if (Fhelp) {
-	    (void) fprintf(stderr, "  %-23.23s",
-		"-? list help");
-	    (void) fprintf(stderr, "  %-25.25s",
-		"-a AND selections (OR)");
-	    (void) fprintf(stderr, "  %s\n",
-		"-b avoid kernel blocks");
-
-	    (void) fprintf(stderr, "  %-23.23s",
-
-#if	defined(HASNCACHE)
-		"-C no kernel name cache");
-#else	/* !defined(HASNCACHE) */
-		"                       ");
-#endif	/* defined(HASNCACHE) */
-
-	    (void) fprintf(stderr, "  %-25.25s",
-		"-h list help");
-	    (void) fprintf(stderr, "  %s\n",
-		"-i select Internet files");
-	    (void) fprintf(stderr, "  %-23.23s",
-		"-l list UID numbers");
-	    (void) fprintf(stderr, "  %-25.25s",
-		"-n no host names");
-	    (void) fprintf(stderr, "  %s\n",
-		"-N select NFS files");
-	    (void) fprintf(stderr, "  %-23.23s",
-		"-o list file offset");
-	    (void) fprintf(stderr, "  %-25.25s",
-		"-O avoid overhead *RISKY*");
-	    (void) fprintf(stderr, "  %s\n",
-		"-P no port names");
-	    (void) fprintf(stderr, "  %-23.23s",
-
-#if	defined(HASPPID)
-		"-R list paRent PID"
-#else	/* !defined(HASPPID) */
-		""
-#endif	/* defined(HASPPID) */
-
-	    );
-
-	    (void) fprintf(stderr, "  %-23.23s",
-		"-s list file size");
-	    (void) fprintf(stderr, "  %s\n",
-		"  -t terse listing");
-	    (void) fprintf(stderr, "  %-23.23s",
-		"-T disable TCP/TPI info");
-	    (void) fprintf(stderr, "  %-25.25s",
-		"-U select Unix socket");
-	    (void) fprintf(stderr, "  -v display version info\n");
-	    (void) fprintf(stderr, "  %-23.23s",
-		"-V verbose search");
-	    (void) sprintf(buf, "-w %s warnings",
-
-#if	defined(WARNINGSTATE)
-		"enable"
-#else	/* !defined(WARNINGSTATE) */
-		"disable"
-#endif	/* defined(WARNINGSTATE) */
-
-	    );
-
-#if	defined(HASXOPT)
-	    (void) fprintf(stderr, "  %-25.25s", buf);
-# if	defined(HASXOPT_ROOT)
-	    if (Myuid == 0)
-		(void) fprintf(stderr, "  -X %s\n", HASXOPT);
-# else	/* !defined(HASXOPT_ROOT) */
-	    (void) fprintf(stderr, "  -X %s\n", HASXOPT);
-# endif	/* defined(HASXOPT_ROOT) */
-#else	/* !defined(HASXOPT) */
-	    (void) fprintf(stderr, "  %s\n", buf);
-#endif	/* defined(HASXOPT) */
-
-	    (void) fprintf(stderr, "Defaults are enclosed in parentheses.");
-	    (void) fprintf(stderr,
-		"  Separate individual items in a set (s)\n");
-	    (void) fprintf(stderr,
-		"by commas and separate ranges by dashes.\n");
-
-#if	defined(HAS_AFS) && defined(HASAOPT)
-	    (void) fprintf(stderr,
-		"  -A A   AFS name list file (%s)\n", AFSAPATHDEF);
-#endif	/* defined(HAS_AFS) && defined(HASAOPT) */
-
-	    (void) fprintf(stderr, "  %-36.36s",
-		"-c c   list command c");
-
-#if	defined(HASDCACHE)
-	    if (Setuidroot)
-		cp = "?|i|r";
-
-# if	!defined(WILLDROPGID)
-	    else if (Myuid)
-		cp = "?|i|r<path>";
-# endif	/* !defined(WILLDROPGID) */
-
-	    else
-		cp = "?|i|b|r|u[path]";
-	    (void) fprintf(stderr, "  %s%s\n",
-		"-D D  ", cp);
-#else	/* !defined(HASDCACHE) */
-	    putc('\n', stderr);
-#endif	/* defined(HASDCACHE) */
-
-	    (void) fprintf(stderr, "  %-36.36s",
-		"-d s   select by FD set");
-	    (void) fprintf(stderr, "  %s\n",
-		"-F [f] select fields (-F? for help)");
-	    (void) fprintf(stderr,
-		"  -g [s] select by process group ID set and print");
-	    (void) fprintf(stderr,
-		" process group IDs\n");
-	    (void) fprintf(stderr,
-		"  -i i   select by Internet address:");
-	    (void) fprintf(stderr,
-		" [proto][@host|addr][:svc_list|port_list]\n");
-
-#if	defined(HASKOPT)
-
-	    (void) fprintf(stderr,
-		"  -k k   kernel symbol file (%s)\n",
-		Nmlst ? Nmlst
-
-# if	defined(N_UNIX)
-		      : N_UNIX
-# else	/* !defined(N_UNIX) */
-		      : (Nmlst = get_nlist_path(1)) ? Nmlst
-						    : "not yet determined"
-# endif	/* defined(N_UNIX) */
-
-	    );
-#endif	/* defined(HASKOPT) */
-
-# if	defined(HASMOPT)
-	    (void) fprintf(stderr,
-		"  -m m   kernel memory file (%s)\n", KMEM);
-#endif	/* defined(HASMOPT) */
-
-	    (void) fprintf(stderr, "  %-36.36s",
-		"-p s   select by PID set");
-	    (void) fprintf(stderr,
-		"  -r [t] repeat every t seconds (%d)\n", RPTTM);
-	    (void) sprintf(buf,
-		"-S [t] t second stat timeout (%d)", TMLIMIT);
-	    (void) fprintf(stderr, "  %-36.36s", buf);
-	    (void) fprintf(stderr, "  %s\n",
-		"-u s   exclude(^)/select login/UID set");
-	    (void) fprintf(stderr,
-		"  -T %ss%s TCP/TPI info:%s s=state%s (s)\n",
-
-#if 	defined(HASTCPTPIQ)
-		"q",
-#else	/* !defined(HASTCPTPIQ) */
-		" ",
-#endif	/* defined(HASTCPTPIQ) */
-
-#if 	defined(HASTCPTPIW)
-		"w",
-#else	/* !defined(HASTCPTPIW) */
-		" ",
-#endif	/* defined(HASTCPTPIW) */
-
-#if 	defined(HASTCPTPIQ)
-		" q=queues",
-#else	/* !defined(HASTCPTPIQ) */
-		"",
-#endif	/* defined(HASTCPTPIQ) */
-
-#if 	defined(HASTCPTPIW)
-		" w=windows"
-#else	/* !defined(HASTCPTPIW) */
-		""
-#endif	/* defined(HASTCPTPIW) */
-
-		);
-	    (void) fprintf(stderr,
-		"  names  select named files or files on named file systems\n");
-	    (void) fprintf(stderr, "%s can list all files.",
-
-#if	defined(HASSECURITY)
-		"Only root"
-#else	/* !defined(HASSECURITY) */
-		"Anyone"
-#endif	/* defined(HASSECURITY) */
-
-	    );
-	    (void) fprintf(stderr, "  Inaccessible /dev warnings are %s.\n",
-
-#if	defined(WARNDEVACCESS)
-		"enabled"
-#else	/* !defined(WARNDEVACCESS) */
-		"disabled"
-#endif	/* defined(WARNDEVACCESS) */
-
-	    );
-
-#if defined(HASDCACHE)
-# if	defined(HASENVDC) || defined(HASPERSDC) || defined(HASSYSDC)
-	    cp = NULL;
-#  if	defined(HASENVDC)
-	    if (dx == -2 && (dx = dcpath(1, 0)) >= 0)
-		cp = DCpath[1];
-#  endif	/* defined(HASENVDC) */
-#  if	defined(HASSYSDC)
-	    if (!cp)
-		cp = HASSYSDC;
-#  endif	/* defined(HASSYSDC) */
-#  if	defined(HASPERSDC)
-	    if (!cp && dx != -1 && (dx = dcpath(1, 0)) >= 0)
-		cp = DCpath[3];
-#  endif	/* defined(HASPERSDC) */
-	    if (cp)
-		(void) fprintf(stderr,
-		    "%s is the default device cache file read path.\n", cp);
-# endif    /* defined(HASENVDC) || defined(HASPERSDC) || defined(HASSYSDC) */
-#endif	/* defined(HASDCACHE) */
-
-	}
-	if (fh) {
-	    (void) fprintf(stderr, "%s:\tID    field description\n", Pn);
-	    for (i = 0; FieldSel[i].nm; i++) {
-
-#if	!defined(HASPPID)
-		if (FieldSel[i].id == LSOF_FID_PPID)
-		    continue;
-#endif	/* !defined(HASPPID) */
-
-		(void) fprintf(stderr, "\t %c    %s\n",
-		    FieldSel[i].id, FieldSel[i].nm);
-	    }
-	}
-
-#if	defined(HASDCACHE)
-	if (DChelp) {
-
-	/*
-	 * Display device cache file read-only and write paths.
-	 */
-	    (void) fprintf(stderr, "%s: device cache file read-only paths:\n",
-		Pn);
-	    if ((dx = dcpath(1, 0)) < 0)
-		(void) fprintf(stderr, "\tnone\n");
-	    else {
-		(void) fprintf(stderr, "\tNamed via -D: %s\n",
-		    DCpath[0] ? DCpath[0] : "none");
-
-# if	defined(HASENVDC)
-		(void) fprintf(stderr,
-		    "\tNamed in environment variable %s: %s\n",
-		    HASENVDC, DCpath[1] ? DCpath[1] : "none");
-# endif	/* defined(HASENVDC) */
-
-# if	defined(HASSYSDC)
-		if (DCpath[2])
-		    (void) fprintf(stderr,
-			"\tSystem-wide device cache: %s\n", DCpath[2]);
-# endif	/* defined(HASSYSDC) */
-
-# if	defined(HASPERSDC)
-		(void) fprintf(stderr,
-		    "\tPersonal path format (HASPERSDC): \"%s\"\n",
-		    HASPERSDC);
-#  if	defined(HASPERSDCPATH)
-		(void) fprintf(stderr,
-		    "\tModified personal path environment variable: %s\n",
-		    HASPERSDCPATH);
-		cp = getenv(HASPERSDCPATH);
-		(void) fprintf(stderr, "\t%s value: %s\n",
-			HASPERSDCPATH, cp ? cp : "none");
-#  endif	/* defined(HASPERSDCPATH) */
-		(void) fprintf(stderr, "\tPersonal path: %s\n",
-		    DCpath[3] ? DCpath[3] : "none");
-# endif	/* defined(HASPERSDC) */
-	    }
-	    (void) fprintf(stderr, "%s: device cache file write paths:\n", Pn);
-	    if ((dx = dcpath(2, 0)) < 0)
-		(void) fprintf(stderr, "\tnone\n");
-	    else {
-		(void) fprintf(stderr, "\tNamed via -D: %s\n",
-		    DCstate == 2 ? "none"
-				 : DCpath[0] ? DCpath[0] : "none");
-
-# if	defined(HASENVDC)
-		(void) fprintf(stderr,
-		    "\tNamed in environment variable %s: %s\n",
-		    HASENVDC, DCpath[1] ? DCpath[1] : "none");
-# endif	/* defined(HASENVDC) */
-
-# if	defined(HASPERSDC)
-		(void) fprintf(stderr,
-		    "\tPersonal path format (HASPERSDC): \"%s\"\n",
-		    HASPERSDC);
-#  if	defined(HASPERSDCPATH)
-		(void) fprintf(stderr,
-		    "\tModified personal path environment variable: %s\n",
-		    HASPERSDCPATH);
-		cp = getenv(HASPERSDCPATH);
-		(void) fprintf(stderr, "\t%s value: %s\n",
-			HASPERSDCPATH, cp ? cp : "none");
-#  endif	/* defined(HASPERSDCPATH) */
-		 (void) fprintf(stderr, "\tPersonal path: %s\n",
-		    DCpath[3] ? DCpath[3] : "none");
-# endif	/* defined(HASPERSDC) */
-	    }
-	}
-#endif	/* defined(HASDCACHE) */
-
-	if (version) {
-
-	/*
-	 * Display version information in reponse to ``-v''.
-	 */
-		(void) fprintf(stderr, "%s version information:\n", Pn);
-		(void) fprintf(stderr,
-			"\trevision: %s -- find the latest revision at:\n\t",
-			LSOF_VERSION);
-		(void) fprintf(stderr, "\t%s\n", LSOF_URL);
-		if ((cp = isnullstr(LSOF_CCDATE)))
-			(void) fprintf(stderr, "\tconstructed: %s\n", cp);
-		cp = isnullstr(LSOF_HOST);
-		if (!(cp1 = isnullstr(LSOF_LOGNAME)))
-			cp1 = isnullstr(LSOF_USER);
-		if (cp || cp1) {
-			if (cp && cp1)
-				cp2 = "by and on";
-			else if (cp)
-				cp2 = "on";
-			else
-				cp2 = "by";
-			(void) fprintf(stderr, "\tconstructed %s: %s%s%s\n",
-				cp2,
-				cp1 ? cp1 : "",
-				cp1 ? "@" : "",
-				cp  ? cp  : ""
-			);
-		}
-		if ((cp = isnullstr(LSOF_CC)))
-			(void) fprintf(stderr, "\tcompiler: %s\n", cp);
-		if ((cp = isnullstr(LSOF_CCV)))
-			(void) fprintf(stderr, "\tcompiler version: %s\n", cp);
-		if ((cp = isnullstr(LSOF_CCFLAGS)))
-			(void) fprintf(stderr, "\tcompiler flags: %s\n", cp);
-		if ((cp = isnullstr(LSOF_LDFLAGS)))
-			(void) fprintf(stderr, "\tloader flags: %s\n", cp);
-		if ((cp = isnullstr(LSOF_SYSINFO)))
-			(void) fprintf(stderr, "\tsystem info: %s\n", cp);
-	}
-	Exit(xv);
-}
