@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.4 97/08/15 08:16:36 abe Exp $
+ * $Id: dlsof.h,v 1.7 98/02/13 12:42:49 abe Exp $
  */
 
 
@@ -84,6 +84,9 @@
 #include <netinet/tcp_var.h>
 #include <sys/vattr.h>
 #include <sys/statfs.h>
+
+#include <rpc/rpc.h>
+#include <rpc/pmap_prot.h>
 #include <sys/vfs.h>
 #include <sys/vmount.h>
 
@@ -208,6 +211,13 @@ typedef struct afs_lock afs_rwlock_t;
 #define QSORT_P		void
 #define	READLEN_T	unsigned
 #define STRNCPY_L	size_t
+
+# if	AIXV>=4200
+#define	SZOFFTYPE	unsigned long long
+				/* size and offset type definition */
+#define	SZOFFPSPEC	"ll"	/* SZOFFTYPE print specification modifier */
+# endif	/* AIXV>=4200 */
+
 #define U_SIZE		sizeof(struct user)
 
 
@@ -299,13 +309,17 @@ extern struct l_vfs *Lvfs;
  */
 
 struct mounts {
-        dev_t dev;              	/* st_dev */
-        char *dir;              	/* directory */
-        char *fsname;           	/* file system */
-	ino_t inode;			/* st_ino */
-	u_short mode;			/* st_mode */
+        char *dir;              	/* directory (mounted on) */
+	char *fsname;           	/* file system
+					 * (symbolic links unresolved) */
+	char *fsnmres;           	/* file system
+					 * (symbolic links resolved) */
+        dev_t dev;              	/* directory st_dev */
+	dev_t rdev;			/* directory st_rdev */
+	ino_t inode;			/* directory st_ino */
+	u_short mode;			/* directory st_mode */
+	u_short fs_mode;		/* file system st_mode */
         struct mounts *next;    	/* forward link */
-	dev_t rdev;			/* st_rdev */
 };
 extern struct mounts *Mtab;
 
