@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dmnt.c,v 1.1 97/02/24 07:30:50 abe Exp $";
+static char *rcsid = "$Id: dmnt.c,v 1.2 98/02/13 12:46:19 abe Exp $";
 #endif
 
 
@@ -47,145 +47,188 @@ int
 readmnt()
 {
 	int err = 0;
-	char *dir, *fs, *h, *ty;
+	char *dir, *fs, *h, *ln, *ty;
+	char *dn = (char *)NULL;
 	int fsl;
 	struct mounts *mtp;
 	int nm;
 	struct stat sb;
 	unsigned sz;
-	struct vmount *v, *vt;
+	struct vmount *v;
+	struct vmount *vt = (struct vmount *)NULL;
 /*
  * Read the table of vmount structures.
  */
 	for (sz = sizeof(struct vmount);;) {
-		if ((vt = (struct vmount *)malloc(sz)) == NULL) {
-			(void) fprintf(stderr,
-				"%s: no space for vmount table\n", Pn);
-			return(0);
-		}
-		nm = mntctl(MCTL_QUERY, sz, vt);
-		if (nm > 0) {
-			if (vt->vmt_revision != VMT_REVISION) {
-				(void) fprintf(stderr,
-					"%s: stale file system, rev %d != %d\n",
-					Pn, vt->vmt_revision, VMT_REVISION);
-				return(0);
-			}
-			break;
-		}
-		if (nm == 0) {
-			sz = (unsigned)vt->vmt_revision;
-			(void) free((FREE_P *)vt);
-		} else {
-			(void) fprintf(stderr, "%s: mntctl error: %s\n",
-				Pn, strerror(errno));
-			return(0);
+	    if (!(vt = (struct vmount *)malloc(sz))) {
+		(void) fprintf(stderr, "%s: no space for vmount table\n", Pn);
+		return(0);
+	    }
+	    nm = mntctl(MCTL_QUERY, sz, vt);
+	    if (nm > 0) {
+		if (vt->vmt_revision != VMT_REVISION) {
+		    (void) fprintf(stderr,
+			"%s: stale file system, rev %d != %d\n",
+			Pn, vt->vmt_revision, VMT_REVISION);
+		    return(0);
 		}
+		break;
+	    }
+	    if (nm == 0) {
+		sz = (unsigned)vt->vmt_revision;
+		(void) free((FREE_P *)vt);
+	    } else {
+		(void) fprintf(stderr, "%s: mntctl error: %s\n",
+		    Pn, strerror(errno));
+		return(0);
+	    }
 	}
 /*
  * Scan the vmount structures and build Mtab.
  */
 	for (v = vt; nm--; v = (struct vmount *)((char *)v + v->vmt_length)) {
-		dir = (char *)vmt2dataptr(v, VMT_STUB);
-		fs = (char *)vmt2dataptr(v, VMT_OBJECT);
-		h = (char *)vmt2dataptr(v, VMT_HOST);
-		fsl = strlen(fs);
-		if (v->vmt_flags & MNT_REMOTE)
-			fsl += strlen(h) + 1;
-                if (statsafely(dir, &sb)) {
-		    err = 2;
-		    if (!Fwarn) {
+	    dir = (char *)vmt2dataptr(v, VMT_STUB);
+	    fs = (char *)vmt2dataptr(v, VMT_OBJECT);
+	    h = (char *)vmt2dataptr(v, VMT_HOST);
+	    fsl = strlen(fs);
+	    if (v->vmt_flags & MNT_REMOTE)
+		fsl += strlen(h) + 1;
+            if (statsafely(dir, &sb)) {
+		err = 2;
+		if (!Fwarn) {
 
-		    /*
-		     * Issue stat() failure warning.
-		     */
-			switch(v->vmt_gfstype) {
-
-#if	defined(HAS_AFS)
-			case MNT_AFS:
-				ty = "afs";
-				break;
-#endif	/* defined(HAS_AFS) */
-
-			case MNT_AIX:
-				ty = "oaix";
-				break;
-			case MNT_NFS:
-				ty = "nfs";
-				break;
-			case MNT_JFS:
-				ty = "jfs";
-				break;
-			case MNT_CDROM:
-				ty = "cdrom";
-				break;
-			default:
-				ty = "unknown";
-			}
-			(void) fprintf(stderr,
-			    "%s: WARNING: can't stat() %s file system %s\n",
-			    Pn, ty, dir);
-			(void) fprintf(stderr,
-			    "      Output information may be incomplete.\n");
-		    }
 		/*
-		 * Assemble alternate device number and mode flags.
+		 * Issue stat() failure warning.
 		 */
-		    (void) bzero((char *)&sb, sizeof(sb));
-		    if (v->vmt_flags & MNT_REMOTE)
-			sb.st_dev = (dev_t)(SDEV_REMOTE | v->vmt_vfsnumber);
-		    else {
+		    switch(v->vmt_gfstype) {
 
 #if	defined(HAS_AFS)
-			if (v->vmt_gfstype == MNT_AFS)
-			    sb.st_dev = AFSDEV;
-			else
+		    case MNT_AFS:
+			ty = "afs";
+			break;
 #endif	/* defined(HAS_AFS) */
 
-			    sb.st_dev = (dev_t)v->vmt_fsid.val[0];
+		    case MNT_AIX:
+			ty = "oaix";
+			break;
+		    case MNT_NFS:
+			ty = "nfs";
+			break;
+		    case MNT_JFS:
+			ty = "jfs";
+			break;
+		    case MNT_CDROM:
+			ty = "cdrom";
+			break;
+		    default:
+			ty = "unknown";
 		    }
-		    if (!Fwarn)
-			(void) fprintf(stderr,
-			    "      assuming \"dev=%#lx\" from mount table\n",
-			    sb.st_dev);
-		    sb.st_mode = S_IFDIR | 0777;
-		}
-		if ((mtp = (struct mounts *)malloc(sizeof(struct mounts)))
-		== NULL) {
-			err = 1;
-			break;
-		}
-		if ((mtp->dir = (char *)malloc(strlen(dir) + 1)) == NULL) {
-			err = 1;
-			break;
-		}
-		(void) strcpy(mtp->dir, dir);
-		if ((mtp->fsname = (char *)malloc(fsl + 1)) == NULL) {
-			err = 1;
-			break;
+		    (void) fprintf(stderr,
+			"%s: WARNING: can't stat() %s file system %s\n",
+			Pn, ty, dir);
+		    (void) fprintf(stderr,
+			"      Output information may be incomplete.\n");
 		}
+	    /*
+	     * Assemble alternate device number and mode flags.
+	     */
+		(void) bzero((char *)&sb, sizeof(sb));
 		if (v->vmt_flags & MNT_REMOTE)
-			(void) sprintf(mtp->fsname, "%s:%s", h, fs);
-		else
-			(void) strcpy(mtp->fsname, fs);
-		mtp->dev = sb.st_dev;
-		mtp->inode = sb.st_ino;
-		mtp->mode = sb.st_mode;
-		mtp->rdev = sb.st_rdev;
-		mtp->next = Mtab;
-		Mtab = mtp;
+		    sb.st_dev = (dev_t)(SDEV_REMOTE | v->vmt_vfsnumber);
+		else {
+
+#if	defined(HAS_AFS)
+		    if (v->vmt_gfstype == MNT_AFS)
+			sb.st_dev = AFSDEV;
+		    else
+#endif	/* defined(HAS_AFS) */
+
+			sb.st_dev = (dev_t)v->vmt_fsid.val[0];
+		}
+		if (!Fwarn)
+		    (void) fprintf(stderr,
+			"      assuming \"dev=%#lx\" from mount table\n",
+			sb.st_dev);
+		sb.st_mode = S_IFDIR | 0777;
+	    }
+	/*
+	 * Allocate space for the directory (mounted on) and resolve
+	 * any symbolic links.
+	 */
+	    if (dn)
+		(void) free((FREE_P *)dn);
+	    if (!(dn = (char *)malloc(strlen(dir) + 1))) {
+		err = 1;
+		break;
+	    }
+	    (void) strcpy(dn, dir);
+	    if (!(ln = Readlink(dn))) {
+		if (!Fwarn) {
+		    (void) fprintf(stderr,
+			"      Output information may be incomplete.\n");
+		}
+		err = 2;
+		continue;
+	    }
+	    if (ln != dn) {
+		(void) free((FREE_P *)dn);
+		dn = ln;
+	    }
+	/*
+	 * Allocate a local mounts structure and fill the directory information.
+	 */
+	    if (!(mtp = (struct mounts *)malloc(sizeof(struct mounts)))) {
+		err = 1;
+		break;
+	    }
+	    mtp->dir = dn;
+	    dn = (char *)NULL;
+	    mtp->dev = sb.st_dev;
+	    mtp->inode = sb.st_ino;
+	    mtp->mode = sb.st_mode;
+	    mtp->rdev = sb.st_rdev;
+	    mtp->next = Mtab;
+	/*
+	 * Form the file system (mounted-on) device name.  Resolve any
+	 * symbolic links.  Allocate space for the result and store it in
+	 * the local mounts structure.
+	 */
+	    if (!(dn = (char *)malloc(fsl + 1))) {
+		err = 1;
+		break;
+	    }
+	    if (v->vmt_flags & MNT_REMOTE)
+		(void) sprintf(dn, "%s:%s", h, fs);
+	    else
+		(void) strcpy(dn, fs);
+	    mtp->fsname = dn;
+	    ln = Readlink(dn);
+	    dn = (char *)NULL;
+	/*
+	 * Stat the file system (mounted-on) device name to get its modes.
+	 * Set the modes to zero if the stat fails.  Add file system
+	 * (mounted-on) device information to the local mountsstructure.
+	 */
+	    if (!ln || statsafely(ln, &sb))
+		sb.st_mode = 0;
+	    mtp->fsnmres = ln;
+	    mtp->fs_mode = sb.st_mode;
+	    Mtab = mtp;
         }
 /*
- * Handle errors.
+ * Clean up, handle errors, and return or Exit.
  */
-	(void) free((FREE_P *)vt);
+	if (dn)
+	    (void) free((FREE_P *)dn);
+	if (vt)
+	    (void) free((FREE_P *)vt);
 	switch(err) {
 	case 1:
-		(void) fprintf(stderr, "%s: no space for mount at %s (%s)\n",
-			Pn, fs, dir);
-		Exit(1);
+	    (void) fprintf(stderr, "%s: no space for mount at %s (%s)\n",
+		Pn, fs, dir);
+	    Exit(1);
 	case 2:
-		return(1);
+	    return(1);
 	}
 	return(1);
 }
