@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.3 97/09/23 08:20:25 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.4 97/10/23 12:10:25 abe Exp $";
 #endif
 
 
@@ -110,11 +110,11 @@ process_socket(sa)
  */
 	if (Fsize) {
 		if (Lf->access == 'r')
-			Lf->sz = (unsigned long)s.so_rcv.sb_cc;
+			Lf->sz = (SZOFFTYPE)s.so_rcv.sb_cc;
 		else if (Lf->access == 'w')
-			Lf->sz = (unsigned long)s.so_snd.sb_cc;
+			Lf->sz = (SZOFFTYPE)s.so_snd.sb_cc;
 		else
-			Lf->sz = (unsigned long)s.so_rcv.sb_cc + s.so_snd.sb_cc;
+			Lf->sz = (SZOFFTYPE)(s.so_rcv.sb_cc + s.so_snd.sb_cc);
 		Lf->sz_def = 1;
 	} else
 		Lf->off_def = 1;
