@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.4 97/08/15 08:19:18 abe Exp $
+ * $Id: dlsof.h,v 1.6 98/02/13 12:43:21 abe Exp $
  */
 
 
@@ -77,6 +77,10 @@
 #include <sys/vnode.h>
 #include <net/raw_cb.h>
 #include <sys/domain.h>
+#define	pmap	RPC_pmap
+#include <rpc/rpc.h>
+#include <rpc/pmap_prot.h>
+#undef	pmap
 
 # if	FREEBSDV<200
 #include <ufs/quota.h>
@@ -235,13 +239,17 @@ struct l_vfs {
 extern struct l_vfs *Lvfs;
 
 struct mounts {
-        dev_t dev;              	/* st_dev */
-        char *dir;              	/* directory */
-        char *fsname;           	/* file system */
-	ino_t inode;			/* st_ino */
-	mode_t mode;			/* st_mode */
+        char *dir;              	/* directory (mounted on) */
+	char *fsname;           	/* file system
+					 * (symbolic links unresolved) */
+	char *fsnmres;           	/* file system
+					 * (symbolic links resolved) */
+        dev_t dev;              	/* directory st_dev */
+	dev_t rdev;			/* directory st_rdev */
+	ino_t inode;			/* directory st_ino */
+	mode_t mode;			/* directory st_mode */
+	mode_t fs_mode;			/* file system st_mode */
         struct mounts *next;    	/* forward link */
-	dev_t rdev;			/* st_rdev */
 };
 
 #define	X_NCACHE	"ncache"
