@@ -32,17 +32,12 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.3 97/04/29 10:02:10 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.5 98/02/16 19:48:42 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
-#if	defined(HASPROCFS)
-#define PROCFSNAME_TMP(x)	#x
-#define	PROCFSNAME_STR(x)	PROCFSNAME_TMP(x)
-#define	PROCFSNAME		PROCFSNAME_STR(HASPROCFS)
-#endif	/* defined(HASPROCFS) */
 
 #if	defined(HASFDESCFS)
 _PROTOTYPE(static int lkup_dev_tty,(dev_t *dr, unsigned long *ir));
@@ -71,8 +66,8 @@ getmemsz(pid)
 			if (p->P_VMSPACE == NULL
 			||  kread((KA_T)p->P_VMSPACE, (char *)&vm, sizeof(vm)))
 				return;
-			Lf->sz = (unsigned long)ctob(vm.vm_tsize + vm.vm_dsize
-						     + vm.vm_ssize);
+			Lf->sz = (SZOFFTYPE)ctob(vm.vm_tsize + vm.vm_dsize
+							     + vm.vm_ssize);
 			Lf->sz_def = 1;
 			return;
 		}
@@ -454,7 +449,7 @@ process_node(va)
 			break;
 		case N_NFS:
 			if (n != NULL) {
-				Lf->sz = (unsigned long)n->n_vattr.va_size;
+				Lf->sz = (SZOFFTYPE)n->n_vattr.va_size;
 				Lf->sz_def = 1;
 			}
 			break;
@@ -468,7 +463,7 @@ process_node(va)
 				break;
 			if (pgsz < 0)
 				pgsz = getpagesize();
-			Lf->sz = (unsigned long)((pgsz * vm.vm_tsize)
+			Lf->sz = (SZOFFTYPE)((pgsz * vm.vm_tsize)
 			       +         (pgsz * vm.vm_dsize)
 			       +         (pgsz * vm.vm_ssize));
 			Lf->sz_def = 1;
@@ -478,20 +473,18 @@ process_node(va)
 				switch(p->pfs_type) {
 				case Proot:
 				case Pproc:
-					Lf->sz = (unsigned long)DEV_BSIZE;
+					Lf->sz = (SZOFFTYPE)DEV_BSIZE;
 					Lf->sz_def = 1;
 					break;
 				case Pmem:
 					(void) getmemsz(p->pfs_pid);
 					break;
 				case Pregs:
-					Lf->sz = (unsigned long)
-						 sizeof(struct reg);
+					Lf->sz = (SZOFFTYPE)sizeof(struct reg);
 					Lf->sz_def = 1;
 					break;
 				case Pfpregs:
-					Lf->sz = (unsigned long)
-						 sizeof(struct fpreg);
+					Lf->sz = (SZOFFTYPE)sizeof(struct fpreg);
 					Lf->sz_def = 1;
 					break;
 				}
@@ -502,16 +495,16 @@ process_node(va)
 		case N_REGLR:
 			if (type == VREG || type == VDIR) {
 				if (i != NULL) {
-					Lf->sz = (unsigned long)i->i_size;
+					Lf->sz = (SZOFFTYPE)i->i_size;
 					Lf->sz_def = 1;
 				} else if (m != NULL) {
-					Lf->sz = (unsigned long)m->mfs_size;
+					Lf->sz = (SZOFFTYPE)m->mfs_size;
 					Lf->sz_def = 1;
 				}
 
 #if	defined(HAS9660FS)
 				else if (iso_stat) {
-					Lf->sz = iso_sz;
+					Lf->sz = (SZOFFTYPE)iso_sz;
 					Lf->sz_def = 1;
 				}
 #endif	/* defined(HAS9660FS) */
@@ -620,14 +613,14 @@ process_node(va)
 
 # if	FREEBSDV<200
 		if (type == VDIR)
-			(void) sprintf(Namech, "/%s", PROCFSNAME);
+			(void) sprintf(Namech, "/%s", HASPROCFS);
 		else
-			(void) sprintf(Namech, "/%s/%0*d", PROCFSNAME,
+			(void) sprintf(Namech, "/%s/%0*d", HASPROCFS,
 				PNSIZ, p->pfs_pid);
 		enter_nm(Namech);
 # else	/* FREEBSDV>=200 */
 		ty = NULL;
-		(void) sprintf(Namech, "/%s", PROCFSNAME);
+		(void) sprintf(Namech, "/%s", HASPROCFS);
 		switch (p->pfs_type) {
 		case Proot:
 			ty = "PDIR";
@@ -668,6 +661,18 @@ process_node(va)
 			(void) sprintf(endnm(), "/%d/notepg", p->pfs_pid);
 			ty = "PGRP";
 			break;
+
+#  if	FREEBSDV>=300
+		case Pmap:
+			(void) sprintf(endnm(), "/%d/map", p->pfs_pid);
+			ty = "PMAP";
+			break;
+		case Ptype:
+			(void) sprintf(endnm(), "/%d/etype", p->pfs_pid);
+			ty = "PETY";
+			break;
+#  endif	/* FREEBSDV>=300 */
+
 		}
 		if (ty)
 			(void) strcpy(Lf->type, ty);
@@ -698,23 +703,30 @@ process_node(va)
 
 #if	defined(HASPROCFS)
 	if (Ntype == N_PROC) {
-		if (Procsrch)
-			Lf->sf |= SELNM;
-		else {
-			for (pfi = Procfsid; pfi; pfi = pfi->next) {
+	    if (Procsrch) {
+		Procfind = 1;
+		Lf->sf |= SELNM;
+	    } else {
+		for (pfi = Procfsid; pfi; pfi = pfi->next) {
+		    if ((pfi->pid && pfi->pid == p->pfs_pid)
 
 # if	defined(HASPINODEN)
-				if (pfi->inode == p->pfs_fileno)
+		    ||  (Lf->inp_ty == 1 && Lf->inode == pfi->inode)
 # else	/* !defined(HASPINODEN) */
 				if (pfi->pid == p->pfs_pid)
 # endif	/* defined(HASPINODEN) */
 
-				{
-					Lf->sf |= SELNM;
-					break;
-				}
+		    ) {
+			pfi->f = 1;
+			if (!Namech[0]) {
+			    (void) strncpy(Namech, pfi->nm, MAXPATHLEN - 1);
+			    Namech[MAXPATHLEN - 1] = '\0';
 			}
+			Lf->sf |= SELNM;
+			break;
+		    }
 		}
+	    }
 	} else
 #endif	/* defined(HASPROCFS) */
 
@@ -754,7 +766,7 @@ process_pipe(pa)
 	if (Foffset)
 		Lf->off_def = 1;
 	else {
-		Lf->sz = p.pipe_buffer.size;
+		Lf->sz = (SZOFFTYPE)p.pipe_buffer.size;
 		Lf->sz_def = 1;
 	}
 	if (p.pipe_peer)
