@@ -32,12 +32,21 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.4 97/04/29 06:48:14 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.9 98/02/16 19:50:23 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
+#if	IRIXV>=50300 && IRIXV<60400
+#include <sys/scsi.h>
+#include <sys/buf.h>
+#include <sys/mtio.h>
+#define	_KERNEL	1
+#include <sys/tpsc.h>
+#undef	_KERNEL
+#endif	/* IRIXV>=50300 && IRIXV<60400 */
+
 
 #if	IRIXV>=50101
 _PROTOTYPE(static char isvlocked,(struct vnode *va));
@@ -221,16 +230,11 @@ process_node(na)
 		if (Fnfs)
 			Lf->sf |- SELNFS;
 		if (kread((KA_T)i.i_mntdev, (char *)&m, sizeof(m))) {
-			(void) sprintf(Namech,
-			    "can't read NFS mount structure (%#x)",
-			    i.i_mntdev);
-			enter_nm(Namech);
+			enter_nm("can't read NFS mount structure");
 			return;
 		}
 		if (kread((KA_T)i.i_fsptr, (char *)&r, sizeof(r))) {
-			(void) sprintf(Namech, "can't read rnode (%#x)",
-			    i.i_fsptr);
-			enter_nm(Namech);
+			enter_nm("can't read rnode");
 			return;
 		}
 	} else if (strcasecmp(Fsinfo[ity].fs_name, "SOC") == 0) {
@@ -269,9 +273,7 @@ process_node(na)
  */
 	if (i.i_filocks) {
 		if ((kread((KA_T)i.i_filocks, (char *)&fl, sizeof(fl)))) {
-			(void) sprintf(Namech, "can't read filock (%#x)",
-				i.i_filocks);
-			enter_nm(Namech);
+			enter_nm("can't read filock");
 			return;
 		} else {
 			if (fl.set.l_type == F_RDLCK)
@@ -310,9 +312,8 @@ process_node(na)
 			||  readstqinit(sh.q_qinfo, &qi)
 			||  readstmin(qi.qi_minfo, &mi)
 			||  readstidnm(mi.mi_idname, Namech, STRNML-1)) {
-				(void) sprintf(Namech,
-					"no module name for stream at %#x",
-					i.i_sptr);
+				(void) strcpy(Namech,
+					"no module name for stream");
 			} else {
 				Namech[STRNML-1] = '\0';
 				Lf->is_stream = 1;
@@ -374,12 +375,12 @@ process_node(na)
 				Lf->off_def = 1;
 			break;
 		case N_NFS:
-			Lf->sz = (unsigned long)r.r_snfsattr.na_size;
+			Lf->sz = (SZOFFTYPE)r.r_snfsattr.na_size;
 			Lf->sz_def = 1;
 			break;
 		case N_REGLR:
 			if (type == IFREG || type == IFDIR)
-				Lf->sz = (unsigned long)i.i_size;
+				Lf->sz = (SZOFFTYPE)i.i_size;
 				Lf->sz_def = 1;
 			break;
 		}
@@ -505,8 +506,10 @@ process_node(na)
 	struct snode s;
 	struct stdata sd;
 	struct queue sh;
+	int st = 0;
 	char *ty;
 	enum vtype type;
+	unsigned long ul;
 	struct l_vfs *vfs;
 	int xs = 0;
 	int xt = 0;
@@ -617,15 +620,13 @@ process_node(na)
 				return;
 			}
 		} else {
-			(void) sprintf(Namech, "unknown file system index: %s",
+			(void) sprintf(Namech, "unknown file system index: %d",
 				fx);
 			enter_nm(Namech);
 			return;
 		}
 	} else {
-		(void) sprintf(Namech,
-			"unusable virtual file system pointer: %#x", v.v_vfsp);
-		enter_nm(Namech);
+		enter_nm("unusable virtual file system pointer");
 		return;
 	}
 	if (Selinet)
@@ -649,8 +650,7 @@ process_node(na)
 		vfs = (struct l_vfs *)NULL;
 	else {
 	    if (!(vfs = readvfs(v.v_vfsp, &kv))) {
-		(void) sprintf(Namech, "bad vfs for %#x at %#x", na, v.v_vfsp);
-		enter_nm(Namech);
+		enter_nm("unusable vfs struct");
 		return;
 	    }
 	}
@@ -662,9 +662,7 @@ process_node(na)
 # if	defined(HASCACHEFS)
 	case N_CACHE:
 	    if (!vd || kread((KA_T)vd, (char *)&c, sizeof(c))) {
-		(void) sprintf(Namech,
-		    "vnode at %#x: can't read cnode at: %#x", na, vd);
-		enter_nm(Namech);
+		enter_nm("can't read cnode for vnode");
 		return;
 	    }
 
@@ -687,17 +685,12 @@ process_node(na)
 	 * Non-pipe fifonodes are linked to a vnode thorough fn_realvp.
 	 */
 	    if (!vd || readfifonode(vd, &f)) {
-		(void) sprintf(Namech,
-		    "vnode at %#x: can't read fifonode at: %#x", na, vd);
-		enter_nm(Namech);
+		enter_nm("can't read vnode's fifonode");
 		return;
 	    }
 	    if (f.fn_realvp) {
 		if (readvnode((caddr_t)f.fn_realvp, &rv)) {
-		    (void) sprintf(Namech,
-			"fifonode at %#x: can't read vnode at: %#x",
-			vd, f.fn_realvp);
-		    enter_nm(Namech);
+		    enter_nm("can't fifonode's vnode");
 		    return;
 		}
 
@@ -708,19 +701,14 @@ process_node(na)
 # endif	/* IRIXV>=60400 */
 
 		if (!rvd) {
-		    (void) sprintf(Namech,
-			"fifonode at %#x: no successor address", rvd);
-		    enter_nm(Namech);
+		    enter_nm("fifonode has no successor address");
 		    return;
 		}
 
 # if	IRIXV==50300 || IRIXV>=60200
 		if (xt == N_XFS) {
 		    if (kread((KA_T)rvd, (char *)&xfs, sizeof(xfs))) {
-			(void) sprintf(Namech,
-			    "fifonode at %#x: can't read xfs_inode at: %#x",
-			    vd, rvd);
-			enter_nm(Namech);
+			enter_nm("can't read fifonode's xfs_inode");
 			return;
 		    }
 		    xs = 1;
@@ -729,18 +717,12 @@ process_node(na)
 
 		if (xt == N_NFS) {
 		    if (readrnode((caddr_t)rvd, &r)) {
-			(void) sprintf(Namech,
-			    "vnode at %#x: can't read rnode at: %#x",
-			    na, rvd);
-			enter_nm(Namech);
+			enter_nm("can't read vnode's rnode");
 			return;
 		    }
 		} else {
 		    if (readinode((struct inode *)rvd, &i)) {
-			(void) sprintf(Namech,
-			    "fifonode at %#x: can't read inode at: %#x",
-			    vd, rvd);
-			enter_nm(Namech);
+			enter_nm("can't read fifonode's inode");
 			return;
 		    }
 		    is = 1;
@@ -749,17 +731,13 @@ process_node(na)
 	    break;
 	case N_NFS:
 	    if (!vd || readrnode((caddr_t)vd, &r)) {
-		(void) sprintf(Namech,
-		    "vnode at %#x: can't read rnode at: %#x", na, vd);
-		enter_nm(Namech);
+		enter_nm("can't read vnode's rnode");
 		return;
 	    }
 	    break;
 	case N_PIPE:
 	    if (!vd || readpipenode((caddr_t)vd, &p)) {
-		(void) sprintf(Namech,
-		    "vnode at %#x: can't read pipenode at: %#x", na, vd);
-		enter_nm(Namech);
+		enter_nm("can't read vnode's pipenode");
 		return;
 	    }
 	    break;
@@ -773,15 +751,12 @@ process_node(na)
 	 */
 	    if (!vd
 	    ||  kread((KA_T)vd, (char *)&prn, sizeof(prn))) {
-		(void) sprintf(Namech,
-		    "vnode at %#x: can't read prnode at: %#x", na, vd);
-		enter_nm(Namech);
+		enter_nm("can't read vnode's prnode");
 		return;
 	    }
 	    if (!prn.pr_proc) {
 		if (v.v_type == VDIR) {
-		    (void) sprintf(Namech, "/%s%s", HASPROCFS,
-			(prn.pr_type == PR_PROCDIR) ? "" : HASPINFO);
+		    (void) sprintf(Namech, "/%s", HASPROCFS);
 		    enter_nm(Namech);
 		    Lf->inode = (unsigned long)((prn.pr_type == PR_PROCDIR)
 			      ? PR_ROOTINO
@@ -789,38 +764,30 @@ process_node(na)
 		    Lf->inp_ty = 1;
 		    proc.p_pid = (pid_t)Lf->inode;
 		} else {
-		    (void) sprintf(Namech, "/%s%s/???", HASPROCFS,
-			(prn.pr_type == PR_PROCDIR) ? "" : HASPINFO);
+		    (void) sprintf(Namech, "/%s/???", HASPROCFS);
 		    enter_nm(Namech);
 		    Lf->inp_ty = 0;
 		}
 		break;
 	    }
 	    if (kread((KA_T)prn.pr_proc, (char *)&proc, sizeof(proc))) {
-		(void) sprintf(Namech,
-		    "prnode at %#x: can't read proc (%#x)",
-		    vd, prn.pr_proc);
-		enter_nm(Namech);
+		enter_nm("can't read prnode's proc struct");
 		return;
 	    }
 	    if (pgsz < 0L)
 		pgsz = sysconf(_SC_PAGESIZE);
-	    Lf->sz = (unsigned long)(pgsz * proc.p_size);
+	    Lf->sz = (SZOFFTYPE)(pgsz * proc.p_size);
 	    Lf->sz_def = 1;
-	    (void) sprintf(Namech, "/%s%s/%0*d", HASPROCFS,
-		(prn.pr_type == PR_PROCDIR) ? "" : HASPINFO,
-		PNSIZ, proc.p_pid);
+	    (void) sprintf(Namech, "/%s/%d", HASPROCFS, proc.p_pid);
 	    Lf->inode = (unsigned long)ptoi(proc.p_pid);
 	    Lf->inp_ty = 1;
 	    enter_nm(Namech);
 	    break;
-# endif	/* HASPROCFS */
+# endif	/* defined(HASPROCFS) */
 
 	case N_STREAM:
 	    if (!vd || readsnode((caddr_t)vd, &s)) {
-		(void) sprintf(Namech,
-		    "vnode at %#x: can't read snode at: %#x", na, vd);
-		enter_nm(Namech);
+		enter_nm("can't read vnode's snode");
 		return;
 	    }
 	    if (v.v_stream) {
@@ -868,18 +835,12 @@ process_node(na)
 	 */
 	    if (v.v_type == VCHR) {
 		if (!vd || readsnode(vd, &s)) {
-		    (void) sprintf(Namech,
-			"vnode at %#x: can't read snode at: %#x",
-			na, vd);
-		    enter_nm(Namech);
+		    enter_nm("can't read vnode's snode");
 		    return;
 		}
 		if (s.s_realvp) {
 		    if (readvnode((caddr_t)s.s_realvp, &rv)) {
-			(void) sprintf(Namech,
-			    "snode at %#x: can't read real vnode (%#x)",
-			    vd, s.s_realvp);
-			enter_nm(Namech);
+			enter_nm("can't read snode's real vnode");
 			return;
 		    }
 
@@ -890,19 +851,14 @@ process_node(na)
 # endif	/* IRIXV>=60400 */
 
 		    if (!rvd) {
-			(void) sprintf(Namech,
-			    "snode at %#x: no successor address", rvd);
-			enter_nm(Namech);
+			enter_nm("snode has no successor address");
 			return;
 		    }
 
 # if	IRIXV==50300 || IRIXV>=60200
 		    if (xt == N_XFS) {
 			if (kread((KA_T)rvd, (char *)&xfs, sizeof(xfs))) {
-			    (void) sprintf(Namech,
-				"snode at %#x: can't read xfs_inode at: %#x",
-				vd, rvd);
-			    enter_nm(Namech);
+			    enter_nm("can't read snode's xfs_inode");
 			    return;
 		    	}
 			xs = 1;
@@ -911,10 +867,7 @@ process_node(na)
 
 		    {
 			if (readinode((struct inode *)rvd, &i)) {
-			    (void) sprintf(Namech,
-				"snode at %#x: can't read inode at: %#x",
-				vd, rvd);
-			    enter_nm(Namech);
+			    enter_nm("can't read snode's inode");
 			    return;
 			}
 			is = 1;
@@ -936,10 +889,7 @@ process_node(na)
 
 # if	IRIXV<60400
 		    if (readvnode((caddr_t)s.s_commonvp, &rv)) {
-			  (void) sprintf(Namech,
-				"snode at %#x: can't read real vnode at: %#x",
-				vd, s.s_commonvp);
-			    enter_nm(Namech);
+			    enter_nm("can't read snode's common vnode");
 			    return;
 		    }
 		    rvd = (caddr_t)rv.v_data;
@@ -948,36 +898,21 @@ process_node(na)
 # endif	/* IRIXV<60400 */
 	
 		    if (!rvd || readsnode(rvd, &rs)) {
-			(void) sprintf(Namech,
-			    "vnode at %#x: can't read snode at: %#x",
-
-# if	IRIXV<60400
-			    s.s_commonvp,
-# else	/* IRIXV>=60400 */
-			    vd,
-# endif	/* IRIXV<60400 */
-
-			    rvd);
-			enter_nm(Namech);
+			enter_nm("can't read vnode's snode");
 			return;
 		    }
 		}
 		break;
 	    } else if (v.v_type != VBLK) {
 		if (!vd) {
-		    (void) sprintf(Namech,
-			"vnode at %#x: no successor address", na);
-		    enter_nm(Namech);
+		    enter_nm("vnode has no successor address");
 		    return;
 		}
 
 # if	IRIXV==50300 || IRIXV>=60200
 		if (xt == N_XFS) {
 		    if (kread((KA_T)vd, (char *)&xfs, sizeof(xfs))) {
-			(void) sprintf(Namech,
-			    "vnode at %#x: can't read xfs_inode at: %#x",
-			    na, vd);
-			enter_nm(Namech);
+			enter_nm("can't read vnode's xfs_inode");
 			return;
 		    }
 		    xs = 1;
@@ -986,10 +921,7 @@ process_node(na)
 
 	 	{
 		    if (readinode((struct inode *)vd, &i)) {
-			(void) sprintf(Namech,
-			    "vnode at %#x: can't read inode at: %#x",
-			    na, vd);
-			enter_nm(Namech);
+			enter_nm("can't read vnode's inode");
 			return;
 		    }
 		    is = 1;
@@ -1008,7 +940,7 @@ process_node(na)
 
 # if	defined(HASPROCFS)
 	case N_PROC:
-# endif
+# endif	/* defined(HASPROFS) */
 
 	case N_FIFO:
 		if (xt != N_NFS) {
@@ -1041,6 +973,32 @@ process_node(na)
 	}
 	if (vfs && !vfs->dir)
 		(void) completevfs(vfs, &dev);
+
+#if	IRIXV>=50300 && IRIXV<60400
+/*
+ * Re-map an internal IRIX >= 5.3 and < 6.4 SCSI tape driver device
+ * number to an external one.
+ */
+	if (type == VCHR
+	&&  major(dev) >= TPSC_MAJOR && major(dev) < TPSC_STAT_MAJOR)
+	{
+	    dev_t xd;
+
+	    st = 1;
+	    xd = dev & ~L_MAXMIN;
+	    xd |= ((dev & SCSIID_MSK) << TPSC_UNIT_SHIFT);
+	    xd |= (TPSCTLRID_DEV(dev) << TPSC_CTLR_SHIFT);
+	    if (dev & REWIND_MSK)
+		xd |= 1;
+	    if (!(dev & SWAP_MSK))
+		xd |= 2;
+	    if (dev & VAR_MSK)
+		xd |= 4;
+	    xd |= (((dev & DENSITY_MSK) >> 3) << DENSITY_SHIFT);
+	    dev = xd;
+	}
+#endif	/* IRIXV>=50300 && IRIXV<60400 */
+
 /*
  * Obtain the inode number.
  */
@@ -1085,18 +1043,23 @@ process_node(na)
 	 * prnode is read.
 	 */
 		break;
-# endif
+# endif	/* defined(HASPROCFS) */
 
 	case N_FIFO:
 		if ( ! f.fn_realvp) {
 			Lf->inode = (unsigned long)f.fn_ino;
 			Lf->inp_ty = 1;
-			(void) sprintf(dev_ch, "0x%08x", vd);
+			ul = (unsigned long)vd;
+			ul &= 0xffffffff;
+			(void) sprintf(dev_ch, "0x%08lx", ul);
 			enter_dev_ch(dev_ch);
 			if (f.fn_flag & ISPIPE)
 				(void) strcpy(Namech, "PIPE");
-			if (f.fn_mate)
-				(void) sprintf(endnm(), "->0x%08x", f.fn_mate);
+			if (f.fn_mate) {
+				ul = (unsigned long)f.fn_mate;
+				ul &= 0xffffffff;
+				(void) sprintf(endnm(), "->0x%08lx", ul);
+			}
 			break;
 		} else {
 			if (xt == N_NFS) {
@@ -1140,7 +1103,7 @@ process_node(na)
 
 # if	defined(HASCACHEFS)
 		case N_CACHE:
-			Lf->sz = (unsigned long)c.c_size;
+			Lf->sz = (SZOFFTYPE)c.c_size;
 			Lf->sz_def = 1;
 			break;
 # endif	/* defined(HASCACHEFS) */
@@ -1150,15 +1113,15 @@ process_node(na)
 
 #if	IRIXV>=60200
 			    if (nfs3s)
-				Lf->sz = (unsigned long)r.r_attr.va_size;
+				Lf->sz = (SZOFFTYPE)r.r_attr.va_size;
 			    else
 #endif	/* IRIXV>=60200 */
 
-				Lf->sz = (unsigned long)r.r_nfsattr.na_size;
+				Lf->sz = (SZOFFTYPE)r.r_nfsattr.na_size;
 			    Lf->sz_def = 1;
 			} else if (is) {
 			    if (is) {
-				Lf->sz = (unsigned long)i.i_size;
+				Lf->sz = (SZOFFTYPE)i.i_size;
 				Lf->sz_def = 1;
 			    }
 			} else
@@ -1172,15 +1135,15 @@ process_node(na)
 
 #if	IRIXV>=60200
 			if (nfs3s)
-			    Lf->sz = (unsigned long)r.r_attr.va_size;
+			    Lf->sz = (SZOFFTYPE)r.r_attr.va_size;
 			else
 #endif	/* IRIXV>=60200 */
 
-			    Lf->sz = (unsigned long)r.r_nfsattr.na_size;
+			    Lf->sz = (SZOFFTYPE)r.r_nfsattr.na_size;
 			Lf->sz_def = 1;
 			break;
 		case N_PIPE:
-			Lf->sz = (unsigned long)p.pi_size;
+			Lf->sz = (SZOFFTYPE)p.pi_size;
 			Lf->sz_def = 1;
 			break;
 
@@ -1197,13 +1160,13 @@ process_node(na)
 		case N_REGLR:
 			if (type == VREG || type == VDIR) {
 				if (is) {
-					Lf->sz = (unsigned long)i.i_size;
+					Lf->sz = (SZOFFTYPE)i.i_size;
 					Lf->sz_def = 1;
 				}
 
 # if	IRIXV==50300 || IRIXV>=60200
 				else if (xs) {
-					Lf->sz = xfs.i_d.di_size;
+					Lf->sz = (SZOFFTYPE)xfs.i_d.di_size;
 					Lf->sz_def = 1;
 				}
 # endif	/* IRIXV==50300 || IRIXV>=060200 */
@@ -1319,7 +1282,7 @@ process_node(na)
 
 #if	defined(HASBLKDEV)
 /*
- * If this is a VCHR file and it's missing an inode number, try to
+ * If this is a VBLK file and it's missing an inode number, try to
  * supply one.
  */
 	if ((!Lf->inp_ty || !Lf->inode) && type == VBLK && Lf->dev_def)
@@ -1327,10 +1290,10 @@ process_node(na)
 #endif	/* defined(HASBLKDEV) */
 
 /*
- * If this is a VCHR file and it's missing an inode number, try to
- * supply one.
+ * If this is a VCHR file, but not an IRIX >= 5.3 and < 6.4 SCSI tape device
+ * file, and it's missing an inode number, try to supply one.
  */
-	if ((!Lf->inp_ty || !Lf->inode) && type == VCHR && Lf->dev_def)
+	if (!st && (!Lf->inp_ty || !Lf->inode) && type == VCHR && Lf->dev_def)
 		find_ch_ino();
 /*
  * Indicate a stream device file.
@@ -1345,52 +1308,49 @@ process_node(na)
 
 # if	defined(HASPROCFS)
 	if (Ntype == N_PROC) {
-	    if (Procsrch)
+	    if (Procsrch) {
+		Procfind = 1;
 	    	Lf->sf |= SELNM;
-	    else {
+	    } else {
 		for (pfi = Procfsid; pfi; pfi = pfi->next) {
 
-#  if	defined(HASPINFO)
-		
 		/*
-		 * Check for a match on any HASPINFO file.
+		 * Check for a match on PID -- and optionally on inode number.
 		 */
-		    if (pfi->type == 2) {
-			if (prn.pr_type == PR_PSINFO) {
-			    Lf->sf |= SELNM;
-			    break;
-			} else
-			    continue;
-		    }
-#  endif	/* HASPINFO */
+		    if ((pfi->pid && pfi->pid == proc.p_pid)
 
-		/*
-		 * Check for a match on PID.
-		 */
-		    if (pfi->pid == proc.p_pid) {
-
-#  if	defined(HASPINFO)
-
-		    /*
-		     * Check for a match on type.
-		     */
-			if ((prn.pr_type == PR_PROCDIR && pfi->type != 0)
-			||  (prn.pr_type == PR_PSINFO  && pfi->type != 1))
-			    continue;
-#  endif
+# if	defined(HASPINODEN)
+		    ||  (Lf->inp_ty == 1 && Lf->inode == pfi->inode)
+# endif	/* defined(HASPINODEN) */
 
+		    ) {
+			pfi->f = 1;
+			if (!Namech[0] && pfi->nm) {
+			    (void) strncpy(Namech, pfi->nm, MAXPATHLEN - 1);
+			    Namech[MAXPATHLEN-1] = '\0';
+			}
 			Lf->sf |= SELNM;
 			break;
 		    }
 		}
 	    }
 	} else
-# endif	/* HASPROCFS */
+# endif	/* defined(HASPROCFS) */
 
 	{
 		if (Sfile && is_file_named((char *)NULL, type))
 			 Lf->sf |= SELNM;
 	}
+
+#if	IRIXV>=50300 && IRIXV<60400
+/*
+ * If there's no name for an IRIX >= 5.3 and < 6.4 SCSI tape file, look
+ * for and enter one of the possibly multiple device name aliases.
+ */
+	if (st && !Namech[0] && (dp = lkupdev(&dev, 0, 0)))
+		(void) strcpy(Namech, dp->name);
+#endif	/* IRIXV>=50300 && IRIXV<60400 */
+
 /*
  * Enter name characters.
  */
