@@ -32,14 +32,14 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.1 97/02/24 07:45:28 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.3 98/02/13 12:49:42 abe Exp $";
 #endif
 
 #include "lsof.h"
 
 
 _PROTOTYPE(static void get_kernel_access,(void));
-_PROTOTYPE(static void process_text,(struct pregion *prp));
+_PROTOTYPE(static void process_text,(struct pregion *prp, struct pregion *sprp));
 _PROTOTYPE(static void readfsinfo,(void));
 
 #if	IRIXV==50300 || !defined(IRIX32BITS) || IRIXV>=60200
@@ -95,7 +95,7 @@ gather_proc_info()
 {
 	struct vnode *cdir, *rdir;
 	char *comm;
-	int i, j;
+	int i, j, nf, sgs;
 	struct proc *p;
 	KA_T pa;
 	struct proc ps;
@@ -105,7 +105,7 @@ gather_proc_info()
 
 #if	IRIXV<50101
 	static struct file **fp;
-	int nf, ul;
+	int ul;
 	static char *uaf = NULL;
 	static struct user *u;
 	static MALLOC_S uafl;
@@ -118,18 +118,17 @@ gather_proc_info()
 	struct file *f;
 	struct cred pcred;
 # if	IRIXV>=50300
+	struct shaddr_s sg;
 	struct ufchunk uc;
 # endif	/* IRIXV>=50300 */
 # if	IRIXV<60400
 	static struct user *u;
 	struct user us;
+# else	/* IRIXV>=60400 */
+	struct fdt fdt, *pf;
 # endif	/* IRIXV<60400 */
 #endif	/* IRIXV<50101 */
 
-#if	IRIXV>=60400
-	struct fdt fdt;
-#endif	/* IRIXV>=60400 */
-
 #if	IRIXV<50101
 /*
  * Allocate space for user area and file pointers for IRIX version
@@ -199,6 +198,7 @@ gather_proc_info()
 #endif	/* IRIXV<60400 */
 
 	{
+		sgs = 0;
 
 #if	IRIXV<60400
 		pa = Kp + (KA_T)(px * Kpst);
@@ -250,6 +250,15 @@ gather_proc_info()
 # endif	/* IRIXV>=50101 */
 #endif	/* IRIXV<60400 */
 
+#if	IRIXV==50300 || IRIXV>=60100
+	/*
+	 * Read the IRIX 5.3, and IRIX 6.1 and above shared group descriptor.
+	 */
+		if (p->p_shaddr
+		&&  kread((KA_T)p->p_shaddr, (char *)&sg, sizeof(sg)) == 0)
+			sgs = 1;
+#endif	/* IRIXV==50300 || IRIXV>=60100 */
+
 #if	IRIXV>=50101 && IRIXV<60200
 	/*
 	 * Read the IRIX 5.1.1 through 6.1 current working directory and
@@ -322,10 +331,12 @@ gather_proc_info()
 
 #if	IRIXV<50300 || (IRIXV>50300 && IRIXV<60100)
 		if (p->p_region)
-			process_text(p->p_region);
+			process_text(p->p_region, (struct pregion *)NULL);
 #else	/* IRIXV==50300 || IRIXV>=60100 */
-		if (PREG_FIRST(p->p_region))
-			process_text(PREG_FIRST(p->p_region));
+		if (PREG_FIRST(p->p_region) || (sgs && PREG_FIRST(sg.s_region)))
+			process_text(PREG_FIRST(p->p_region),
+				     sgs ? PREG_FIRST(sg.s_region)
+					 : (struct pregion *)NULL);
 #endif	/* IRIXV<50300 || (IRIXV>50300 && IRIXV<60100) */
 
 	/*
@@ -379,10 +390,12 @@ gather_proc_info()
 	 * file pointers.  The first ufchunk structure is addressed via the
 	 * u_flist pointer in the user structure.
 	 */
-		for (uc.uf_next = u->u_flist, i = 0, j = NFPCHUNK;
-		     i < u->u_nofiles;
-		     i++)
-		{
+		uc.uf_next = u->u_flist;
+		if ((nf = u->u_nofiles) == 0 && sgs) {
+		    nf = sg.s_nofiles;
+		    uc.uf_next = sg.s_flist;
+		}
+		for (i = 0, j = NFPCHUNK; i < nf; i++) {
 		    if (++j > NFPCHUNK) {
 			if( ! uc.uf_next)
 			    break;
@@ -408,8 +421,9 @@ gather_proc_info()
 	 * NFPCHUNK file pointers.  The first ufchunk struct is in the fdt
 	 * struct
 	 */
-	    if (!p->p_fdt
-	    ||  kread((KA_T)p->p_fdt, (char *)&fdt, sizeof(fdt)))
+	    if (!(pf = p->p_fdt) && sgs)
+		pf = sg.s_fdt;
+	    if (!pf || kread((KA_T)pf, (char *)&fdt, sizeof(fdt)))
 		continue;
 	    for (i = j = 0; i < fdt.fd_nofiles; i++, j++) {
 		if (j >= NFPCHUNK) {
@@ -557,7 +571,7 @@ get_kernel_access()
 		    " output may not be correct.\n");
 		(void) fprintf(stderr,
 		    "      kernel size = %d, sizeof(struct proc) = %d\n",
-		    Kpst, PROCSIZE);
+		    Kpst, (int)PROCSIZE);
 		if (Kpst < (PROCSIZE - (PROCSIZE >> 3))
 		||  Kpst > (PROCSIZE + (PROCSIZE >> 3)))
 		    Kpst = PROCSIZE;
@@ -580,8 +594,7 @@ get_kernel_access()
  * Read var structure.
  */
 	if (kread(kvar, (char *)&Var, sizeof(Var))) {
-	    (void) fprintf(stderr, "%s: can't read var struct at %#x\n",
-		Pn, kvar);
+	    (void) fprintf(stderr, "%s: can't read var struct\n", Pn);
 	    Exit(1);
 	}
 }
@@ -629,46 +642,52 @@ kread(addr, buf, len)
  */
 
 static void
-process_text(prp)
+process_text(prp, sprp)
 	preg_t *prp;		/* process region pointer */
+	preg_t *sprp;		/* shared process region pointer */
 {
-	int i, j, lm;
+	int i, j, lm, rx;
 	struct pregion p;
+	preg_t *pp;
 	struct region r;
 	caddr_t na;
-
 /*
- * Follow the virtual address space pregion structure chain.
+ * Loop through both process regions.
  */
+	for (rx = 0; rx < 2; rx++) {
+	    pp = rx ? sprp : prp;
+	/*
+	 * Follow the virtual address space pregion structure chain.
+	 */
+
 #if	IRIXV<50300 || (IRIXV>50300 && IRIXV<60100)
-	for (i = lm = 0; prp; prp = p.p_forw, lm++)
+	    for (i = lm = 0; pp; pp = p.p_forw, lm++)
 #else	/* IRIXV==50300 || IRIXV>=60100 */
 	for (i = lm = 0;
-	     prp; 
-	     prp = (preg_t *)p.p_avlnode.avl_nextino, lm++)
+	     pp; 
+	     pp = (preg_t *)p.p_avlnode.avl_nextino, lm++)
 #endif	/* IRIXV<50300 || (IRIXV>50300 && IRIXV<60100) */
 
-	{
+	    {
 
-	/*
-	 * Avoid infinite loop.
-	 */
+	    /*
+	     * Avoid infinite loop.
+	     */
 		if (lm > 1000) {
-			if (!Fwarn)
-				(void) fprintf(stderr,
-					"%s: too many regions, PID %d\n",
-					Pn, Lp->pid);
-			return;
+		    if (!Fwarn)
+			(void) fprintf(stderr, "%s: too many regions, PID %d\n",
+			    Pn, Lp->pid);
+		    return;
 		}
-	/*
-	 * Read the pregion; then read the region.
-	 */
-		if (kread((KA_T)prp, (char *)&p, sizeof(p))
+	    /*
+	     * Read the pregion; then read the region.
+	     */
+		if (kread((KA_T)pp, (char *)&p, sizeof(p))
 		||  kread((KA_T)p.p_reg, (char *)&r, sizeof(r)))
-			return;
-	/*
-	 * Skip file entries with no file pointers and duplicate entries.
-	 */
+		    return;
+	    /*
+	     * Skip file entries with no file pointers and duplicate entries.
+	     */
 
 #if	IRIXV<50101
 		if ((na = (caddr_t)r.r_iptr) == NULL)
@@ -676,67 +695,65 @@ process_text(prp)
 		if ((na = (caddr_t)r.r_vnode) == NULL)
 #endif	/* IRIXV<50101 */
 
-			continue;
+		    continue;
 		for (j = 0; j < i; j++) {
-			if (Ndc[j] == na)
-				break;
-			}
+		    if (Ndc[j] == na)
+			break;
+		}
 		if (j < i)
-			continue;
-	/*
-	 * Cache the vnode address for duplicate checking.
-	 */
-		if (Ndc == NULL) {
-			if ((Ndc = (caddr_t *)malloc((MALLOC_S)
-				  (sizeof(KA_T) * 10)))
-			== NULL) {
-				(void) fprintf(stderr,
-					"%s: no txt ptr space, PID %d\n",
-					Pn, Lp->pid);
-				Exit(1);
-			}
-			Nn = 10;
+		    continue;
+	    /*
+	     * Cache the vnode address for duplicate checking.
+	     */
+		if (!Ndc) {
+		    if (!(Ndc = (caddr_t *)malloc((MALLOC_S)(sizeof(KA_T)*32))))
+		    {
+			(void) fprintf(stderr, "%s: no txt ptr space, PID %d\n",
+			    Pn, Lp->pid);
+			Exit(1);
+		    }
+		    Nn = 32;
 		} else if (i >= Nn) {
-			Nn += 10;
-			if ((Ndc = (caddr_t *)realloc((MALLOC_P *)Ndc,
-				   (MALLOC_S)(Nn * sizeof(KA_T))))
-			== NULL) {
-				(void) fprintf(stderr,
-					"%s: no more txt ptr space, PID %d\n",
-					Pn, Lp->pid);
-				Exit(1);
-			}
+		    Nn += 32;
+		    if (!(Ndc = (caddr_t *)realloc((MALLOC_P *)Ndc,
+			       (MALLOC_S)(Nn * sizeof(KA_T)))))
+		    {
+			(void) fprintf(stderr,
+			    "%s: no more txt ptr space, PID %d\n", Pn, Lp->pid);
+			Exit(1);
+		    }
 		}
 		Ndc[i++] = na;
-	/*
-	 * Save text node information.
-	 */
+	    /*
+	     * Save text node information.
+	     */
 
 #if	IRIXV<50200
 		switch (p.p_type) {
-			case PT_MAPFILE:
-				alloc_lfile(" mem", -1);
-				break;
-			case PT_LIBTXT:
-			case PT_LIBDAT:
-				alloc_lfile(" ltx", -1);
-				break;
-			case PT_DATA:
-			case PT_TEXT:
-			default:
-				alloc_lfile(" txt", -1);
+		    case PT_MAPFILE:
+			alloc_lfile(" mem", -1);
+			break;
+		    case PT_LIBTXT:
+		    case PT_LIBDAT:
+			alloc_lfile(" ltx", -1);
+			break;
+		    case PT_DATA:
+		    case PT_TEXT:
+		    default:
+			alloc_lfile(" txt", -1);
 		}
 
 #else	/* IRIXV>=50200 */
 		if (r.r_flags & RG_TEXT)
-			alloc_lfile(" txt", -1);
+		    alloc_lfile(" txt", -1);
 		else
-			alloc_lfile(" mem", -1);
+		    alloc_lfile(" mem", -1);
 #endif	/* IRIXV<50200 */
 
 		process_node(na);
 		if (Lf->sf)
-			link_lfile();
+		    link_lfile();
+	    }
 	}
 }
 
@@ -796,8 +813,7 @@ readfsinfo()
 	if (get_Nl_value("nfstyp", Drive_Nl, &v) < 0 || !v
 	||  kread((KA_T)v, (char *)&Nfstyp, sizeof(Nfstyp)) || Nfstyp < 1) {
 		(void) fprintf(stderr,
-			"%s: nfstyp value (%d) error, kernel @ %#x\n",
-			Pn, Nfstyp, v);
+			"%s: nfstyp value (%d) error\n", Pn, Nfstyp);
 		Exit(1);
 	}
 	len = (MALLOC_S)(Nfstyp * sizeof(struct fsinfo));
@@ -806,8 +822,7 @@ readfsinfo()
 	||  (Fsinfo = (struct fsinfo *)malloc(len)) == NULL
 	||  kread((KA_T)v, (char *)Fsinfo, len)) {
 		(void) fprintf(stderr,
-			"%s: fstyp read error, length = %d, kernel @ %#x\n",
-			Pn, len, v);
+			"%s: fstyp read error, length = %d\n", Pn, len);
 		Exit(1);
 	}
 /*
@@ -822,8 +837,7 @@ readfsinfo()
 		}
 		if (kread((KA_T)Fsinfo[i].fs_name, cp, FSNAMEL)) {
 		    (void) fprintf(stderr,
-			"%s: can't read file system name[i] from %#x\n",
-			Pn, i, Fsinfo[i].fs_name);
+			"%s: can't read file system name[%d]\n", Pn, i);
 		    Exit(1);
 		}
 		cp[FSNAMEL - 1] = '\0';
@@ -860,7 +874,7 @@ read_proc()
 	    if (get_Nl_value("phash", Drive_Nl, &v) < 0 || !v
 	    ||  kread((KA_T)v, (char *)&pm, sizeof(pm))) {
 		(void) fprintf(stderr,
-		    "%s: can't get proc hash table size: %x\n", Pn, v);
+		    "%s: can't get proc hash table size\n", Pn);
 		Exit(1);
 	    }
 	    pm++;
@@ -868,7 +882,7 @@ read_proc()
 	    if ((ph = (struct proc **)malloc(l)) == (struct proc **)NULL) {
 		(void) fprintf(stderr,
 		    "%s: can't allocate %d bytes for proc hash table\n",
-		    Pn, l);
+		    Pn, (int)l);
 		Exit(1);
 	    }
 	}
@@ -893,8 +907,7 @@ read_proc()
  * Read the proc hash table.
  */
 	if (kread(Kp, (char *)ph, (pm * sizeof(struct proc *)))) {
-	    (void) fprintf(stderr, "%s: can't read proc hash table: %#x\n",
-		Pn, Kp);
+	    (void) fprintf(stderr, "%s: can't read proc hash table\n", Pn);
 	    Exit(1);
 	}
 /*
