@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.6 97/07/09 15:52:57 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.9 98/02/16 19:51:59 abe Exp $";
 #endif
 
 
@@ -44,10 +44,6 @@ _PROTOTYPE(static int lkup_dev_tty,(dev_t *dr, unsigned long *ir));
 #endif	/* defined(HASFDESCFS) */
 
 #if	defined(HASPROCFS)
-#define PROCFSNAME_TMP(x)	#x
-#define	PROCFSNAME_STR(x)	PROCFSNAME_TMP(x)
-#define	PROCFSNAME		PROCFSNAME_STR(HASPROCFS)
-
 _PROTOTYPE(static void getmemsz,(pid_t pid));
 
 # if	!defined(PGSHIFT)
@@ -72,8 +68,8 @@ getmemsz(pid)
 			if (p->P_VMSPACE == NULL
 			||  kread((KA_T)p->P_VMSPACE, (char *)&vm, sizeof(vm)))
 				return;
-			Lf->sz = (unsigned long)ctob(vm.vm_tsize + vm.vm_dsize
-			       +		     vm.vm_ssize);
+			Lf->sz = (SZOFFTYPE)ctob(vm.vm_tsize + vm.vm_dsize
+							     + vm.vm_ssize);
 			Lf->sz_def = 1;
 			return;
 		}
@@ -176,6 +172,13 @@ process_node(va)
 	char ktnm[MAXPATHLEN+1];
 #endif	/* defined(HASKERNFS) */
 
+#if	defined(HASNFSVATTRP)
+	struct vattr nv;
+#define	NVATTR	nv
+#else	/* !defined(HASNFSVATTRP) */
+#define	NVATTR	n.n_vattr
+#endif	/* defined(HASNFSVATTRP) */
+
 #if	defined(HASPROCFS)
 	struct pfsnode p;
 	struct procfsid *pfi;
@@ -241,7 +244,9 @@ process_node(va)
 
 #if	defined(HAS9660FS)
 	case VT_ISOFS:
-		if (read_iso_node(v, &iso_dev, &iso_ino, &iso_sz)) {
+		if (read_iso_node(v, &iso_dev, (unsigned long *)&iso_ino,
+				  (SZOFFTYPE *)&iso_sz))
+		{
 			(void) sprintf(Namech, "can't read iso_node at: %x",
 			    v->v_data);
 			enter_nm(Namech);
@@ -323,6 +328,17 @@ process_node(va)
 			enter_nm(Namech);
 			return;
 		}
+
+#if	defined(HASNFSVATTRP)
+		if (!n.n_vattr
+		||  kread((KA_T)n.n_vattr, (char *)&nv, sizeof(nv))) {
+			(void) sprintf(Namech, "can't read n_vattr at: %x",
+				n.n_vattr);
+			enter_nm(Namech);
+			return;
+		}
+#endif	/* defined(HASNFSVATTRP) */
+
 		nty = NFSNODE;
 		break;
 
@@ -487,7 +503,7 @@ process_node(va)
 #endif	/* defined(HAS9660FS) */
 
 	case NFSNODE:
-		dev = n.n_vattr.va_fsid;
+		dev = NVATTR.va_fsid;
 		devs = 1;
 	}
 /*
@@ -525,7 +541,7 @@ process_node(va)
 #endif	/* defined(HAS9660FS) */
 
 	case NFSNODE:
-		Lf->inode = (unsigned long)n.n_vattr.va_fileid;
+		Lf->inode = (unsigned long)NVATTR.va_fileid;
 		Lf->inp_ty = 1;
 		break;
 
@@ -549,7 +565,7 @@ process_node(va)
 #if	defined(HAS9660FS)
 		case N_CDFS:
 			if (iso_stat) {
-				Lf->sz = iso_sz;
+				Lf->sz = (SZOFFTYPE)iso_sz;
 				Lf->sz_def = 1;
 			}
 			break;
@@ -563,7 +579,7 @@ process_node(va)
 #if	defined(HASKERNFS)
 		case N_KERN:
 			if (ksbs) {
-				Lf->sz = (unsigned long)ksb.st_size;
+				Lf->sz = (SZOFFTYPE)ksb.st_size;
 				Lf->sz_def = 1;
 			}
 			break;
@@ -571,7 +587,7 @@ process_node(va)
 
 		case N_NFS:
 			if (nty == NFSNODE) {
-				Lf->sz = (unsigned long)n.n_vattr.va_size;
+				Lf->sz = (SZOFFTYPE)NVATTR.va_size;
 				Lf->sz_def = 1;
 			}
 			break;
@@ -582,28 +598,24 @@ process_node(va)
 				switch (p.pfs_type) {
 				case Proot:
 				case Pproc:
-					Lf->sz = (unsigned long)DEV_BSIZE;
+					Lf->sz = (SZOFFTYPE)DEV_BSIZE;
 					Lf->sz_def = 1;
 					break;
 				case Pcurproc:
-					Lf->sz = (unsigned long)
-						 sprintf(dev_ch, "%ld",
-						(long)p.pfs_pid);
+					Lf->sz = (SZOFFTYPE)DEV_BSIZE;
 					Lf->sz_def = 1;
 					break;
 				case Pmem:
 					(void) getmemsz(p.pfs_pid);
 					break;
 				case Pregs:
-					Lf->sz = (unsigned long)
-						  sizeof(struct reg);
+					Lf->sz = (SZOFFTYPE)sizeof(struct reg);
 					Lf->sz_def = 1;
 					break;
 
 # if	defined(FP_QSIZE)
 				case Pfpregs:
-					Lf->sz = (unsigned long)
-						 sizeof(struct fpreg);
+					Lf->sz = (SZOFFTYPE)sizeof(struct fpreg);
 					Lf->sz_def = 1;
 					break;
 # endif	/* defined(FP_QSIZE) */
@@ -618,14 +630,14 @@ process_node(va)
 				if (nty == INODE) {
 
 #if	defined(HASI_FFS)
-					Lf->sz = (unsigned long)i.i_ffs_size;
+					Lf->sz = (SZOFFTYPE)i.i_ffs_size;
 #else	/* !defined(HASI_FFS) */
-					Lf->sz = (unsigned long)i.i_size;
+					Lf->sz = (SZOFFTYPE)i.i_size;
 #endif	/* defined(HASI_FFS) */
 
 					Lf->sz_def = 1;
 				} else if (nty == MFSNODE) {
-					Lf->sz = (unsigned long)m.mfs_size;
+					Lf->sz = (SZOFFTYPE)m.mfs_size;
 					Lf->sz_def = 1;
 				}
 
@@ -633,9 +645,9 @@ process_node(va)
 				else if (nty == EXT2NODE) {
 
 # if	defined(HASI_E2FS)
-					Lf->sz = (unsigned long)i.i_e2fs_size;
+					Lf->sz = (SZOFFTYPE)i.i_e2fs_size;
 # else	/* !defined(HASI_E2FS) */
-					Lf->sz = (unsigned long)i.i_size;
+					Lf->sz = (SZOFFTYPE)i.i_size;
 # endif	/* defined(HASI_E2FS) */
 
 					Lf->sz_def = 1;
@@ -745,7 +757,7 @@ process_node(va)
 	else if (nty == PFSNODE) {
 		Lf->dev_def = 0;
 		ty = NULL;
-		(void) sprintf(Namech, "/%s", PROCFSNAME);
+		(void) sprintf(Namech, "/%s", HASPROCFS);
 		switch (p.pfs_type) {
 		case Proot:
 			ty = "PDIR";
@@ -818,23 +830,28 @@ process_node(va)
 
 #if	defined(HASPROCFS)
 	if (Ntype == N_PROC) {
-		if (Procsrch)
-			Lf->sf |= SELNM;
-		else if (nty == PFSNODE) {
-			for (pfi = Procfsid; pfi; pfi = pfi->next) {
-
+	    if (Procsrch) {
+		Procfind = 1;
+		Lf->sf |= SELNM;
+	    } else if (nty == PFSNODE) {
+		for (pfi = Procfsid; pfi; pfi = pfi->next) {
+		    if ((pfi->pid && pfi->pid == p.pfs_pid)
+		    
 # if	defined(HASPINODEN)
-				if (pfi->inode == p.pfs_fileno)
-# else	/* !defined(HASPINODEN) */
-				if (pfi->pid == p.pfs_pid)
+		    ||  (Lf->inp_ty == 1 && pfi->inode == Lf->inode)
 # endif	/* defined(HASPINODEN) */
 
-				{
-					Lf->sf |= SELNM;
-					break;
-				}
+		    ) {
+			pfi->f = 1;
+			if (Namech[0] && pfi->nm) {
+			    (void) strncpy(Namech, pfi->nm, MAXPATHLEN - 1);
+			    Namech[MAXPATHLEN - 1] = '\0';
 			}
+			Lf->sf |= SELNM;
+			break;
+		    }
 		}
+	    }
 	} else
 #endif	/* defined(HASPROCFS) */
 
@@ -881,7 +898,7 @@ process_pipe(pa)
 	if (Foffset)
 		Lf->off_def = 1;
 	else {
-		Lf->sz = p.pipe_buffer.size;
+		Lf->sz = (SZOFFTYPE)p.pipe_buffer.size;
 		Lf->sz_def = 1;
 	}
 	if (p.pipe_peer)
