@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.7 97/09/23 09:50:29 abe Exp $
+ * $Id: machine.h,v 1.11 98/02/16 19:53:48 abe Exp $
  */
 
 
@@ -244,6 +244,14 @@
 /* #define	HASPIPENODE	1 */
 
 
+/*
+ * HASPMAPENABLED is defined when the reporting of portmapper registration
+ * info is enabled by default.
+ */
+
+/* #define	HASPMAPENABLED	1 */
+
+
 /*
  * HASPPID is defined for those dialects that support identification of
  * the parent process IDentifier (PPID) of a process.
@@ -252,6 +260,20 @@
 #define	HASPPID		1
 
 
+/*
+ * HASPRINTDEV, HASPRINTINO, HASPRINTNM, HASPRINTOFF, and HASPRINTSZ
+ * define private dialect-specific functions for printing DEVice numbers,
+ * INOde numbers, NaMes, file OFFsets, and file SiZes.  The functions are
+ * called from print_file().
+ */
+
+#define	HASPRINTDEV	print_dev
+/* #define	HASPRINTINO	print_ino	*/
+#define	HASPRINTNM	print_nm
+/* #define	HASPRINTOFF	print_off	*/
+/* #define	HASPRINTSZ	print_sz	*/
+
+
 /*
  * HASPRIVFILETYPE and PRIVFILETYPE are defined for dialects that have a
  * file structure type that isn't defined by a DTYPE_* symbol.  They are
@@ -266,6 +288,18 @@
 /* #define	PRIVFILETYPE	??	*/
 
 
+/*
+ * HASPRIVNMCACHE is defined for dialects that have a private method for
+ * printing cached NAME column values for some files.  HASPRIVNAMECACHE
+ * is defined to be the name of the function.
+ *
+ * The function takes one argument, a struct lfile pointer to the file, and
+ * returns non-zero if it prints a name to stdout.
+ */
+
+/* #define	HASPRIVNMCACHE	<function name>	*/
+
+
 /*
  * HASPROCFS is defined for those dialects that have a proc file system --
  * usually /proc and usually in SYSV4 derivatives.
@@ -279,14 +313,14 @@
  * definition and the providing of an fstype element in the local mounts
  * structure (defined in dlsof.h).
  *
- * The HASPROCFS definition may be accompanied by the HASPINFO definition.
- * HASPINFO names the subdirectory of HASPROCFS that contains read-only
- * information (typically /pinfo).
+ * The HASPROCFS definition may be accompanied by the HASPINODEN definition.
+ * HASPINODEN specifies that searching for files in HASPROCFS is to be done
+ * by inode number.
  */
 
-/* #define		HASPROCFS	"proc" */
-/* #define		HASFSTYPE	1 */
-/* #define		HASPINFO	"/pinfo" */
+/* #define	HASPROCFS	"proc" */
+/* #define	HASFSTYPE	1 */
+/* #define	HASPINODEN	1 */
 
 
 /*
@@ -381,14 +415,12 @@
  * header files.
  */
 
-#define	USE_LIB_CK_FILE_ARG			1	/* ckfa.c */
 #define	USE_LIB_COMPLETEVFS			1	/* cvfs.c */
 #define	USE_LIB_FIND_CH_INO			1	/* fino.c */
 #define	USE_LIB_IS_FILE_NAMED			1	/* isfn.c */
 #define	USE_LIB_LKUPDEV				1	/* lkud.c */
 /* #define	USE_LIB_PRINTCHDEVNAME		1	   pdvn.c */
 /* #define	USE_LIB_PROCESS_FILE		1	   prfp.c */
-/* #define	USE_LIB_PRINT_FILE		1	   prtf.c */
 /* #define	USE_LIB_PRINT_TCPTPI		1	   ptti.c */
 #define	USE_LIB_READDEV				1	/* rdev.c */
 #define	USE_LIB_READMNT				1	/* rmnt.c */
