@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1996 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.6 97/06/27 07:58:46 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.9 98/02/16 19:54:17 abe Exp $";
 #endif
 
 
@@ -275,12 +275,13 @@ process_node(na)
 	struct procfsid *pfi;
 	struct pid pids;
 	struct prnode pr;
-#endif	/* HASPROCFS */
+#endif	/* defined(HASPROCFS) */
 
 #if	defined(HASOBJFS)
 	struct objnode ob;
 #endif	/* defined(HASOBJFS) */
 
+
 /*
  * Get ttyfs_vnodeops address.
  */
@@ -325,7 +326,7 @@ process_node(na)
 #if	defined(HASPROCFS)
 		else if (strcmp(Fsinfo[fx-1], "proc") == 0)
 		    Ntype = N_PROC;
-#endif	/* HASPROCFS */
+#endif	/* defined(HASPROCFS) */
 
 #if	defined(HASOBJFS)
 		else if (strcmp(Fsinfo[fx-1], "objfs") == 0)
@@ -426,15 +427,10 @@ process_node(na)
 	    }
 	    if (pr.pr_proc == NULL) {
 		sd = 0;
-		if (v.v_type == VDIR) {
-		    (void) sprintf(Namech, "/%s", HASPROCFS);
-		    enter_nm(Namech);
+		if (v.v_type == VDIR)
 		    i.number = PR_ROOTINO;
-		} else {
-		    (void) sprintf(Namech, "/%s/???", HASPROCFS);
-		    enter_nm(Namech);
-		    i.number = -1L;
-		}
+		else
+		    i.number = 0L;
 		break;
 	    }
 	    if (kread((KA_T)pr.pr_proc, (char *)&p, sizeof(p))) {
@@ -456,11 +452,11 @@ process_node(na)
 		sd = 0;
 	    else
 		i.size = as.a_size;
-	    (void) sprintf(Namech, "/%s/%0*d", HASPROCFS, PNSIZ, pids.pid_id);
+	    (void) sprintf(Namech, "/%s/%d", HASPROCFS, pids.pid_id);
 	    i.number = (long)pids.pid_id + PR_INOBIAS;
 	    enter_nm(Namech);
 	    break;
-#endif	/* HASPROCFS */
+#endif	/* defined(HASPROCFS) */
 
 	case N_STREAM:
 	    if (v.v_stream) {
@@ -638,7 +634,7 @@ process_node(na)
 	    dev = kv.vfs_dev;
 	    ds = 1;
 	    break;
-#endif	/* HASPROCFS */
+#endif	/* defined(HASPROCFS) */
 
 	case N_STREAM:
 	    if (sqp) {
@@ -657,7 +653,7 @@ process_node(na)
 	    }
 	}
 	type = v.v_type;
-	if (lvfs && lvfs->dir == NULL)
+	if (lvfs && ds && !lvfs->dir)
 	    (void) completevfs(lvfs, &dev);
 /*
  * Obtain the inode number.
@@ -680,7 +676,7 @@ process_node(na)
 	    Lf->inode = (unsigned long)i.number;
 	    Lf->inp_ty = 1;
 	    break;
-#endif	/* HASPROCFS */
+#endif	/* defined(HASPROCFS) */
 
 	case N_FIFO:
 	    if (!f.fn_realvp) {
@@ -722,28 +718,28 @@ process_node(na)
 		    Lf->off_def = 1;
 		break;
 	    case N_NFS:
-		Lf->sz = (unsigned long)r.r_attr.va_size;
+		Lf->sz = (SZOFFTYPE)r.r_attr.va_size;
 		Lf->sz_def = sd;
 		break;
 
 #if	defined(HASOBJFS)
 	    case N_OBJF:
-		Lf->sz = (unsigned long)ob.o_size;
+		Lf->sz = (SZOFFTYPE)ob.o_size;
 		Lf->sz_def = sd;
 		break;
 #endif	/* defined(HASOBJFS) */
 
 #if	defined(HASPROCFS)
 	    case N_PROC:
-		Lf->sz = (unsigned long)i.size;
+		Lf->sz = (SZOFFTYPE)i.size;
 		Lf->sz_def = sd;
 		break;
-#endif	/* HASPROCFS */
+#endif	/* defined(HASPROCFS) */
 
 	    case N_REGLR:
 		if (type == VREG || type == VDIR) {
 		    if (!ni) {
-			Lf->sz = (unsigned long)i.size;
+			Lf->sz = (SZOFFTYPE)i.size;
 			Lf->sz_def = sd;
 		    }
 		} else if (type == VCHR && !Fsize)
@@ -824,14 +820,37 @@ process_node(na)
  * If this is a VBLK file and it's missing an inode number, try to
  * supply one.
  */
-	if (Lf->inp_ty == 0 && Lf->ntype == N_BLK && Lf->dev_def)
+	if (!Lf->inp_ty && Lf->ntype == N_BLK && Lf->dev_def)
 	    find_bl_ino();
 /*
  * If this is a VCHR file and it's missing an inode number, try to
  * supply one.
  */
-	if (Lf->inp_ty == 0 && type == VCHR && Lf->dev_def)
+	if (!Lf->inp_ty && type == VCHR && Lf->dev_def)
 	    find_ch_ino();
+
+#if	defined(RELIANTV)
+/*
+ * If this is a Reliant UNIX VCHR file, missing an inode number, see if it's
+ * an "mipc" file.
+ */
+	if (!Lf->inp_ty && Lf->dev_def && type == VCHR && !Namech[0]
+	&&  HaveMipcMaj && (int)major(Lf->dev) == MipcMaj)
+	{
+	    dev_t md;
+	    int ns;
+
+	    Lf->inode = mipc_index(minor(Lf->dev)) & MIPC_INDEX_MASK;
+	    Lf->inp_ty = 1;
+	    ns = mipc_ns(minor(Lf->dev));
+	    md = makedevice(MipcMaj, ns);
+	    if ((dp = lkupdev(&md, 0, 0)))
+		(void) sprintf(Namech, "%s (ns %d)", dp->name, ns);
+	    else
+		(void) sprintf(Namech, "MIPC (ns %d)", ns);
+	}
+#endif	/* defined(RELIANTV) */
+
 /*
  * If this is a stream with a "sockmod" module whose q_ptr leads to an
  * so_so structure, assume it's a UNIX domain socket and try to get
@@ -855,18 +874,30 @@ process_node(na)
 
 #if	defined(HASPROCFS)
 	if (Ntype == N_PROC) {
-	    if (Procsrch)
+	    if (Procsrch) {
+		Procfind = 1;
 		Lf->sf |= SELNM;
-	    else {
+	    } else {
 		for (pfi = Procfsid; pfi; pfi = pfi->next) {
-		    if (pfi->pid == pids.pid_id) {
+		    if ((pfi->pid && pfi->pid == pids.pid_id)
+
+# if	defined(HASPINODEN)
+		    ||  (Lf->inp_ty == 1 && Lf->inode == pfi->inode)
+# endif	/* defined(HASPINODEN) */
+
+		    ) {
+			pfi->f = 1;
+			if (!Namech[0]) {
+			    (void) strncpy(Namech, pfi->nm, MAXPATHLEN - 1);
+			    Namech[MAXPATHLEN - 1] = '\0';
+			}
 			Lf->sf |= SELNM;
 			break;
 		    }
 		}
 	    }
 	} else
-#endif	/* HASPROCFS */
+#endif	/* defined(HASPROCFS) */
 
 	{
 	    if (Sfile && is_file_named(NULL, type))
