@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dfile.c,v 1.1 97/02/24 07:43:19 abe Exp $";
+static char *rcsid = "$Id: dfile.c,v 1.2 97/10/23 15:54:28 abe Exp $";
 #endif
 
 
@@ -59,7 +59,7 @@ process_file(fp)
 		enter_nm(Namech);
 		return;
 	}
-	Lf->off = (unsigned long)f.f_offset;
+	Lf->off = (SZOFFTYPE)f.f_offset;
 
 	if (f.f_count) {
 
