@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.6 97/08/15 08:27:23 abe Exp $
+ * $Id: dlsof.h,v 1.10 98/02/13 12:44:24 abe Exp $
  */
 
 
@@ -65,8 +65,13 @@
 #include <rpc/types.h>
 #include <sys/protosw.h>
 #include <sys/socket.h>
+
+# if	!defined(solaris)
 #include <sys/socketvar.h>
+# endif	/* !defined(solaris) */
+
 #include <sys/un.h>
+#include <sys/wait.h>
 #include <netinet/in.h>
 
 # if	defined(solaris)
@@ -88,6 +93,7 @@
 #define	rval_t		char
 #define	strsignal	kernel_strsignal
 #include <sys/strsubr.h>
+#include <sys/socketvar.h>
 #undef	exit
 #undef	rval_t
 #undef	strsignal
@@ -104,10 +110,10 @@
 #include <netinet/in_pcb.h>
 #include <sys/stream.h>
 
-#  if	solaris<20600
+# if	defined(solaris) && solaris<20600
 #undef	MAX
 #undef	MIN
-#  endif	/* solaris<20600 */
+# endif	/* defined(solaris) && solaris<20600 */
 
 #include <sys/sysmacros.h>
 #include <sys/vfs.h>
@@ -134,12 +140,23 @@
 #include <sys/tiuser.h>
 #include <rpc/auth.h>
 #include <rpc/clnt.h>
+#include <rpc/clnt_soc.h>
+#include <rpc/pmap_prot.h>
 #define	_KERNEL
 #include <sys/fs/autofs.h>
 #include <sys/door.h>
 #undef	_KERNEL
+#  else	/* solaris<20500 */
+#include <rpc/rpc.h>
+#include <rpc/clnt_soc.h>
+#include <rpc/pmap_prot.h>
 #  endif	/* solaris>=20500 */
+#  if	!defined(_NETDB_H_)
+#include <rpc/rpcent.h>
+#  endif	/* !defined(_NETDB_H_) */
 # else	/* !solaris */
+#include <rpc/rpc.h>
+#include <rpc/pmap_prot.h>
 
 /*
  * Dummy hsnode structure for SunOS 4.1.3 and below.  This should come from
@@ -288,7 +305,7 @@ extern char *sys_errlist[];
 
 #define	CWDLEN		(MAXPATHLEN+1)
 #define DEVINCR		1024	/* device table malloc() increment */
-#define	KA_T		off_t
+#define	KA_T		unsigned long
 #define	KMEM		"/dev/kmem"
 #define MALLOC_P	char
 #define FREE_P		MALLOC_P
@@ -307,6 +324,7 @@ extern char *sys_errlist[];
 
 # else	/* !defined(solaris) */
 #define CLONEMAJ	37	/* major device number for clone device */
+#define	MEMMOVE		memcpy	/* for fill_portmap() in print.c */
 #define N_UNIX		"/vmunix"
 #define	PROCMIN		3	/* processes that make a "good" scan */
 # endif	/* defined(solaris) */
@@ -318,6 +336,13 @@ extern char *sys_errlist[];
 #define	READLEN_T	int
 #define STRNCPY_L	int
 #define	STRNML		32	/* stream name length (maximum) */
+
+# if	defined(solaris) && solaris>=20600
+#define	SZOFFTYPE	unsigned long long
+				/* size and offset internal storage type */
+#define	SZOFFPSPEC	"ll"	/* SZOFFTYPE print specification modifier */
+# endif	/* defined(solaris) && solaris>=20600 */
+
 #define U_SIZE		sizeof(struct user)
 
 
@@ -345,18 +370,15 @@ struct clone {
 };
 extern struct clone *Clone;
 
-extern int CloneMaj;
-
-# if	!defined(solaris)
+# if	defined(solaris)
+extern	major_t	CloneMaj;
+extern char **Fsinfo;
+extern int Fsinfomax;
+# else	/* !defined(solaris) */
 extern char *Cwd;
 extern struct file *Fileptr;
 #define	FILEPTR	Fileptr			/* for process_file() in dfile.c */
-# endif
-
-# if	defined(solaris)
-extern char **Fsinfo;
-extern int Fsinfomax;
-# endif
+# endif	/* defined(solaris) */
 
 extern int HaveCloneMaj;
 extern kvm_t *Kd;
@@ -376,18 +398,22 @@ struct l_vfs {
 extern struct l_vfs *Lvfs;
 
 struct mounts {
-        dev_t dev;              	/* st_dev */
-        char *dir;              	/* directory */
-        char *fsname;           	/* file system */
-	ino_t inode;			/* st_ino */
-	mode_t mode;			/* st_mode */
+        char *dir;              	/* directory (mounted on) */
+        char *fsname;           	/* file system
+					 * (symbolic links unresolved) */
+	char *fsnmres;			/* file system
+					 * (symbolic links resolved) */
+        dev_t dev;              	/* directory st_dev */
+	dev_t rdev;			/* directory st_rdev */
+	ino_t inode;			/* directory st_ino */
+	mode_t mode;			/* directory st_mode */
+	mode_t fs_mode;			/* file system st_mode */
         struct mounts *next;    	/* forward link */
 
 # if	defined(HASFSTYPE)
 	char *fstype;			/* file system type */
 # endif
 
-	dev_t rdev;			/* st_rdev */
 };
 
 # if	defined(solaris)
