@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.5 97/07/22 10:41:37 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.7 98/02/16 19:55:10 abe Exp $";
 #endif
 
 
@@ -136,9 +136,10 @@ ent_fa(a1, a2, d)
 	if (Lf->nma)
 		return;
 	if (!a1)
-		(void) sprintf(buf, "(FA:%s%#x)", d, *a2);
+		(void) sprintf(buf, "(FA:%s%#lx)", d, (unsigned long)*a2);
 	else
-		(void) sprintf(buf, " (FA:%#x%s%#x)", *a1, d, *a2);
+		(void) sprintf(buf, " (FA:%#lx%s%#lx)", (unsigned long)*a1,
+			d, (unsigned long)*a2);
 	len = strlen(buf) + 1;
 	if ((cp = (char *)malloc(len)) == NULL) {
 		(void) fprintf(stderr,
@@ -460,7 +461,6 @@ process_node(va)
 	struct snode s;
 	int so_st = 0;
 	struct tmpnode t;
-	char *tn;
 	char *ty;
 	enum vtype type;
 	struct sockaddr_un ua;
@@ -468,7 +468,8 @@ process_node(va)
 	static struct vnode *v = (struct vnode *)NULL;
 	struct l_vfs *vfs;
 	struct stdata *vs;
-	int vty, vty_tmp;
+	int vty = 0;
+	int  vty_tmp;
 
 #if	defined(solaris)
 # if	solaris>=20500
@@ -492,10 +493,7 @@ process_node(va)
 	struct cnode cn;
 	struct dev_info di;
 	char din[DINAMEL];
-	caddr_t fafr = NULL;
-	caddr_t fato = NULL;
 	struct vfs favfs;
-	struct filock fi;
 	struct lnode lo;
 	struct namenode nn;
 	int nns = 0;
@@ -624,8 +622,8 @@ process_node(va)
 
 vfs_read_error:
 
-		(void) sprintf(Namech, "vnode at %#x: can't read vfs: %#x",
-			va, v->v_vfsp);
+		(void) sprintf(Namech, "vnode at %#lx: can't read vfs: %#lx",
+			(unsigned long)va, (unsigned long)v->v_vfsp);
 		enter_nm(Namech);
 		return;
 	    }
@@ -655,7 +653,8 @@ vfs_read_error:
 		}
 		if (Ntype < 0) {
 			(void) sprintf(Namech,
-				"unknown file system type, v_op: %#x", v->v_op);
+				"unknown file system type, v_op: %#lx",
+				(unsigned long)v->v_op);
 			enter_nm(Namech);
 			return;
 		}
@@ -912,8 +911,8 @@ vfs_read_error:
 			vty = N_FIFO;
 		    } else {
 			(void) sprintf(Namech,
-			    "FIFO namenode at %#x: no fifonode pointer",
-			    v->v_data);
+			    "FIFO namenode at %#lx: no fifonode pointer",
+			    (unsigned long)v->v_data);
 			return;
 		    }
 		} else {
@@ -924,17 +923,20 @@ vfs_read_error:
 		if (!realvp) {
 			Lf->inode = (unsigned long)f.fn_ino;
 			Lf->inp_ty = 1;
-			(void) sprintf(dev_ch, "0x%08x", v->v_data);
+			(void) sprintf(dev_ch, "0x%08lx",
+				(unsigned long)v->v_data);
 			enter_dev_ch(dev_ch);
 			if (f.fn_flag & ISPIPE)
 				(void) strcpy(Namech, "PIPE");
 
 # if	solaris<20500
 			if (f.fn_mate)
-				(void) sprintf(endnm(), "->0x%08x", f.fn_mate);
+				(void) sprintf(endnm(), "->0x%08lx",
+					(unsigned long)f.fn_mate);
 # else	/* solaris>=20500 */
 			if (f.fn_dest)
-				(void) sprintf(endnm(), "->0x%08x", f.fn_dest);
+				(void) sprintf(endnm(), "->0x%08lx",
+					(unsigned long)f.fn_dest);
 # endif	/* solaris<20500 */
 			break;
 		}
@@ -967,8 +969,8 @@ vfs_read_error:
 		    if (read_nln(va, v->v_data, &lo))
 			return;
 		    if ((realvp = lo.lo_vp) == (struct vnode *)NULL) {
-			(void) sprintf(Namech, "lnode at %#x: no real vnode",
-			    v->v_data);
+			(void) sprintf(Namech, "lnode at %#lx: no real vnode",
+			    (unsigned long)v->v_data);
 			return;
 		    }
 		    if (read_nvn(v->v_data, (caddr_t)realvp, v))
@@ -1050,8 +1052,8 @@ vfs_read_error:
 				vty = N_STREAM;
 			else {
 				(void) sprintf(Namech,
-				    "unknown file system type, v_op: %#x",
-				    v->v_op);
+				    "unknown file system type, v_op: %#lx",
+				    (unsigned long)v->v_op);
 				enter_nm(Namech);
 				return;
 			}
@@ -1187,7 +1189,7 @@ vfs_read_error:
 	case N_DOOR:
 		dev_def = 0;
 		if (da) {
-			(void) sprintf(dev_ch, "0x%08x", da);
+			(void) sprintf(dev_ch, "0x%08lx", (unsigned long)da);
 			enter_dev_ch(dev_ch);
 		}
 		break;
@@ -1213,7 +1215,7 @@ vfs_read_error:
 		 *    Get a non-NULL foreign sockaddr_un and enter it in Namech;
 		 *    Check for matches on sockaddr_un.sun_path names.
 		 */
-		    (void) sprintf(dev_ch, "0x%08x", v->v_data);
+		    (void) sprintf(dev_ch, "0x%08lx", (unsigned long)v->v_data);
 		    enter_dev_ch(dev_ch);
 		    nl = 0;
 		    if ((len = read_nusa(&so.so_laddr, &ua))) {
@@ -1501,7 +1503,7 @@ vfs_read_error:
 
 #if	defined(HAS_AFS)
 		case N_AFS:
-			Lf->sz = an.size;
+			Lf->sz = (SZOFFTYPE)an.size;
 			Lf->sz_def = 1;
 			break;
 #endif	/* defined(HAS_AFS) */
@@ -1511,15 +1513,15 @@ vfs_read_error:
 		case N_AUTO:
 
 #  if	solaris<20600
-			Lf->sz = (unsigned long)au.an_size;
+			Lf->sz = (SZOFFTYPE)au.an_size;
 #  else	/* solaris >=20600 */
-			Lf->sz = (unsigned long)fnn.fn_size;
+			Lf->sz = (SZOFFTYPE)fnn.fn_size;
 #  endif	/* solaris < 20600 */
 			Lf->sz_def = 1;
 			break;
 # endif	/* solaris>=20500 */
 		case N_CACHE:
-			Lf->sz = (unsigned long)cn.c_size;
+			Lf->sz = (SZOFFTYPE)cn.c_size;
 			Lf->sz_def = 1;
 			break;
 # if	defined(solaris) && solaris>=20600
@@ -1531,13 +1533,13 @@ vfs_read_error:
 #endif	/* defined(solaris) */
 
 		case N_HSFS:
-			Lf->sz = (unsigned long)h.hs_dirent.ext_size;
+			Lf->sz = (SZOFFTYPE)h.hs_dirent.ext_size;
 			Lf->sz_def = 1;
 			break;
 
 #if	defined(solaris)
 		case N_NM:
-			Lf->sz = (unsigned long)nn.nm_vattr.va_size;
+			Lf->sz = (SZOFFTYPE)nn.nm_vattr.va_size;
 			Lf->sz_def = 1;
 			break;
 #endif	/* defined(solaris) */
@@ -1555,9 +1557,9 @@ vfs_read_error:
 			else {
 
 #if	defined(solaris)
-				Lf->sz = (unsigned long)r.r_size;
+				Lf->sz = (SZOFFTYPE)r.r_size;
 #else	/* !defined(solaris) */
-				Lf->sz = (unsigned long)r.r_attr.va_size;
+				Lf->sz = (SZOFFTYPE)r.r_attr.va_size;
 #endif	/* defined(solaris) */
 
 				Lf->sz_def = 1;
@@ -1565,7 +1567,7 @@ vfs_read_error:
 			break;
 
 		case N_PCFS:
-			Lf->sz = (unsigned long)pc.pc_size;
+			Lf->sz = (SZOFFTYPE)pc.pc_size;
 			Lf->sz_def = 1;
 			break;
 
@@ -1582,7 +1584,7 @@ vfs_read_error:
 		case N_REGLR:
 			if (type == VREG || type == VDIR) {
 				if (ins) {
-					Lf->sz = (unsigned long)i.i_size;
+					Lf->sz = (SZOFFTYPE)i.i_size;
 					Lf->sz_def = 1;
 				}
 			} else if ((type == VCHR || type == VBLK) && !Fsize)
@@ -1593,13 +1595,13 @@ vfs_read_error:
 				Lf->off_def = 1;
 			break;
 		case N_TMP:
-			Lf->sz = (unsigned long)t.tn_attr.va_size;
+			Lf->sz = (SZOFFTYPE)t.tn_attr.va_size;
 			Lf->sz_def = 1;
 			break;
 
 #if	defined(HASVXFS)
 		case N_VXFS:
-			Lf->sz = vx_sz;
+			Lf->sz = (SZOFFTYPE)vx_sz;
 			Lf->sz_def = 1;
 			break;
 #endif	/* defined(HASVXFS) */
@@ -1792,16 +1794,28 @@ vfs_read_error:
 
 #if	defined(HASPROCFS)
 	if (Ntype == N_PROC) {
-		if (Procsrch)
-			Lf->sf |= SELNM;
-		else {
-			for (pfi = Procfsid; pfi; pfi = pfi->next) {
-				if (pfi->pid == pids.pid_id) {
-					Lf->sf |= SELNM;
-					break;
-				}
+	    if (Procsrch) {
+		Procfind = 1;
+		Lf->sf |= SELNM;
+	    } else {
+		for (pfi = Procfsid; pfi; pfi = pfi->next) {
+		    if ((pfi->pid && pfi->pid == pids.pid_id)
+
+# if	defined(HASPINODEN)
+		    ||  (Lf->inp_ty == 1 && Lf->inode == pfi->inode)
+# endif	/* defined(HASPINODEN) */
+
+		    ) {
+			pfi->f = 1;
+			if (!Namech[0]) {
+			    (void) strncpy(Namech, pfi->nm, MAXPATHLEN - 1);
+			    Namech[MAXPATHLEN - 1] = '\0';
 			}
+			Lf->sf |= SELNM;
+			break;
+		    }
 		}
+	    }
 	} else
 #endif	/* defined(HASPROCFS) */
 
@@ -1840,8 +1854,8 @@ read_cni(s, rv, v, rs, di, din, dinl)
 	if (rs->s_dip) {
 	    if (kread((KA_T)rs->s_dip, (char *)di, sizeof(struct dev_info))) {
 		(void) sprintf(Namech,
-		    "common snode at %#x: no dev info: %#x",
-		    rv->v_data, rs->s_dip);
+		    "common snode at %#lx: no dev info: %#lx",
+		    (unsigned long)rv->v_data, (unsigned long)rs->s_dip);
 		enter_nm(Namech);
 		return(1);
 	    }
@@ -1864,7 +1878,8 @@ readinode(ia, i)
 	struct inode *i;		/* inode buffer */
 {
 	if (kread((KA_T)ia, (char *)i, sizeof(struct inode))) {
-		(void) sprintf(Namech, "can't read inode at %#x", ia);
+		(void) sprintf(Namech, "can't read inode at %#lx",
+			(unsigned long)ia);
 		return(1);
 	}
 	return(0);
@@ -1885,7 +1900,8 @@ read_ndn(na, da, dn)
 {
 	if (!da || kread((KA_T)da, (char *)dn, sizeof(struct door_node))) {
 		(void) sprintf(Namech,
-			"node at %#x: can't read door_node: %#x", na, da);
+			"node at %#lx: can't read door_node: %#lx",
+			(unsigned long)na, (unsigned long)da);
 		enter_nm(Namech);
 		return(1);
 	}
@@ -1923,7 +1939,8 @@ read_mi(s, dev, so, so_st)
 	if (!s)
 		return;
 	if (kread((KA_T)s, (char *)&sd, sizeof(sd))) {
-		(void) sprintf(Namech, "can't read stream head: %#x", s);
+		(void) sprintf(Namech, "can't read stream head: %#lx",
+			(unsigned long)s);
 		return;
 	}
 /*
@@ -2022,12 +2039,12 @@ read_nan(na, aa, rn)
 		(void) sprintf(Namech,
 
 # if    solaris<20600
-		    "node at %#x: can't read autonode: %#x",
+		    "node at %#lx: can't read autonode: %#lx",
 # else  /* solaris>=20600 */
-		    "node at %#x: can't read fnnode: %#x",
+		    "node at %#lx: can't read fnnode: %#lx",
 # endif /* solaris<20600 */
 
-		    na, aa);
+		    (unsigned long)na, (unsigned long)aa);
 		enter_nm(Namech);
 		return(1);
 	}
@@ -2047,7 +2064,8 @@ read_ncn(na, ca, cn)
 {
 	if (!ca || kread((KA_T)ca, (char *)cn, sizeof(struct cnode))) {
 		(void) sprintf(Namech,
-			"node at %#x: can't read cnode: %#x", na, ca);
+			"node at %#lx: can't read cnode: %#lx",
+			(unsigned long)na, (unsigned long)ca);
 		enter_nm(Namech);
 		return(1);
 	}
@@ -2068,7 +2086,8 @@ read_nfn(na, fa, f)
 {
 	if (!fa || readfifonode(fa, f)) {
 		(void) sprintf(Namech,
-			"node at %#x: can't read fifonode: %#x", na, fa);
+			"node at %#lx: can't read fifonode: %#lx",
+			(unsigned long)na, (unsigned long)fa);
 		enter_nm(Namech);
 		return(1);
 	}
@@ -2088,7 +2107,8 @@ read_nhn(na, ha, h)
 {
 	if (!ha || readhsnode(ha, h)) {
 		(void) sprintf(Namech,
-			"node at %#x: can't read hsnode: %#x", na, ha);
+			"node at %#lx: can't read hsnode: %#lx",
+			(unsigned long)na, (unsigned long)ha);
 		enter_nm(Namech);
 		return(1);
 	}
@@ -2108,7 +2128,8 @@ read_nin(na, ia, i)
 {
 	if (!ia || readinode((struct inode *)ia, i)) {
 		(void) sprintf(Namech,
-			"node at %#x: can't read inode: %#x", na, ia);
+			"node at %#lx: can't read inode: %#lx",
+			(unsigned long)na, (unsigned long)ia);
 		enter_nm(Namech);
 		return(1);
 	}
@@ -2129,7 +2150,8 @@ read_nln(na, la, ln)
 {
 	if (!la || kread((KA_T)la, (char *)ln, sizeof(struct lnode))) {
 		(void) sprintf(Namech,
-			"node at %#x: can't read lnode: %#x", na, la);
+			"node at %#lx: can't read lnode: %#lx",
+			(unsigned long)na, (unsigned long)la);
 		enter_nm(Namech);
 		return(1);
 	}
@@ -2149,7 +2171,8 @@ read_nnn(na, nna, nn)
 {
 	if (!nna || kread((KA_T)nna, (char *)nn, sizeof(struct namenode))) {
 		(void) sprintf(Namech,
-			"node at %#x: can't read namenode: %#x", na, nna);
+			"node at %#lx: can't read namenode: %#lx",
+			(unsigned long)na, (unsigned long)nna);
 		enter_nm(Namech);
 		return(1);
 	}
@@ -2171,7 +2194,8 @@ read_nmn(na, ma, m)
 	if (!ma
 	||  kread((KA_T)ma, (char *)m, sizeof(struct mvfsnode))) {
 		(void) sprintf(Namech,
-			"node at %#x: can't read mvfsnode: %#x", na, ma);
+			"node at %#lx: can't read mvfsnode: %#lx",
+			(unsigned long)na, (unsigned long)ma);
 		enter_nm(Namech);
 		return(1);
 	}
@@ -2205,9 +2229,10 @@ read_npi(na, v, pids)
 
 	if (!v->v_data || kread((KA_T)v->v_data, (char *)&pr, sizeof(pr))) {
 	    (void) sprintf(Namech,
-		"node at %#x: can't read prnode: %#x", na, v->v_data);
+		"node at %#lx: can't read prnode: %#lx",
+		(unsigned long)na, (unsigned long)v->v_data);
 	    enter_nm(Namech);
-	    return;
+	    return(1);
 	}
 
 # if	!defined(solaris) || solaris<20600
@@ -2232,24 +2257,24 @@ read_npi(na, v, pids)
 	    return(0);
 	}
 	if (kread((KA_T)pr.pr_proc, (char *)&p, sizeof(p))) {
-	    (void) sprintf(Namech, "prnode at %#x: can't read proc: %#x",
-		v->v_data, pr.pr_proc);
+	    (void) sprintf(Namech, "prnode at %#lx: can't read proc: %#lx",
+		(unsigned long)v->v_data, (unsigned long)pr.pr_proc);
 	    enter_nm(Namech);
 	    return(1);
 	}
 	if (p.p_as && kread((KA_T)p.p_as, (char *)&as, sizeof(as)) == 0) {
-	    Lf->sz = (unsigned long)as.a_size;
+	    Lf->sz = (SZOFFTYPE)as.a_size;
 	    Lf->sz_def = 1;
 	}
 	if (!p.p_pidp
 	||  kread((KA_T)p.p_pidp, (char *)pids, sizeof(struct pid))) {
 	    (void) sprintf(Namech,
-		"proc struct at %#x: can't read pid: %#x",
-		pr.pr_proc, p.p_pidp);
+		"proc struct at %#lx: can't read pid: %#lx",
+		(unsigned long)pr.pr_proc, (unsigned long)p.p_pidp);
 	    enter_nm(Namech);
 	    return(1);
 	}
-	(void) sprintf(Namech, "/%s/%0*d", HASPROCFS, PNSIZ, pids->pid_id);
+	(void) sprintf(Namech, "/%s/%d", HASPROCFS, (int)pids->pid_id);
 	Lf->inode = (unsigned long)ptoi(pids->pid_id);
 	Lf->inp_ty = 1;
 # endif	/* !defined(solaris) || solaris<20600 */
@@ -2308,163 +2333,170 @@ read_npi(na, v, pids)
 	    ty = "PDIR";
 	    break;
 	case PR_PIDDIR:
-	    (void) sprintf(Namech,  "/%s/%d", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d", HASPROCFS, (int)prpid);
 	    ty = "PDIR";
 	    break;
 	case PR_AS:
-	    (void) sprintf(Namech,  "/%s/%d/as", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/as", HASPROCFS, (int)prpid);
 	    ty = "PAS";
 	    if (prpcs
 	    &&  kread((KA_T)pc.prc_proc, (char *)&p, sizeof(p)) == 0
 	    &&  p.p_as
 	    &&  kread((KA_T)p.p_as, (char *)&as, sizeof(as)) == 0) {
-		Lf->sz = (unsigned long)as.a_size;
+		Lf->sz = (SZOFFTYPE)as.a_size;
 		Lf->sz_def = 1;
 	    }
 	    break;
 	case PR_CTL:
-	    (void) sprintf(Namech,  "/%s/%d/ctl", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/ctl", HASPROCFS, (int)prpid);
 	    ty = "PCTL";
 	    break;
 	case PR_STATUS:
-	    (void) sprintf(Namech,  "/%s/%d/status", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/status", HASPROCFS, (int)prpid);
 	    ty = "PSTA";
 	    break;
 	case PR_LSTATUS:
-	    (void) sprintf(Namech,  "/%s/%d/lstatus", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/lstatus", HASPROCFS, (int)prpid);
 	    ty = "PLST";
 	    break;
 	case PR_PSINFO:
-	    (void) sprintf(Namech,  "/%s/%d/psinfo", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/psinfo", HASPROCFS, (int)prpid);
 	    ty = "PSIN";
 	    break;
 	case PR_LPSINFO:
-	    (void) sprintf(Namech,  "/%s/%d/lpsinfo", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/lpsinfo", HASPROCFS, (int)prpid);
 	    ty = "PLPI";
 	    break;
 	case PR_MAP:
-	    (void) sprintf(Namech,  "/%s/%d/map", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/map", HASPROCFS, (int)prpid);
 	    ty = "PMAP";
 	    break;
 	case PR_RMAP:
-	    (void) sprintf(Namech,  "/%s/%d/rmap", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/rmap", HASPROCFS, (int)prpid);
 	    ty = "PRMP";
 	    break;
+	case PR_XMAP:
+	    (void) sprintf(Namech,  "/%s/%d/xmap", HASPROCFS, (int)prpid);
+	    ty = "PXMP";
+	    break;
 	case PR_CRED:
-	    (void) sprintf(Namech,  "/%s/%d/cred", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/cred", HASPROCFS, (int)prpid);
 	    ty = "PCRE";
 	    break;
 	case PR_SIGACT:
-	    (void) sprintf(Namech,  "/%s/%d/sigact", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/sigact", HASPROCFS, (int)prpid);
 	    ty = "PSGA";
 	    break;
 	case PR_AUXV:
-	    (void) sprintf(Namech,  "/%s/%d/auxv", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/auxv", HASPROCFS, (int)prpid);
 	    ty = "PAXV";
 	    break;
 
 #  if	defined(HASPR_LDT)
 	case PR_LDT:
-	    (void) sprintf(Namech,  "/%s/%d/ldt", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/ldt", HASPROCFS, (int)prpid);
 	    ty = "PLDT";
 	    break;
 #  endif	/* defined(HASPR_LDT) */
 
 	case PR_USAGE:
-	    (void) sprintf(Namech,  "/%s/%d/usage", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/usage", HASPROCFS, (int)prpid);
 	    ty = "PUSG";
 	    break;
 	case PR_LUSAGE:
-	    (void) sprintf(Namech,  "/%s/%d/lusage", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/lusage", HASPROCFS, (int)prpid);
 	    ty = "PLU";
 	    break;
 	case PR_PAGEDATA:
-	    (void) sprintf(Namech,  "/%s/%d/pagedata", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/pagedata", HASPROCFS, (int)prpid);
 	    ty = "PGD";
 	    break;
 	case PR_WATCH:
-	    (void) sprintf(Namech,  "/%s/%d/watch", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/watch", HASPROCFS, (int)prpid);
 	    ty = "PW";
 	    break;
 	case PR_CURDIR:
-	    (void) sprintf(Namech,  "/%s/%d/cwd", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/cwd", HASPROCFS, (int)prpid);
 	    ty = "PCWD";
 	    break;
 	case PR_ROOTDIR:
-	    (void) sprintf(Namech,  "/%s/%d/root", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/root", HASPROCFS, (int)prpid);
 	    ty = "PRTD";
 	    break;
 	case PR_FDDIR:
-	    (void) sprintf(Namech,  "/%s/%d/fd", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/fd", HASPROCFS, (int)prpid);
 	    ty = "PFDR";
 	    break;
 	case PR_FD:
-	    (void) sprintf(Namech,  "/%s/%d/fd/%d", HASPROCFS, prpid,
+	    (void) sprintf(Namech,  "/%s/%d/fd/%d", HASPROCFS, (int)prpid,
 		pr.pr_index);
 	    ty = "PFD";
 	    break;
 	case PR_OBJECTDIR:
-	    (void) sprintf(Namech,  "/%s/%d/object", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/object", HASPROCFS, (int)prpid);
 	    ty = "PODR";
 	    break;
 	case PR_OBJECT:
-	    (void) sprintf(Namech,  "/%s/%d/object/", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d/object/", HASPROCFS, (int)prpid);
 	    ty = "POBJ";
 	    break;
 	case PR_LWPDIR:
-	    (void) sprintf(Namech, "/%s/%d/lpw", HASPROCFS, prpid);
+	    (void) sprintf(Namech, "/%s/%d/lpw", HASPROCFS, (int)prpid);
 	    ty = "PLDR";
 	    break;
 	case PR_LWPIDDIR:
-	    (void) sprintf(Namech, "/%s/%d/lwp/%d", HASPROCFS, prpid, prtid);
+	    (void) sprintf(Namech, "/%s/%d/lwp/%d", HASPROCFS, (int)prpid,
+		(int)prtid);
 	    ty = "PLDR";
 	    break;
 	case PR_LWPCTL:
 	    (void) sprintf(Namech, "/%s/%d/lwp/%d/lwpctl", HASPROCFS,
-		prpid, prtid);
+		(int)prpid, (int)prtid);
 	    ty = "PLC";
 	    break;
 	case PR_LWPSTATUS:
 	    (void) sprintf(Namech, "/%s/%d/lwp/%d/lwpstatus", HASPROCFS,
-		prpid, prtid);
+		(int)prpid, (int)prtid);
 	    ty = "PLWS";
 	    break;
 	case PR_LWPSINFO:
 	    (void) sprintf(Namech, "/%s/%d/lwp/%d/lwpsinfo", HASPROCFS,
-		prpid, prtid);
+		(int)prpid, (int)prtid);
 	    ty = "PLWI";
 	    break;
 	case PR_LWPUSAGE:
 	    (void) sprintf(Namech,  "/%s/%d/lwp/%d/lwpusage", HASPROCFS,
-		prpid, prtid);
+		(int)prpid, (int)prtid);
 	    ty = "PLWU";
 	    break;
 	case PR_XREGS:
 	    (void) sprintf(Namech,  "/%s/%d/lwp/%d/xregs", HASPROCFS,
-		prpid, prtid);
+		(int)prpid, (int)prtid);
 	    ty = "PLWX";
 	    break;
 
 #  if	defined(HASPR_GWINDOWS)
 	case PR_GWINDOWS:
 	    (void) sprintf(Namech,  "/%s/%d/lwp/%d/gwindows", HASPROCFS,
-		prpid, prtid);
+		(int)prpid, (int)prtid);
 	    ty = "PLWG";
 	    break;
 #  endif	/* defined(HASPR_GWINDOWS) */
 
 	case PR_PIDFILE:
-	    (void) sprintf(Namech,  "/%s/%d", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d", HASPROCFS, (int)prpid);
 	    ty = "POPF";
 	    break;
 	case PR_LWPIDFILE:
-	    (void) sprintf(Namech,  "/%s/%d", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d", HASPROCFS, (int)prpid);
 	    ty = "POLP";
 	    break;
 	case PR_OPAGEDATA:
-	    (void) sprintf(Namech,  "/%s/%d", HASPROCFS, prpid);
+	    (void) sprintf(Namech,  "/%s/%d", HASPROCFS, (int)prpid);
 	    ty = "POPG";
 	    break;
+	default:
+	    ty = (char *)NULL;
 	}
 	if (!ty) {
 	    if (pr.pr_type > 9999)
@@ -2501,7 +2533,8 @@ read_npn(na, pa, p)
 {
 	if (pa == NULL || kread((KA_T)pa, (char *)p, sizeof(struct pcnode))) {
 		(void) sprintf(Namech,
-			"node at %#x: can't read pcnode: %#x", na, pa);
+			"node at %#lx: can't read pcnode: %#lx",
+			(unsigned long)na, (unsigned long)pa);
 		enter_nm(Namech);
 		return(1);
 	}
@@ -2521,7 +2554,8 @@ read_nrn(na, ra, r)
 {
 	if (!ra || readrnode(ra, r)) {
 		(void) sprintf(Namech,
-			"node at %#x: can't read rnode: %#x", na, ra);
+			"node at %#lx: can't read rnode: %#lx",
+			(unsigned long)na, (unsigned long)ra);
 		enter_nm(Namech);
 		return(1);
 	}
@@ -2543,7 +2577,8 @@ read_nson(na, sa, sn)
 {
 	if (!sa || kread((KA_T)sa, (char *)sn, sizeof(struct sonode))) {
 		(void) sprintf(Namech,
-			"node at %#x: can't read sonode: %#x", na, sa);
+			"node at %#lx: can't read sonode: %#lx",
+			(unsigned long)na, (unsigned long)sa);
 		enter_nm(Namech);
 		return(1);
 	}
@@ -2564,7 +2599,8 @@ read_nsn(na, sa, s)
 {
 	if (!sa || readsnode((caddr_t)sa, s)) {
 		(void) sprintf(Namech,
-			"node at %#x: can't read snode: %#x", na, sa);
+			"node at %#lx: can't read snode: %#lx",
+			(unsigned long)na, (unsigned long)sa);
 		enter_nm(Namech);
 		return(1);
 	}
@@ -2584,7 +2620,8 @@ read_ntn(na, ta, t)
 {
 	if (!ta || readtnode(ta, t)) {
 		(void) sprintf(Namech,
-			"node at %#x: can't read tnode: %#x", na, ta);
+			"node at %#lx: can't read tnode: %#lx",
+			(unsigned long)na, (unsigned long)ta);
 		enter_nm(Namech);
 		return(1);
 	}
@@ -2609,7 +2646,7 @@ read_nusa(so, ua)
 	ua->sun_path[0] = '\0';
 	if (!(a = (KA_T)so->soa_sa)
 	||  (len = so->soa_len) < (min + 2)
-	||  len > sizeof(struct sockaddr_un)
+	||  len > (int)sizeof(struct sockaddr_un)
 	||  kread(a, (char *)ua, len)
 	||  ua->sun_family != AF_UNIX)
 	    return(0);
@@ -2632,7 +2669,8 @@ read_nvn(na, va, v)
 {
 	if (readvnode(va, v)) {
 		(void) sprintf(Namech,
-			"node at %#x: can't read real vnode: %#x", na, va);
+			"node at %#lx: can't read real vnode: %#lx",
+			(unsigned long)na, (unsigned long)va);
 		enter_nm(Namech);
 		return(1);
 	}
