@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1996 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.3 97/06/07 20:22:30 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.5 98/02/16 19:56:03 abe Exp $";
 #endif
 
 
@@ -257,7 +257,7 @@ get_vty(v, va, kv, fx)
 #if	defined(HASPROCFS)
 		else if (strcmp(Fsinfo[fxt-1], "proc") == 0)
 		    nty = N_PROC;
-#endif	/* HASPROCFS */
+#endif	/* defined(HASPROCFS) */
 
 	    } else {
 		(void) sprintf(Namech,
@@ -364,10 +364,16 @@ process_node(na)
 #if	defined(HASPROCFS)
 	struct as as;
 	struct proc p;
+	KA_T pa;
 	struct procfsid *pfi;
-	struct pid pids;
+	long pid;
 	struct prnode pr;
-#endif	/* HASPROCFS */
+# if	UNIXWAREV<20102
+	struct pid pids;
+# else	/* UNIXWAREV>=20102 */
+	struct prcommon prc;
+# endif	/* UNIXWAREV>=20102 */
+#endif	/* defined(HASPROCFS) */
 
 /*
  * Read the vnode.
@@ -495,17 +501,69 @@ get_lock_state:
 		enter_nm(Namech);
 		return;
 	    }
-	    if (pr.pr_proc == NULL) {
-		sd = 0;
-		if (v.v_type == VDIR) {
-		    (void) sprintf(Namech, "/%s", HASPROCFS);
-		    enter_nm(Namech);
-		    i.number = PR_ROOTINO;
-		} else {
-		    (void) sprintf(Namech, "/%s/???", HASPROCFS);
-		    enter_nm(Namech);
-		    i.number = -1L;
+
+# if	UNIXWAREV>=20102
+	    i.number = (long)pr.pr_ino;
+	    sd = 0;
+	    if (pr.pr_common
+	    &&  kread((KA_T)pr.pr_common, (char *)&prc, sizeof(prc)) == 0) {
+		pid = (long)prc.prc_pid;
+		switch(pr.pr_type) {
+		case PR_PIDDIR:
+		    (void) sprintf(Namech, "/%s/%ld", HASPROCFS, pid);
+		    break;
+		case PR_AS:
+		    (void) sprintf(Namech, "/%s/%ld/as", HASPROCFS, pid);
+		    break;
+		case PR_CTL:
+		    (void) sprintf(Namech, "/%s/%ld/ctl", HASPROCFS, pid);
+		    break;
+		case PR_STATUS:
+		    (void) sprintf(Namech, "/%s/%ld/status", HASPROCFS, pid);
+		    break;
+		case PR_MAP:
+		    (void) sprintf(Namech, "/%s/%ld/map", HASPROCFS, pid);
+		    break;
+		case PR_CRED:
+		    (void) sprintf(Namech, "/%s/%ld/cred", HASPROCFS, pid);
+		    break;
+		case PR_SIGACT:
+		    (void) sprintf(Namech, "/%s/%ld/sigact", HASPROCFS, pid);
+		    break;
+		case PR_OBJECTDIR:
+		    (void) sprintf(Namech, "/%s/%ld/object", HASPROCFS, pid);
+		    break;
+		case PR_LWPDIR:
+		    (void) sprintf(Namech, "/%s/%ld/lwp", HASPROCFS, pid);
+		    break;
+		case PR_LWPIDDIR:
+		    (void) sprintf(Namech, "/%s/%ld/lwp/%d",
+			HASPROCFS, pid, prc.prc_lwpid);
+		    break;
+		case PR_LWPCTL:
+		    (void) sprintf(Namech, "/%s/%ld/lwp/%d/lwpctl",
+			HASPROCFS, pid, prc.prc_lwpid);
+		    break;
+		case PR_LWPSTATUS:
+		    (void) sprintf(Namech, "/%s/%ld/lwp/%d/lwpstatus",
+			HASPROCFS, pid, prc.prc_lwpid);
+		    break;
+		case PR_LWPSINFO:
+		    (void) sprintf(Namech, "/%s/%ld/lwp/%d/lwpsinfo",
+			HASPROCFS, pid, prc.prc_lwpid);
+		    break;
 		}
+	    } else
+		pid = 0l;
+	    break;
+# else	/* UNIXWAREV<20102 */
+	    if (!pr.pr_proc) {
+		sd = 0;
+		pid = 0l;
+		if (v.v_type == VDIR)
+		    i.number = (long)PR_ROOTINO;
+		else
+		    i.number = (long)0;
 		break;
 	    }
 	    if (kread((KA_T)pr.pr_proc, (char *)&p, sizeof(p))) {
@@ -522,15 +580,16 @@ get_lock_state:
 		enter_nm(Namech);
 		return;
 	    }
+	    pid = (long)pids.p_pid;
+	    (void) sprintf(Namech, "/%s/%ld", HASPROCFS, pid);
+	    i.number = pid + PR_INOBIAS;
 	    if (!p.p_as ||  kread((KA_T)p.p_as, (char *)&as, sizeof(as)))
 		sd = 0;
 	    else
 		i.size = as.a_size;
-	    (void) sprintf(Namech, "/%s/%0*d", HASPROCFS, PNSIZ, pids.pid_id);
-	    i.number = (long)pids.pid_id + PR_INOBIAS;
-	    enter_nm(Namech);
 	    break;
-#endif	/* HASPROCFS */
+# endif	/* UNIXWAREV>=20102 */
+#endif	/* defined(HASPROCFS) */
 
 	case N_STREAM:
 	    if (v.v_stream) {
@@ -682,7 +741,7 @@ get_lock_state:
 	case N_PROC:
 	    dev = kv.vfs_dev;
 	    break;
-#endif	/* HASPROCFS */
+#endif	/* defined(HASPROCFS) */
 
 	case N_STREAM:
 	    if (sqp) {
@@ -709,7 +768,7 @@ get_lock_state:
 	    Lf->inode = (unsigned long)i.number;
 	    Lf->inp_ty = 1;
 	    break;
-#endif	/* HASPROCFS */
+#endif	/* defined(HASPROCFS) */
 
 	case N_FIFO:
 	    if (!f.fn_realvp) {
@@ -749,21 +808,21 @@ get_lock_state:
 		    Lf->off_def = 1;
 		break;
 	    case N_NFS:
-		Lf->sz = (unsigned long)r.r_attr.va_size;
+		Lf->sz = (SZOFFTYPE)r.r_attr.va_size;
 		Lf->sz_def = sd;
 		break;
 
 #if	defined(HASPROCFS)
 	    case N_PROC:
-		Lf->sz = (unsigned long)i.size;
+		Lf->sz = (SZOFFTYPE)i.size;
 		Lf->sz_def = sd;
 		break;
-#endif	/* HASPROCFS */
+#endif	/* defined(HASPROCFS) */
 
 	    case N_REGLR:
 		if (type == VREG || type == VDIR) {
 		    if (!ni) {
-			Lf->sz = (unsigned long)i.size;
+			Lf->sz = (SZOFFTYPE)i.size;
 			Lf->sz_def = sd;
 		    }
 		} else if (type == VCHR && !Fsize)
@@ -878,18 +937,31 @@ get_lock_state:
 
 #if	defined(HASPROCFS)
 	if (Ntype == N_PROC) {
-	    if (Procsrch)
+	    if (Procsrch) {
+		Procfind = 1;
 		Lf->sf |= SELNM;
-	    else {
+	    } else {
 		for (pfi = Procfsid; pfi; pfi = pfi->next) {
-		    if (pfi->pid == pids.pid_id) {
+
+		    if ((pfi->pid && pid && pfi->pid == (pid_t)pid)
+
+# if	defined(HASPINODEN)
+		    || (Lf->inp_ty == 1 && Lf->inode == pfi->inode)
+# endif	/* defined(HASPINODEN) */
+
+		    ) {
+			pfi->f = 1;
 			Lf->sf |= SELNM;
+			if (!Namech[0] && pfi->nm) {
+			    (void) strncpy(Namech, pfi->nm, MAXPATHLEN - 1);
+			    Namech[MAXPATHLEN-1] = '\0';
+			}
 			break;
 		    }
 		}
 	    }
 	} else
-#endif	/* HASPROCFS */
+#endif	/* defined(HASPROCFS) */
 
 	{
 	    if (Sfile && is_file_named(NULL, type))
