@@ -34,7 +34,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: main.c,v 1.6 97/10/10 08:27:30 abe Exp $";
+static char *rcsid = "$Id: main.c,v 1.13 98/02/16 15:54:56 abe Exp $";
 #endif
 
 
@@ -46,23 +46,26 @@ static char *rcsid = "$Id: main.c,v 1.6 97/10/10 08:27:30 abe Exp $";
  */
 
 static int GObk[] = { 1, 1 };		/* option backspace values */
+static char GOp;			/* option prefix -- '+' or '-' */
+static char *GOv = (char *)NULL;	/* option `:' value pointer */
 static int GOx1 = 1;			/* first opt[][] index */
-static int GOx2 = 1;			/* second opt[][] index */
-static char *GOv = NULL;		/* option `:' value pointer */
+static int GOx2 = 0;			/* second opt[][] index */
 
 
 _PROTOTYPE(static int GetOpt,(int ct, char *opt[], char *rules, int *err));
+_PROTOTYPE(static char *sv_fmt_str,(char *f));
 
 
 /*
  * main() - main function for lsof
  */
 
+int
 main(argc, argv)
 	int argc;
 	char *argv[];
 {
-	int c, i, rv;
+	int c, i, n, rv;
 	char *cp;
 	int err = 0;
 	int fh = 0;
@@ -70,12 +73,12 @@ main(argc, argv)
 	struct nwad *np, *npn;
 	char options[128];
 	struct sfile *sfp;
-	struct lproc **slp = NULL;
+	struct lproc **slp = (struct lproc **)NULL;
 	int sp = 0;
 	struct str_lst *str;
 	int version = 0;
 
-	if ((Pn = strrchr(argv[0], '/')) != NULL)
+	if ((Pn = strrchr(argv[0], '/')))
 		Pn++;
 	else
 		Pn = argv[0];
@@ -87,7 +90,7 @@ main(argc, argv)
 		Setgid = 1;
 	if ((Myuid = (uid_t)getuid()) && !geteuid())
 		Setuidroot = 1;
-	if ((Namech = (char *)malloc(MAXPATHLEN)) == NULL) {
+	if (!(Namech = (char *)malloc(MAXPATHLEN))) {
 		(void) fprintf(stderr, "%s: no space for name buffer\n", Pn);
 		Exit(1);
 	}
@@ -95,7 +98,7 @@ main(argc, argv)
  * Create option mask.
  */
 	(void) sprintf(options,
-		"?ab%sc:%sd:%sF:g:hi:%sl%snNoOp:Pr:%ssS:tT:u:UvVw%s",
+		"?a%sbc:%sd:%sfF:g:hi:%sl%sMnNo:Op:Pr:%ssS:tT:u:UvVw%s",
 
 #if	defined(HAS_AFS) && defined(HASAOPT)
 		"A:",
@@ -160,7 +163,7 @@ main(argc, argv)
 
 #if	defined(HAS_AFS) && defined(HASAOPT)
 		case 'A':
-			if (GOv == NULL || *GOv == '-') {
+			if (!GOv || *GOv == '-' || *GOv == '+') {
 			    (void) fprintf(stderr,
 				"%s: -A not followed by path\n", Pn);
 			    err = 1;
@@ -179,7 +182,7 @@ main(argc, argv)
 
 #if	defined(HASNCACHE)
 		case 'C':
-			Fncache = 0;
+			Fncache = (GOp == '-') ? 0 : 1;
 			break;
 #endif	/* defined(HASNCACHE) */
 
@@ -195,11 +198,14 @@ main(argc, argv)
 			break;
 #endif	/* defined(HASDCACHE) */
 
+		case 'f':
+			Ffilesys = (GOp == '+') ? 2 : 1;
+			break;
 		case 'F':
-			if (GOv == NULL || *GOv == '-'
+			if (!GOv || *GOv == '-' || *GOv == '+'
 			||  strcmp(GOv, "0") == 0) {
-			    if (GOv != NULL) {
-				if (*GOv == '-') {
+			    if (GOv) {
+				if (*GOv == '-' || *GOv == '+') {
 				    GOx1 = GObk[0];
 				    GOx2 = GObk[1];
 				} else if (*GOv == '0')
@@ -245,8 +251,8 @@ main(argc, argv)
 			Ffield = 1;
 			break;
 		case 'g':
-			if (GOv != NULL) {
-			    if (*GOv == '-') {
+			if (GOv) {
+			    if (*GOv == '-' || *GOv == '+') {
 				GOx1 = GObk[0];
 				GOx2 = GObk[1];
 			    } else if (enter_id(PGRP, GOv))
@@ -259,9 +265,9 @@ main(argc, argv)
 			Fhelp = 1;
 			break;
 		case 'i':
-			if (GOv == NULL || *GOv == '-') {
+			if (!GOv || *GOv == '-' || *GOv == '+') {
 			    Fnet = 1;
-			    if (GOv != NULL) {
+			    if (GOv) {
 				GOx1 = GObk[0];
 				GOx2 = GObk[1];
 			    }
@@ -273,7 +279,7 @@ main(argc, argv)
 
 #if	defined(HASKOPT)
 		case 'k':
-			if (GOv == NULL || *GOv == '-') {
+			if (!GOv || *GOv == '-' || *GOv == '+') {
 			    (void) fprintf(stderr,
 				"%s: -k not followed by path\n", Pn);
 			    err = 1;
@@ -288,7 +294,7 @@ main(argc, argv)
 
 #if	defined(HASMOPT)
 		case 'm':
-			if (GOv == NULL || *GOv == '-') {
+			if (!GOv || *GOv == '-' || *GOv == '+') {
 			    (void) fprintf(stderr,
 				"%s: -m not followed by path\n", Pn);
 			    err = 1;
@@ -297,49 +303,71 @@ main(argc, argv)
 			break;
 #endif	/* defined(HASMOPT) */
 
+		case 'M':
+			FportMap = (GOp == '+') ? 1 : 0;
+			break;
 		case 'n':
-			Fhost = 0;
+			Fhost = (GOp == '-') ? 0 : 1;
 			break;
 		case 'N':
 			Fnfs = 1;
 			break;
 		case 'o':
-			Foffset = 1;
+			if (!GOv || *GOv == '-' || *GOv == '+') {
+			    Foffset = 1;
+			    if (GOv) {
+				GOx1 = GObk[0];
+				GOx2 = GObk[1];
+			    }
+			    break;
+			}
+			for (cp = GOv, i = n = 0; *cp; cp++) {
+			    if (!isdigit((unsigned char)*cp))
+				break;
+			    i = (i * 10) + ((int)*cp - '0');
+			    n++;
+			}
+			if (n)
+			    OffDecDig = i;
+			else
+			    Foffset = 1;
+			if (*cp) {
+			    GOx1 = GObk[0];
+			    GOx2 = GObk[1] + n;
+			}
 			break;
 		case 'O':
-			Fovhd = 1;
+			Fovhd = (GOp == '-') ? 1 : 0;
 			break;
 		case 'p':
 			if (enter_id(PID, GOv))
 			    err = 1;
 			break;
 		case 'P':
-			Fport = 0;
+			Fport = (GOp == '-') ? 0 : 1;
 			break;
 		case 'r':
-			if (GOv == NULL || *GOv == '-')
-			    i = -1;
-			else {
-			    for (cp = GOv, i = 0; *cp; cp++) {
-				if (!isdigit(*cp)) {
-				    i = -1;
-				    break;
-				}
-				i = (i * 10) + ((int)*cp - '0');
-			    }
-			}
-			if (i > 0)
-			    RptTm = i;
-			else if (i < 0) {
-			    if (GOv != NULL) {
+			if (!GOv || *GOv == '-' || *GOv == '+') {
+			    if (GOv) {
 				GOx1 = GObk[0];
 				GOx2 = GObk[1];
 			    }
 			    RptTm = RPTTM;
-			} else {
-			    (void) fprintf(stderr,
-				"%s: illegal -r time: %s\n", Pn, GOv);
-			    err = 1;
+			    break;
+			}
+			for (cp = GOv, i = n = 0; *cp; cp++) {
+			    if (!isdigit((unsigned char)*cp))
+				break;
+			    i = (i * 10) + ((int)*cp - '0');
+			    n++;
+			}
+			if (n)
+			    RptTm = i;
+			else
+			    RptTm = RPTTM;
+			if (*cp) {
+			    GOx1 = GObk[0];
+			    GOx2 = GObk[1] + n;
 			}
 			break;
 
@@ -353,14 +381,28 @@ main(argc, argv)
 			Fsize = 1;
 			break;
 		case 'S':
-			if (GOv == NULL || *GOv == '-') {
-			    if (GOv != NULL) {
+			if (!GOv || *GOv == '-' || *GOv == '+') {
+			    if (GOv) {
 				GOx1 = GObk[0];
 				GOx2 = GObk[1];
 			    }
 			    TmLimit = TMLIMIT;
-			} else
-			    TmLimit = atoi(GOv);
+			    break;
+			}
+			for (cp = GOv, i = n = 0; *cp; cp++) {
+			    if (!isdigit((unsigned char)*cp))
+				break;
+			    i = (i * 10) + ((int)*cp - '0');
+			    n++;
+			}
+			if (n)
+			    TmLimit = i;
+			else
+			    TmLimit = TMLIMIT;
+			if (*cp) {
+			    GOx1 = GObk[0];
+			    GOx2 = GObk[1] + n;
+			}
 			if (TmLimit < TMLIMMIN) {
 			    (void) fprintf(stderr,
 				"%s: WARNING: -S time (%d) changed to %d\n",
@@ -372,15 +414,15 @@ main(argc, argv)
 			Fterse = Fwarn = 1;
 			break;
 		case 'T':
-			Ftcptpi = 0;
-			if (!GOv || *GOv == '-') {
+			if (!GOv || *GOv == '-' || *GOv == '+') {
 			    if (GOv) {
 				GOx1 = GObk[0];
 				GOx2 = GObk[1];
 			    }
+			    Ftcptpi = (GOp == '-') ? 0 : TCPTPI_STATE;
 			    break;
 			}
-			for (; *GOv; GOv++) {
+			for (Ftcptpi = 0; *GOv; GOv++) {
 			    switch (*GOv) {
 
 #if	defined(HASTCPTPIQ)
@@ -421,13 +463,7 @@ main(argc, argv)
 			Fverbose = 1;
 			break;
 		case 'w':
-
-#if	defined(WARNINGSTATE)
-			Fwarn = 0;
-#else	/* !defined(WARNINGSTATE) */
-			Fwarn = 1;
-#endif	/* defined(WARNINGSTATE) */
-
+			Fwarn = (GOp == '+') ? 0 : 1;
 			break;
 
 #if	defined(HASXOPT)
@@ -461,9 +497,8 @@ main(argc, argv)
  * Reduce the size of Suid[], if necessary.
  */
 	if (Suid && Nuid && Nuid < Mxuid) {
-		if ((Suid = (struct seluid *)realloc((MALLOC_P *)Suid,
-				(MALLOC_S)(sizeof(struct seluid) * Nuid)))
-		== (struct seluid *)NULL)
+		if (!(Suid = (struct seluid *)realloc((MALLOC_P *)Suid,
+				(MALLOC_S)(sizeof(struct seluid) * Nuid))))
 		{
 			(void) fprintf(stderr, "%s: can't realloc UID table\n",
 				Pn);
@@ -541,6 +576,17 @@ main(argc, argv)
 		readdev(0);
 #endif	/* defined(HASDCACHE) */
 
+/*
+ * Define the size and offset print formats.
+ */
+	(void) sprintf(options, "0t%%%su", SZOFFPSPEC);
+	SzOffFmt_0t = sv_fmt_str(options);
+	(void) sprintf(options, "%%%su", SZOFFPSPEC);
+	SzOffFmt_d = sv_fmt_str(options);
+	(void) sprintf(options, "%%*%su", SZOFFPSPEC);
+	SzOffFmt_dv = sv_fmt_str(options);
+	(void) sprintf(options, "%%#%sx", SZOFFPSPEC);
+	SzOffFmt_x = sv_fmt_str(options);
 /*
  * Gather and report process information every RptTm seconds.
  */
@@ -558,7 +604,7 @@ main(argc, argv)
 	 * function will have produced output as it became available.
 	 */
 	    if (Nlproc > 1) {
-		if (slp == NULL) {
+		if (!slp) {
 		    slp = (struct lproc **)malloc(
 			  (MALLOC_S)(sizeof(struct lproc *) * Nlproc));
 		    sp = Nlproc;
@@ -567,10 +613,9 @@ main(argc, argv)
 			  (MALLOC_S)(sizeof(struct lproc *) * Nlproc));
 		    sp = Nlproc;
 		}
-		if (slp == NULL) {
+		if (!slp) {
 		    (void) fprintf(stderr,
-			"%s: WARNING -- no space for %d sort pointers\n",
-			Pn, Nlproc);
+			"%s: no space for %d sort pointers\n", Pn, Nlproc);
 		    Exit(1);
 		}
 		for (i = 0; i < Nlproc; i++) {
@@ -579,12 +624,23 @@ main(argc, argv)
 		(void) qsort((QSORT_P *)slp, (size_t)Nlproc,
 			     (size_t)sizeof(struct lproc *), comppid);
 	    }
-	    for (i = 0; i < Nlproc; i++) {
-		Lp = (Nlproc > 1) ? slp[i] : &Lproc[i];
-		if (Lp->pss)
-		    print_proc();
-		if (RptTm)
-		    (void) free_lproc(Lp);
+	    if (Nlproc) {
+
+	    /*
+	     * Lf contents must be preserved, since they may point to a
+	     * malloc()'d area, and since Lf is used throughout the print
+	     * process.
+	     */
+		for (lf = Lf, print_init(); PrPass < 2; PrPass++) {
+		    for (i = 0; i < Nlproc; i++) {
+			Lp = (Nlproc > 1) ? slp[i] : &Lproc[i];
+			if (Lp->pss)
+			    print_proc();
+			if (RptTm && PrPass)
+			    (void) free_lproc(Lp);
+		    }
+		}
+		Lf = lf;
 	    }
 	/*
 	 * If a repeat time is set, sleep for the specified time.
@@ -628,8 +684,36 @@ main(argc, argv)
 		continue;
 	    rv = 1;
 	    if (Fverbose)
-		(void) printf("%s: file not located: %s\n", Pn, sfp->aname);
+		(void) printf("%s: no file%s use located: %s\n",
+		    Pn,
+		    sfp->type ? "" : " system",
+		    sfp->aname);
 	}
+
+#if	defined(HASPROCFS)
+	/*
+	 * Report on proc file system search results.
+	 */
+	    if (Procsrch && !Procfind) {
+		rv = 1;
+		if (Fverbose)
+		    (void) printf("%s: no file system use located: %s\n",
+		    Pn, Mtprocfs ? Mtprocfs->dir : HASPROCFS);
+	    }
+	    {
+		struct procfsid *pfi;
+
+		for (pfi = Procfsid; pfi; pfi = pfi->next) {
+		    if (!pfi->f) {
+			rv = 1;
+			if (Fverbose)
+			    (void) printf("%s: no file use located: %s\n",
+				Pn, pfi->nm);
+		    }
+		}
+	    }
+#endif	/* defined(HASPROCFS) */
+
 	for (np = Nwad; np; np = npn) {
 
 	/*
@@ -730,22 +814,31 @@ GetOpt(ct, opt, rules, err)
 	char *rules;			/* option rules */
 	int *err;			/* error return */
 {
-	static int sp = 1;
 	register int c;
-	register char *cp;
+	register char *cp = (char *)NULL;
 
-	if (GOx2 == 1) {
+	if (GOx2 == 0) {
 
 	/*
 	 * Move to a new entry of the option array.
+	 *
+	 * EOF if:
+	 *
+	 *	Option list has been exhausted;
+	 *	Next option doesn't start with `-' or `+';
+	 *	Next option has nothing but `-' or `+';
+	 *	Next option is ``--'' or ``++''.
 	 */
-		if (GOx1 >= ct
-		||  opt[GOx1][0] != '-' || opt[GOx1][1] == '\0')
-			return(EOF);
-		else if (strcmp(opt[GOx1], "--") == 0) {
-			GOx1++;
-			return(EOF);
-		}
+	    if (GOx1 >= ct
+	    ||  (opt[GOx1][0] != '-' && opt[GOx1][0] != '+')
+	    ||  !opt[GOx1][1])
+		 return(EOF);
+	    if (strcmp(opt[GOx1], "--") == 0 || strcmp(opt[GOx1], "++") == 0) {
+		GOx1++;
+		return(EOF);
+	    }
+	    GOp = opt[GOx1][0];
+	    GOx2 = 1;
 	}
 /*
  * Flag `:' option character as an error.
@@ -754,13 +847,12 @@ GetOpt(ct, opt, rules, err)
  */
 	*err = 0;
 	if ((c = opt[GOx1][GOx2]) == ':') {
-		(void) fprintf(stderr,
-			"%s: colon is an illegal option character.\n", Pn);
-		*err = 1;
-	} else if ((cp = strchr(rules, c)) == NULL) {
-		(void) fprintf(stderr, "%s: illegal option character: %c\n",
-			Pn, c);
-		*err = 2;
+	    (void) fprintf(stderr,
+		"%s: colon is an illegal option character.\n", Pn);
+	    *err = 1;
+	} else if (!(cp = strchr(rules, c))) {
+	    (void) fprintf(stderr, "%s: illegal option character: %c\n", Pn, c);
+	    *err = 2;
 	}
 	if (*err) {
 
@@ -771,11 +863,11 @@ GetOpt(ct, opt, rules, err)
 	 *
 	 * Return the character causing the error.
 	 */
-		if (opt[GOx1][++GOx2] == '\0') {
-			GOx1++;
-			GOx2 = 1;
-		}
-		return(c);
+	    if (opt[GOx1][++GOx2] == '\0') {
+		GOx1++;
+		GOx2 = 0;
+	    }
+	    return(c);
 	}
 	if (*(cp + 1) == ':') {
 
@@ -796,22 +888,22 @@ GetOpt(ct, opt, rules, err)
 	 * option following the possible value, so that the next call
 	 * to GetOpt() will find it.
 	 */
-		if(opt[GOx1][GOx2 + 1] != '\0') {
-			GObk[0] = GOx1;
-			GObk[1] = ++GOx2;
-			GOv = &opt[GOx1++][GOx2];
-		} else if (++GOx1 >= ct)
-			GOv = NULL;
-		else {
-			GObk[0] = GOx1;
-			GObk[1] = 1;
-			GOv = opt[GOx1];
-			if (strcmp(GOv, "--") == 0)
-				GOv = NULL;
-			else
-				GOx1++;
-		}
-		GOx2 = 1;
+	    if(opt[GOx1][GOx2 + 1] != '\0') {
+		GObk[0] = GOx1;
+		GObk[1] = ++GOx2;
+		GOv = &opt[GOx1++][GOx2];
+	    } else if (++GOx1 >= ct)
+		GOv = (char *)NULL;
+	    else {
+		GObk[0] = GOx1;
+		GObk[1] = 0;
+		GOv = opt[GOx1];
+		if (strcmp(GOv, "--") == 0)
+		    GOv = (char *)NULL;
+		else
+		    GOx1++;
+	    }
+	    GOx2 = 0;
 	} else {
 
 	/*
@@ -819,14 +911,36 @@ GetOpt(ct, opt, rules, err)
 	 *
 	 * Advance to the next option character.
 	 */
-		if (opt[GOx1][++GOx2] == '\0') {
-			GOx2 = 1;
-			GOx1++;
-		}
-		GOv = NULL;
+	    if (opt[GOx1][++GOx2] == '\0') {
+		GOx2 = 0;
+		GOx1++;
+	    }
+	    GOv = (char *)NULL;
 	}
 /*
  * Return the option character.
  */
 	return(c);
 }
+
+
+/*
+ * sv_fmt_str() - save format string
+ */
+
+static char *
+sv_fmt_str(f)
+	char *f;			/* format string */
+{
+	char *cp;
+	MALLOC_S l;
+
+	l = (MALLOC_S)(strlen(f) + 1);
+	if (!(cp = (char *)malloc(l))) {
+	    (void) fprintf(stderr,
+		"%s: can't allocate %d bytes for format: %s\n", Pn, (int)l, f);
+	    Exit(1);
+	}
+	(void) strcpy(cp, f);
+	return(cp);
+}
