@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: misc.c,v 1.6 97/10/10 08:22:17 abe Exp $";
+static char *rcsid = "$Id: misc.c,v 1.9 98/01/29 15:38:48 abe Exp $";
 #endif
 
 
@@ -44,7 +44,7 @@ static char *rcsid = "$Id: misc.c,v 1.6 97/10/10 08:22:17 abe Exp $";
  */
 
 _PROTOTYPE(static void closePipes,(void));
-_PROTOTYPE(static int dolstat,(char *path, char *buf, int len));
+_PROTOTYPE(static int dostat,(char *path, char *buf, int len));
 _PROTOTYPE(static int doreadlink,(char *path, char *buf, int len));
 _PROTOTYPE(static int doinchild,(int (*fn)(), char *fp, char *rbuf, int rbln));
 
@@ -104,7 +104,7 @@ build_Nl(d)
 	{
 	    (void) fprintf(stderr,
 		"%s: can't allocate %d bytes to kernel name list structure\n",
-		Pn, (n + 1) * sizeof(struct NLIST_TYPE));
+		Pn, (int)((n + 1) * sizeof(struct NLIST_TYPE)));
 	    Exit(1);
 	}
 	for (dp = d, i = 0; i < n; dp++, i++) {
@@ -130,7 +130,7 @@ childx()
 		if (!Fwarn)
 		    (void) fprintf(stderr,
 			"%s: WARNING -- child process %d may be hung.\n",
-			Pn, Cpid);
+			Pn, (int)Cpid);
 	    } else {
 		(void) kill(Cpid, SIGINT);
 		(void) kill(Cpid, SIGKILL);
@@ -196,7 +196,7 @@ dropgid()
 	if (!Setuidroot && Setgid) {
 		if (setgid(Mygid) < 0) {
 			(void) fprintf(stderr, "%s: can't setgid(%d): %s\n",
-				Pn, Mygid, strerror(errno));
+				Pn, (int)Mygid, strerror(errno));
 			Exit(1);
 		}
 		Setgid = 0;
@@ -315,7 +315,7 @@ doinchild(fn, fp, rbuf, rbln)
 		    if (!Fwarn && Cpid)
 			(void) fprintf(stderr,
 			    "%s: WARNING -- child process %d may be hung.\n",
-			    Pn, Cpid);
+			    Pn, (int)Cpid);
 		} else if (Cpid) {
 		    (void) kill(Cpid, SIGINT);
 		    (void) kill(Cpid, SIGKILL);
@@ -378,15 +378,15 @@ doinchild(fn, fp, rbuf, rbln)
 		 */
 		    for (;;) {
 			if (read(Pipes[0], (char *)&r_fn, sizeof(r_fn))
-			    != sizeof(r_fn)
+			    != (int)sizeof(r_fn)
 			||  read(Pipes[0], (char *)&r_al, sizeof(int))
-			    != sizeof(int)
+			    != (int)sizeof(int)
 			||  r_al < 1
-			||  r_al > sizeof(r_arg)
+			||  r_al > (int)sizeof(r_arg)
 			||  read(Pipes[0], r_arg, r_al) != r_al
 			||  read(Pipes[0], (char *)&r_rbln, sizeof(r_rbln))
-			    != sizeof(r_rbln)
-			||  r_rbln < 1 || r_rbln > sizeof(r_rbuf))
+			    != (int)sizeof(r_rbln)
+			||  r_rbln < 1 || r_rbln > (int)sizeof(r_rbuf))
 			    break;
 			rv = r_fn(r_arg, r_rbuf, r_rbln);
 			en = errno;
@@ -455,11 +455,14 @@ doinchild(fn, fp, rbuf, rbln)
 
 
 /*
- * dolstat() - do an lstat() function
+ * dostat() - do a *stat() function
+ *
+ * DOSTAT_FUNCTION defines the function -- e.g., stat() or lstat() --
+ * default = lstat().
  */
 
 static int
-dolstat(path, rbuf, rbln)
+dostat(path, rbuf, rbln)
 	char *path;			/* path */
 	char *rbuf;			/* response buffer */
 	int rbln;			/* response buffer length */
@@ -467,7 +470,12 @@ dolstat(path, rbuf, rbln)
 /* ARGSUSED */
 
 {
-	return(lstat(path, (struct stat *)rbuf));
+
+#if	!defined(DOSTAT_FUNCTION)
+#define	DOSTAT_FUNCTION	lstat
+#endif	/* !defined(DOSTAT_FUNCTION) */
+
+	return(DOSTAT_FUNCTION(path, (struct stat *)rbuf));
 }
 
 
@@ -496,10 +504,10 @@ enter_dev_ch(m)
 	MALLOC_S len;
 	char *mp;
 
-	if (m == NULL || *m == '\0')
+	if (!m || *m == '\0')
 		return;
 	len = strlen(m) + 1;
-	if ((mp = (char *)malloc(len)) == NULL) {
+	if (!(mp = (char *)malloc(len))) {
 		(void) fprintf(stderr, "%s: no more dev_ch space at PID %d\n",
 			Pn, Lp->pid);
 		Exit(1);
@@ -522,10 +530,10 @@ enter_nm(m)
 	MALLOC_S len;
 	char *mp;
 
-	if (m == NULL || *m == '\0')
+	if (!m || *m == '\0')
 		return;
 	len = strlen(m) + 1;
-	if ((mp = (char *)malloc(len)) == NULL) {
+	if (!(mp = (char *)malloc(len))) {
 		(void) fprintf(stderr, "%s: no more nm space at PID %d\n",
 			Pn, Lp->pid);
 		Exit(1);
@@ -568,7 +576,7 @@ is_nw_addr(ia, p)
 	struct nwad *n;
 	unsigned char *u;
 
-	if ((n = Nwad) == NULL)
+	if (!(n = Nwad))
 		return(0);
 	u = (unsigned char *)ia;
 	for (; n; n = n->next) {
@@ -608,7 +616,7 @@ Readlink(arg)
 	char lbuf[MAXPATHLEN];
 	static int ss = 0;
 	char *s1;
-	static char **stk = NULL;
+	static char **stk = (char **)NULL;
 	static int sx = 0;
 	char tbuf[MAXPATHLEN];
 /*
@@ -634,7 +642,7 @@ path_too_long:
 				(void) fprintf(stderr,
 					"%s: path too long: %s\n", Pn, arg);
 			}
-			return(NULL);
+			return((char *)NULL);
 		}
 		(void) strncpy(tbuf, arg, len);
 		tbuf[len] = '\0';
@@ -695,7 +703,7 @@ path_too_long:
 	/*
 	 * Add to the path assembly.
 	 */
-		if ((alen + llen + slen) >= sizeof(abuf))
+		if ((alen + llen + slen) >= (int)sizeof(abuf))
 			goto path_too_long;
 		if (slen == 2)
 			*ap++ = '/';
@@ -712,7 +720,7 @@ path_too_long:
 		for (i = 0; i < sx; i++) {
 			if (i < (sx - 1))
 				(void) free((FREE_P *)stk[i]);
-			stk[i] = NULL;
+			stk[i] = (char *)NULL;
 		}
 		sx = 0;
 		return(arg);
@@ -721,7 +729,7 @@ path_too_long:
  * If the assembled path and argument are different, add it to the
  * string stack, then Readlink() it.
  */
-	if ((s1 = (char *)malloc((MALLOC_S)(alen + 1))) == NULL) {
+	if (!(s1 = (char *)malloc((MALLOC_S)(alen + 1)))) {
 
 no_readlink_space:
 
@@ -731,11 +739,11 @@ no_readlink_space:
 	}
 	(void) strcpy(s1, abuf);
 	if (++sx > ss) {
-		if (stk == NULL)
+		if (!stk)
 			stk = (char **)malloc(sizeof(char *) * sx);
 		else
 			stk = (char **)realloc(stk, sizeof(char *) * sx);
-		if (stk == NULL)
+		if (!stk)
 			goto no_readlink_space;
 		ss = sx;
 	}
@@ -754,9 +762,10 @@ readstdata(addr, buf)
 	struct stdata *addr;		/* stdata address in kernel*/
 	struct stdata *buf;		/* buffer addess */
 {
-	if (addr == NULL
+	if (!addr
 	||  kread((KA_T)addr, (char *)buf, sizeof(struct stdata))) {
-		(void) sprintf(Namech, "no stream data in %#x\n", addr);
+		(void) sprintf(Namech, "no stream data in %#lx\n",
+			(unsigned long)addr);
 		return(1);
 	}
 	return(0);
@@ -774,14 +783,14 @@ readsthead(addr, buf)
 {
 	struct queue *qp;
 
-	if (addr == NULL) {
+	if (!addr) {
 		(void) strcpy(Namech, "no stream queue head");
 		return(1);
 	}
 	for (qp = addr; qp; qp = buf->q_next) {
 		if (kread((KA_T)qp, (char *)buf, sizeof(struct queue))) {
-			(void) sprintf(Namech, "bad stream queue link at %#x",
-				qp);
+			(void) sprintf(Namech, "bad stream queue link at %#lx",
+				(unsigned long)qp);
 			return(1);
 		}
 	}
@@ -799,9 +808,9 @@ readstidnm(addr, buf, len)
 	char *buf;			/* receiving buffer address */
 	READLEN_T len;			/* buffer length */
 {
-	if (addr == NULL ||  kread((KA_T)addr, buf, len)) {
-		(void) sprintf(Namech, "can't read module ID name from %#x",
-			addr);
+	if (!addr ||  kread((KA_T)addr, buf, len)) {
+		(void) sprintf(Namech, "can't read module ID name from %#lx",
+			(unsigned long)addr);
 		return(1);
 	}
 	return(0);
@@ -817,9 +826,10 @@ readstmin(addr, buf)
 	struct module_info *addr;	/* module info address in kernel */
 	struct module_info *buf;	/* receiving buffer address */
 {
-	if (addr == NULL
+	if (!addr
 	||  kread((KA_T)addr, (char *)buf, sizeof(struct module_info))) {
-		(void) sprintf(Namech, "can't read module info from %#x", addr);
+		(void) sprintf(Namech, "can't read module info from %#lx",
+			(unsigned long)addr);
 		return(1);
 	}
 	return(0);
@@ -835,9 +845,10 @@ readstqinit(addr, buf)
 	struct qinit *addr;		/* queue info address in kernel */
 	struct qinit *buf;		/* receiving buffer address */
 {
-	if (addr == NULL
+	if (!addr
 	||  kread((KA_T)addr, (char *)buf, sizeof(struct qinit))) {
-		(void) sprintf(Namech, "can't read queue info from %#x", addr);
+		(void) sprintf(Namech, "can't read queue info from %#lx",
+			(unsigned long)addr);
 		return(1);
 	}
 	return(0);
@@ -862,7 +873,7 @@ statsafely(path, buf)
 	    errno = EWOULDBLOCK;
 	    return(1);
 	}
-	return(doinchild(dolstat, path, (char *)buf, sizeof(struct stat)));
+	return(doinchild(dostat, path, (char *)buf, sizeof(struct stat)));
 }
 
 
@@ -889,8 +900,9 @@ x2dev(s, d)
 	if  (strncasecmp(s, "0x", 2) == 0)
 		s += 2;
 	for (cp = s, n = 0; *cp; cp++, n++) {
-		if (isdigit(*cp)
-		||  (isalpha(*cp) && (c = toupper(*cp)) >= 'A' && c <= 'F'))
+		if (isdigit((unsigned char)*cp)
+		||  (isalpha((unsigned char)*cp)
+		&&   (c = toupper((unsigned char)*cp)) >= 'A' && c <= 'F'))
 			continue;
 		if (*cp == ' ' || *cp == ',')
 			break;
@@ -898,7 +910,7 @@ x2dev(s, d)
 	}
 	if (!n)
 		return((char *)NULL);
-	if (n > (2 * sizeof(dev_t))) {
+	if (n > (2 * (int)sizeof(dev_t))) {
 		cp1 = s;
 		s += (n - (2 * sizeof(dev_t)));
 		while (cp1 < s) {
@@ -912,10 +924,11 @@ x2dev(s, d)
  * in the string relevant to sizeof(dev_t).
  */
 	for (r = 0; s < cp; s++) {
-		r = (r << 4)
-		  | ((isdigit(*s) ? (unsigned char)(*s - '0')
-			          : (unsigned char)(toupper(*s) - 'A' + 10))
-		  		  & 0xf);
+	    r = (r << 4)
+	      | ((isdigit((unsigned char)*s)
+		? (unsigned char)(*s - '0')
+		: (unsigned char)(toupper((unsigned char)*s) - 'A' + 10))
+		  & 0xf);
 	}
 	*d = r;
 	return(s);
