@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: print.c,v 1.5 97/09/23 09:13:13 abe Exp $";
+static char *rcsid = "$Id: print.c,v 1.14 98/01/29 15:40:13 abe Exp $";
 #endif
 
 
@@ -62,20 +62,30 @@ struct hostcache {
 
 struct porttab {
 	int port;
+	MALLOC_S nl;			/* name length (excluding '\0') */
+	int ss;				/* service name status, 0 = lookup not
+					 * yet performed */
 	char *name;
 	struct porttab *next;
 };
 
 
-static struct porttab **Pth[2] = { NULL, NULL };
-						/* port hash buckets: Pth[0]
-						   for TCP, Pth[1] for UDP */
+static struct porttab **Pth[4] = { NULL, NULL, NULL, NULL };
+						/* port hash buckets:
+						 * Pth[0] for TCP service names
+						 * Pth[1] for UDP service names
+						 * Pth[2] for TCP portmap info
+						 * Pth[3] for UDP portmap info
+						 */
 #define HASHPORT(p)	(((((int)(p)) * 31415) >> 3) & (PORTHASHBUCKETS - 1))
 
 
+_PROTOTYPE(static void fill_portmap,(void));
 _PROTOTYPE(static void fill_porttab,(void));
-_PROTOTYPE(static char *lkup_port,(int p, int pr));
+_PROTOTYPE(static char *lkup_port,(int p, int pr, int src));
+_PROTOTYPE(static char *lkup_svcnam,(int h, int p, int pr, int ss));
 _PROTOTYPE(static int printinaddr,(void));
+_PROTOTYPE(static void update_portmap,(struct porttab *pt, char *pn));
 
 
 /*
@@ -94,7 +104,155 @@ endnm()
 
 
 /*
- * fill_porttab() -- fill the port table with a getservent() scan
+ * fill_portmap() -- fill the RPC portmap program name table via a conversation
+ *		     with the portmapper
+ *
+ * The following copyright notice acknowledges that this function was adapted
+ * from getrpcportnam() of the source code of the OpenBSD netstat program.
+ */
+
+/*
+* Copyright (c) 1983, 1988, 1993
+*      The Regents of the University of California.  All rights reserved.
+*
+* Redistribution and use in source and binary forms, with or without
+* modification, are permitted provided that the following conditions
+* are met:
+* 1. Redistributions of source code must retain the above copyright
+*    notice, this list of conditions and the following disclaimer.
+* 2. Redistributions in binary form must reproduce the above copyright
+*    notice, this list of conditions and the following disclaimer in the
+*    documentation and/or other materials provided with the distribution.
+* 3. All advertising materials mentioning features or use of this software
+*    must display the following acknowledgement:
+*      This product includes software developed by the University of
+*      California, Berkeley and its contributors.
+* 4. Neither the name of the University nor the names of its contributors
+*    may be used to endorse or promote products derived from this software
+*    without specific prior written permission.
+*
+* THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+* ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+* FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+* OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+* LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+* OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+* SUCH DAMAGE.
+*/
+
+static void
+fill_portmap()
+{
+	char buf[128], *cp, *nm;
+	CLIENT *c;
+	int h, port, pr;
+	struct hostent *he;
+	struct sockaddr_in ia;
+	MALLOC_S nl;
+	struct pmaplist *p = (struct pmaplist *)NULL;
+	struct porttab *pt;
+	struct rpcent *r;
+	int s = RPC_ANYSOCK;
+	struct timeval tm;
+/*
+ * Construct a TCP socket address for communicating with the portmapper.
+ */
+	zeromem(&ia, sizeof(ia));
+	ia.sin_family = AF_INET;
+	if ((he = gethostbyname("localhost")))
+	    MEMMOVE((caddr_t)&ia.sin_addr, he->h_addr, he->h_length);
+	tm.tv_sec = 60;
+	tm.tv_usec = 0;
+	ia.sin_port = htons(PMAPPORT);
+/*
+ * Get an RPC client handle.  Then ask for a dump of the port map.
+ */
+	if (!(c = clnttcp_create(&ia, PMAPPROG, PMAPVERS, &s, 0, 0)))
+	    return;
+	if (clnt_call(c, PMAPPROC_DUMP, XDR_VOID, NULL, XDR_PMAPLIST,
+		      (caddr_t)&p, tm)
+	!= RPC_SUCCESS) {
+	    clnt_destroy(c);
+	    return;
+	}
+/*
+ * Loop through the port map dump, creating portmap table entries from TCP
+ * and UDP members.
+ */
+	for (; p; p = p->pml_next) {
+	
+	/*
+	 * Determine the port map entry's protocol; ignore all but TCP and UDP.
+	 */
+	    if (p->pml_map.pm_prot == IPPROTO_TCP)
+		pr = 2;
+	    else if (p->pml_map.pm_prot == IPPROTO_UDP)
+		pr = 3;
+	    else
+		continue;
+	/*
+	 * See if there's already a portmap entry for this port.  If there is,
+	 * ignore this entry.
+	 */
+	    h = HASHPORT((port = (int)p->pml_map.pm_port));
+	    for (pt = Pth[pr][h]; pt; pt = pt->next) {
+		if (pt->port == port)
+		    break;
+	    }
+	    if (pt)
+		continue;
+	/*
+	 * Save the registration name or number.
+	 */
+	    cp = (char *)NULL;
+	    if ((r = (struct rpcent *)getrpcbynumber(p->pml_map.pm_prog))) {
+		if (r->r_name && strlen(r->r_name))
+		    cp = r->r_name;
+	    }
+	    if (!cp) {
+		(void) sprintf(buf, "%lu", p->pml_map.pm_prog);
+		cp = buf;
+	    }
+	    if (!(nl = strlen(cp)))
+		continue;
+	/*
+	 * Allocate space for the portmap name entry and copy it there.
+	 */
+	    if (!(nm = (char *)malloc(nl + 1))) {
+		(void) fprintf(stderr,
+		    "%s: can't allocate %d bytes for portmap entry: %s\n",
+		    Pn, (int)(nl + 1), cp);
+		Exit(1);
+	    }
+	    (void) strcpy(nm, cp);
+	/*
+	 * Allocate and fill a porttab struct entry for the portmap table.
+	 * Link it to the head of its hash bucket, and make it the new head.
+	 */
+	    if (!(pt = (struct porttab *)malloc(sizeof(struct porttab)))) {
+		(void) fprintf(stderr,
+		    "%s: can't allocate porttab entry for protmap: %s\n",
+		    Pn, nm);
+		Exit(1);
+	    }
+	    pt->name = nm;
+	    pt->nl = nl;
+	    pt->port = port;
+	    pt->next = Pth[pr][h];
+	    pt->ss = 0;
+	    Pth[pr][h] = pt;
+	}
+	clnt_destroy(c);
+}
+
+
+/*
+ * fill_porttab() -- fill the TCP and UDP service name port table with a
+ *		     getservent() scan
  */
 
 static void
@@ -140,21 +298,22 @@ fill_porttab()
 	 */
 	    if (!(pt = (struct porttab *)malloc(sizeof(struct porttab)))) {
 		(void) fprintf(stderr,
-		    "%s: can't allocate porttab entry for port %d:\n",
+		    "%s: can't allocate porttab entry for port %d: %s\n",
 		    Pn, p, se->s_name);
 		Exit(1);
 	    }
-	    nl++;
-	    if (!(nm = (char *)malloc(nl))) {
+	    if (!(nm = (char *)malloc(nl+1))) {
 		(void) fprintf(stderr,
 		    "%s: can't allocate %d bytes for port %d name: %s\n",
-		    Pn, nl, p, se->s_name);
+		    Pn, (int)(nl + 1), p, se->s_name);
 		Exit(1);
 	    }
 	    (void) strcpy(nm, se->s_name);
 	    pt->name = nm;
+	    pt->nl = nl;
 	    pt->port = p;
 	    pt->next = Pth[pr][h];
+	    pt->ss = 0;
 	    Pth[pr][h] = pt;
 	}
 	(void) endservent();
@@ -170,11 +329,11 @@ gethostnm(ia)
 	struct in_addr *ia;		/* pointer to Internet address */
 {
 	char hbuf[32];
-	static struct hostcache *hc = NULL;
+	static struct hostcache *hc = (struct hostcache *)NULL;
 	static int hcx = 0;
 	char *hn, *np;
 	int i;
-	struct hostent *he = NULL;
+	struct hostent *he = (struct hostent *)NULL;
 	static int nhc = 0;
 	unsigned char *u;
 
@@ -196,7 +355,7 @@ gethostnm(ia)
  */
 	if (Fhost)
 		he = gethostbyaddr((char *)ia, sizeof(struct in_addr), AF_INET);
-	if (he == NULL || he->h_name == NULL) {
+	if (!he || !he->h_name) {
 		(void) sprintf(hbuf, "%u.%u.%u.%u", u[0], u[1], u[2], u[3]);
 		hn = hbuf;
 	} else
@@ -204,7 +363,7 @@ gethostnm(ia)
 /*
  * Allocate space for name and copy name to it.
  */
-	if ((np = (char *)malloc((MALLOC_S)(strlen(hn) + 1))) == NULL) {
+	if (!(np = (char *)malloc((MALLOC_S)(strlen(hn) + 1)))) {
 		(void) fprintf(stderr, "%s: no space for host name: %s\n",
 			Pn, hn);
 		Exit(1);
@@ -215,13 +374,13 @@ gethostnm(ia)
  */
 	if (hcx >= nhc) {
 		nhc += HCINC;
-		if (hc == NULL)
+		if (!hc)
 			hc = (struct hostcache *)malloc(
 				(MALLOC_S)(sizeof(struct hostcache) * nhc));
 		else
 			hc = (struct hostcache *)realloc((MALLOC_P *)hc,
 				(MALLOC_S)(sizeof(struct hostcache) * nhc));
-		if (hc == NULL) {
+		if (!hc) {
 			(void) fprintf(stderr, "%s: no space for host cache\n",
 				Pn);
 			Exit(1);
@@ -241,114 +400,453 @@ gethostnm(ia)
  */
 
 static char *
-lkup_port(p, pr)
+lkup_port(p, pr, src)
 	int p;				/* port number */
 	int pr;				/* protocol index: 0 = tcp, 1 = udp */
+	int src;			/* port source: 0 = local
+					 *		1 = foreign */
 {
-	static int gsbp = 0;
-	int h;
-	MALLOC_S len;
-	char *nm, pb[128], *pn;
+	int h, nh;
+	MALLOC_S nl;
+	char *nm, *pn;
+	static char pb[128];
+	static int pm = 0;
 	struct porttab *pt;
-	static int ptf = 0;
-	struct servent *s;
 /*
- * If the hash buckets haven't been allocated, do so now.
+ * If the hash buckets haven't been allocated, do so.
  */
 	if (!Pth[0]) {
-	    for (h = 0; h < 2; h++) {
+	    nh = FportMap ? 4 : 2;
+	    for (h = 0; h < nh; h++) {
 		if (!(Pth[h] = (struct porttab **)calloc(PORTHASHBUCKETS,
 				sizeof(struct porttab *))))
 		{
 		    (void) fprintf(stderr,
-		      "%s: can't allocate %d bytes for %s port hash buckets\n",
-		      Pn, (2 * (PORTHASHBUCKETS * sizeof(struct porttab *))),
-		      (h == 0) ? "TCP" : "UDP");
+		      "%s: can't allocate %d bytes for %s %s hash buckets\n",
+		      Pn,
+		      (int)(2 * (PORTHASHBUCKETS * sizeof(struct porttab *))),
+		      (h & 1) ? "UDP" : "TCP",
+		      (h > 1) ? "portmap" : "port");
 		    Exit(1);
 		}
 	    }
 	}
 /*
- * Don't do any lookups if the -P option has been specified.  In that case,
- * just cache the %d conversion of the port number.
+ * If we're looking up program names for portmapped ports, make sure the
+ * portmap table has been loaded.
  */
-	if (Fport) {
-
-	/*
-	 * Hash the port and see if its name has been cached.
-	 */
-	    h = HASHPORT(p);
-	    for (;;) {
-		for (pt = Pth[pr][h]; pt; pt = pt->next) {
-		    if (pt->port == p)
-			return(pt->name);
-		}
-	    /*
-	     * The port has no cache entry.  If the port table hasn't already
-	     * been filled by a getservent() scan of the services data base,
-	     * and if the count of getservbyport() calls has reached its
-	     * tolerance limit (PORTTABTHRESH), call fill_porttab() to do the
-	     * getservent() scan and look for the port in the cache once more.
-	     */
-		if (ptf || gsbp < PORTTABTHRESH)
-		    break;
-		(void) fill_porttab();
-		ptf++;
-	    }
+	if (FportMap && !pm) {
+	    (void) fill_portmap();
+	    pm++;
 	}
 /*
- * Allocate a new porttab entry.
+ * Hash the port and see if its name has been cached.  Look for a local
+ * port first in the portmap, if portmap searching is enabled.
+ */
+	h = HASHPORT(p);
+	if (!src && FportMap) {
+	    for (pt = Pth[pr+2][h]; pt; pt = pt->next) {
+		if (pt->port != p)
+		    continue;
+		if (!pt->ss) {
+		    pn = Fport ? lkup_svcnam(h, p, pr, 0) : (char *)NULL;
+		    if (!pn) {
+			(void) sprintf(pb, "%d", p);
+			pn = pb;
+		    }
+		    (void) update_portmap(pt, pn);
+		}
+		return(pt->name);
+	    }
+	}
+	for (pt = Pth[pr][h]; pt; pt = pt->next) {
+	    if (pt->port == p)
+		return(pt->name);
+	}
+/*
+ * Search for a possible service name, unless the -P option has been specified.
+ *
+ * If there is no service name, return a %d conversion.
+ *
+ * Don't cache %d conversions; a zero port number is a %d conversion that
+ * is represented by "*".
+ */
+	pn = Fport ? lkup_svcnam(h, p, pr, 1) : (char *)NULL;
+	if (!pn || !strlen(pn)) {
+	    if (p) {
+		(void) sprintf(pb, "%d", p);
+		return(pb);
+	    } else
+		return("*");
+	}
+/*
+ * Allocate a new porttab entry for the TCP or UDP service name.
  */
 	if (!(pt = (struct porttab *)malloc(sizeof(struct porttab)))) {
 	    (void) fprintf(stderr,
 		"%s: can't allocate porttab entry for port %d\n", Pn, p);
 	    Exit(1);
 	}
-/*
- * If the port cache has been filled via getservent(), then this lookup has
- * failed completely; enter a %d cache entry for it.
- *
- * If the port cache hasn't been filled via getservent(), look up the port's
- * service name via getservbyport().  Count the getservbyport() calls.
- *
- * Don't do a getservbyport() call if the -P option has been specified; just
- * enter a %d conversion of the port number.
- */
-	pn = (char *)NULL;
-	if (Fport && !ptf && p) {
-	    gsbp++;
-	    if ((s = getservbyport(htons(p), (pr == 0) ? "tcp" : "udp")))
-		pn = s->s_name;
-	}
-	if (!pn || !strlen(pn)) {
-	    if (p) {
-		(void) sprintf(pb, "%d", p);
-		pn = pb;
-	    } else
-		pn = "*";
-	}
 /*
  * Allocate space for the name; copy it to the porttab entry; and link the
  * porttab entry to its hash bucket.
  *
  * Return a pointer to the name.
  */
-	len = strlen(pn) + 1;
-	if (!(nm = malloc(len))) {
+	nl = strlen(pn);
+	if (!(nm = malloc(nl + 1))) {
 	    (void) fprintf(stderr,
 		"%s: can't allocate %d bytes for port name: %s\n",
-		Pn, len, pn);
+		Pn, (int)(nl + 1), pn);
 	    Exit(1);
 	}
 	(void) strcpy(nm, pn);
 	pt->name = nm;
+	pt->nl = nl;
 	pt->port = p;
 	pt->next = Pth[pr][h];
+	pt->ss = 0;
 	Pth[pr][h] = pt;
 	return(nm);
 }
 
 
+/*
+ * lkup_svcnam() - look up service name for port
+ */
+
+static char *
+lkup_svcnam(h, p, pr, ss)
+	int h;				/* porttab hash index */
+	int p;				/* port number */
+	int pr;				/* protocol: 0 = TCP, 1 = UDP */
+	int ss;				/* search status: 1 = Pth[pr][h]
+					 *		  already searched */
+{
+	static int fl[PORTTABTHRESH];
+	static int fln = 0;
+	static int gsbp = 0;
+	int i;
+	struct porttab *pt;
+	static int ptf = 0;
+	struct servent *se;
+/*
+ * Do nothing if -P has been specified.
+ */
+	if (!Fport)
+	    return((char *)NULL);
+
+	for (;;) {
+
+	/*
+	 * Search service name cache, if it hasn't already been done.
+	 * Return the name of a match.
+	 */
+	    if (!ss) {
+		for (pt = Pth[pr][h]; pt; pt = pt->next) {
+		    if (pt->port == p)
+			return(pt->name);
+		}
+	    }
+/*
+ * If fill_port() has been called, there is no service name.
+ *
+ * Do PORTTABTHRES getservbport() calls, remembering the faulures, so they
+ * won't be repeated.
+ *
+ * After PORTABTHRESH getservbyport() calls, call fill_porttab() once,
+ */
+	    if (ptf)
+		break;
+	    if (gsbp < PORTTABTHRESH) {
+		for (i = 0; i < fln; i++) {
+		    if(fl[i] == p)
+			return((char *)NULL);
+		}
+		gsbp++;
+		if ((se = getservbyport(htons(p), pr ? "udp" : "tcp")))
+		    return(se->s_name);
+		if (fln < PORTTABTHRESH)
+		    fl[fln++] = p;
+		return((char *)NULL);
+	    }
+	    (void) fill_porttab();
+	    ptf++;
+	    ss = 0;
+	}
+	return((char *)NULL);
+}
+
+
+/*
+ * print_file() - print file
+ */
+
+void
+print_file()
+{
+	char buf[128];
+	char *cp = (char *)NULL;
+	int len;
+
+	if (PrPass && !Hdr) {
+
+	/*
+	 * Print the header line of this is the second pass and the
+	 * header hasn't already been printed.
+	 */
+	    (void) printf("%-*.*s %*s", CmdColW, CmdColW, CMDTTL,
+		PidColW, PIDTTL);
+
+#if	defined(HASPPID)
+	    if (Fppid)
+	 	(void) printf(" %*s", PpidColW, PPIDTTL);
+#endif	/* defined(HASPPID) */
+
+	    if (Fpgrp)
+		(void) printf(" %*s", PgrpColW, PGRPTTL);
+	    (void) printf(" %*s %*s   %*s %*s",
+		UserColW, USERTTL,
+		FdColW - 2, FDTTL,
+		TypeColW, TYPETTL,
+		DevColW, DEVTTL);
+	    if (Foffset)
+		(void) printf(" %*s", SzOffColW, OFFTTL);
+	    else if (Fsize)
+		(void) printf(" %*s", SzOffColW, SZTTL);
+	    else
+		(void) printf(" %*s", SzOffColW, SZOFFTTL);
+	    (void) printf(" %*s %s\n", InoColW, INOTTL, NMTTL);
+	    Hdr++;
+	}
+/*
+ * Size or print the command.
+ */
+	if (!PrPass) {
+	    cp = Lp->cmd ? Lp->cmd : "(unknown)";
+	    if ((len = strlen(cp)) > CMDL)
+		len = CMDL;
+	    if (len > CmdColW)
+		CmdColW = len;
+	} else
+	    (void) printf("%-*.*s", CmdColW, CmdColW,
+		Lp->cmd ? Lp->cmd : "(unknown)");
+/*
+ * Size or print the process ID.
+ */
+	if (!PrPass) {
+	    (void) sprintf(buf, "%d", Lp->pid);
+	    if ((len = strlen(buf)) > PidColW)
+		PidColW = len;
+	} else
+	    (void) printf(" %*d", PidColW, Lp->pid);
+	if (Fpgrp) {
+
+	/*
+	 * Size or print the process group ID.
+	 */
+	    if (!PrPass) {
+		(void) sprintf(buf, "%d", Lp->pgrp);
+		if ((len = strlen(buf)) > PgrpColW)
+		    PgrpColW = len;
+	    } else
+		(void) printf(" %*d", PgrpColW, Lp->pgrp);
+	}
+
+#if	defined(HASPPID)
+	if (Fppid) {
+
+	/*
+	 * Size or print the parent process ID.
+	 */
+	    if (!PrPass) {
+		(void) sprintf(buf, "%d", Lp->ppid);
+		if ((len = strlen(buf)) > PpidColW)
+		    PpidColW = len;
+	    } else
+		(void) printf(" %*d", PpidColW, Lp->ppid);
+	}
+#endif	/* defined(HASPPID) */
+
+/*
+ * Size or print the user ID or login name.
+ */
+	if (!PrPass) {
+	    if ((len = strlen(printuid((UID_ARG)Lp->uid, NULL))) > UserColW)
+		UserColW = len;
+	} else
+	    (void) printf(" %*.*s", UserColW, UserColW,
+		printuid((UID_ARG)Lp->uid, NULL));
+/*
+ * Size or print the file descriptor, access mode and lock status.
+ */
+	if (!PrPass) {
+	    (void) sprintf(buf, "%s%c%c", Lf->fd, Lf->access, Lf->lock);
+	    if ((len = strlen(buf)) > FdColW)
+		FdColW = len;
+	} else
+	    (void) printf(" %*.*s%c%c", FdColW - 2, FdColW - 2, Lf->fd,
+		Lf->access, Lf->lock);
+/*
+ * Size or print the type.
+ */
+	if (!PrPass) {
+	    if ((len = strlen(Lf->type)) > TypeColW)
+		TypeColW = len;
+	} else
+	    (void) printf(" %*.*s", TypeColW, TypeColW, Lf->type);
+/*
+ * Size or print the device information.
+ */
+
+	if (Lf->dev_def) {
+
+#if	defined(HASPRINTDEV)
+	    cp = HASPRINTDEV(Lf);
+#else	/* !defined(HASPRINTDEV) */
+	    (void) sprintf(buf, "%d,%d", major(Lf->dev), minor(Lf->dev));
+	    cp = buf;
+#endif	/* defined(HASPRINTDEV) */
+
+	}
+
+	if (!PrPass) {
+	    if (Lf->dev_def)
+		len = strlen(cp);
+	    else if (Lf->dev_ch)
+		len = strlen(Lf->dev_ch);
+	    else
+		len = 0;
+	    if (len > DevColW)
+		DevColW = len;
+	} else {
+	    if (Lf->dev_def)
+		(void) printf(" %*.*s", DevColW, DevColW, cp);
+	    else {
+		if (Lf->dev_ch)
+		    (void) printf(" %*.*s", DevColW, DevColW, Lf->dev_ch);
+		else
+		    (void) printf(" %*.*s", DevColW, DevColW, "");
+	    }
+	}
+/*
+ * Size or print the size or offset.
+ */
+	if (!PrPass) {
+	    if (Lf->sz_def) {
+
+#if	defined(HASPRINTSZ)
+		cp = HASPRINTSZ(Lf);
+#else	/* !defined(HASPRINTSZ) */
+		(void) sprintf(buf, SzOffFmt_d, Lf->sz);
+		cp = buf;
+#endif	/* defined(HASPRINTSZ) */
+
+		len = strlen(cp);
+	    } else if (Lf->off_def) {
+
+#if	defined(HASPRINTOFF)
+		cp = HASPRINTOFF(Lf, 0);
+#else	/* !defined(HASPRINTOFF) */
+		(void) sprintf(buf, SzOffFmt_0t, Lf->off);
+		cp = buf;
+#endif	/* defined(HASPRINTOFF) */
+
+		len = strlen(cp);
+		if (OffDecDig && len > (OffDecDig + 2)) {
+
+#if	defined(HASPRINTOFF)
+		    cp = HASPRINTOFF(Lf, 1);
+#else	/* !defined(HASPRINTOFF) */
+		    (void) sprintf(buf, SzOffFmt_x, Lf->off);
+		    cp = buf;
+#endif	/* defined(HASPRINTOFF) */
+
+		    len = strlen(cp);
+		}
+	    } else
+		len = 0;
+	    if (len > SzOffColW)
+		SzOffColW = len;
+	} else {
+	    putchar(' ');
+	    if (Lf->sz_def)
+
+#if	defined(HASPRINTSZ)
+		(void) printf("%*.*s", SzOffColW, SzOffColW, HASPRINTSZ(Lf));
+#else	/* !defined(HASPRINTSZ) */
+		(void) printf(SzOffFmt_dv, SzOffColW, Lf->sz);
+#endif	/* defined(HASPRINTSZ) */
+
+	    else if (Lf->off_def) {
+
+#if	defined(HASPRINTOFF)
+		cp = HASPRINTOFF(Lf, 0);
+#else	/* !defined(HASPRINTOFF) */
+		(void) sprintf(buf, SzOffFmt_0t, Lf->off);
+		cp = buf;
+#endif	/* defined(HASPRINTOFF) */
+
+		if (OffDecDig && (int)strlen(cp) > (OffDecDig + 2)) {
+
+#if	defined(HASPRINTOFF)
+		    cp = HASPRINTOFF(Lf, 1);
+#else	/* !defined(HASPRINTOFF) */
+		    (void) sprintf(buf, SzOffFmt_x, Lf->off);
+		    cp = buf;
+#endif	/* defined(HASPRINTOFF) */
+
+		}
+		(void) printf("%*.*s", SzOffColW, SzOffColW, cp);
+	    } else
+		(void) printf("%*.*s", SzOffColW, SzOffColW, "");
+	}
+/*
+ * Size or print the inode information.
+ */
+	switch (Lf->inp_ty) {
+	case 1:
+
+#if	defined(HASPRINTINO)
+	    cp = HASPRINTINO(Lf);
+#else	/* !defined(HASPRINTINO) */
+	    (void) sprintf(buf, "%lu", Lf->inode);
+	    cp = buf;
+#endif	/* defined(HASPRINTINO) */
+
+	    break;
+	case 2:
+	    if (Lf->iproto[0]) {
+		cp = Lf->iproto;
+		break;
+	    }
+	    /* else fall through */
+	default:
+	    cp = "";
+	}
+	if (!PrPass) {
+	    if ((len = strlen(cp)) > InoColW)
+		InoColW = len;
+	} else {
+	    (void) printf(" %*.*s", InoColW, InoColW, cp);
+	}
+/*
+ * If this is the second pass, print the name column.  (It doesn't need
+ * to be sized.)
+ */
+	if (PrPass) {
+	    putchar(' ');
+
+#if	defined(HASPRINTNM)
+	    HASPRINTNM(Lf);
+#else	/* !defined(HASPRINTNM) */
+	    printname(1);
+#endif	/* defined(HASPRINTNM) */
+
+	}
+}
+
+
 /*
  * printinaddr() - print Internet addresses
  */
@@ -356,7 +854,7 @@ lkup_port(p, pr)
 static int
 printinaddr()
 {
-	int i, j, len, ty;
+	int i, len, src;
 	char *host, *port;
 	int nl = MAXPATHLEN - 1;
 	char *np = Namech;
@@ -397,16 +895,30 @@ addr_too_long:
 	 * Process the port number.
 	 */
 	    if (Lf->li[i].p > 0) {
-		if (Fport) {
+		if (Fport || FportMap) {
 
 		/*
-		 * If converting port numbers to service names, check the
-		 * protocol.
+		 * If converting port numbers to service names, or looking
+		 * up portmap program names and numbers, do so by protocol.
+		 *
+		 * Identify the port source as local if: 1) it comes from the
+		 * local entry (0) of the file's Internet address array; or
+		 * 2) it comes from  the foreign entry (1), and the foreign
+		 * Internet address matches the local one; or 3) it is the
+		 * loopback address 127.0.0.1.  (Test 2 may not always work
+		 * -- e.g., on hosts with multiple interfaces.)
 		 */
+		    if ((src = i) && FportMap
+		    &&  (Lf->li[i].ia.s_addr == INADDR_LOOPBACK
+		    ||   (Lf->li[0].as &&
+		          Lf->li[0].ia.s_addr == Lf->li[1].ia.s_addr)))
+		    {
+			src = 0;
+		    }
 		    if (strcasecmp(Lf->iproto, "TCP") == 0)
-			port = lkup_port(Lf->li[i].p, 0);
+			port = lkup_port(Lf->li[i].p, 0, src);
 		    else if (strcasecmp(Lf->iproto, "UDP") == 0)
-			port = lkup_port(Lf->li[i].p, 1);
+			port = lkup_port(Lf->li[i].p, 1, src);
 		}
 		if (!port) {
 		    (void) sprintf(pbuf, "%d", Lf->li[i].p);
@@ -438,7 +950,7 @@ addr_too_long:
 		nl -= (len + 1);
 	    }
 	}
-	if (Namech) {
+	if (Namech[0]) {
 	    (void) fputs(Namech, stdout);
 	    return(1);
 	}
@@ -446,6 +958,33 @@ addr_too_long:
 }
 
 
+/*
+ * print_init() - initialize for printing
+ */
+
+void
+print_init()
+{
+	PrPass = (Ffield || Fterse) ? 1 : 0;
+	CmdColW = strlen(CMDTTL);
+	DevColW = strlen(DEVTTL);
+	FdColW = strlen(FDTTL);
+	InoColW = strlen(INOTTL);
+	NmColW = strlen(NMTTL);
+	PgrpColW = strlen(PGRPTTL);
+	PidColW = strlen(PIDTTL);
+	PpidColW = strlen(PPIDTTL);
+	if (Fsize)
+	    SzOffColW = strlen(SZTTL);
+	else if (Foffset)
+	    SzOffColW = strlen(OFFTTL);
+	else
+	    SzOffColW = strlen(SZOFFTTL);
+	TypeColW = strlen(TYPETTL);
+	UserColW = strlen(USERTTL);
+}
+
+
 /*
  * printiproto() - print Internet protocol name
  */
@@ -551,7 +1090,7 @@ printiproto(p)
 #endif
 
 	default:
-		s = NULL;
+		s = (char *)NULL;
 	}
 	if (s)
 		(void) sprintf(Lf->iproto, "%.*s", IPROTOL-1, s);
@@ -615,7 +1154,7 @@ printname(nl)
 	}
 
 #if	defined(HASBLKDEV)
-	if (Lf->ntype == N_BLK && printbdevname(&Lf->dev, 0)) {
+	if (Lf->ntype == N_BLK && Lf->dev_def && printbdevname(&Lf->dev, 0)) {
 
 	/*
 	 * If this is a block device and it has a name, print it.
@@ -634,6 +1173,12 @@ printname(nl)
 	    ps++;
 	    goto print_nma;
 	}
+
+#if	defined(HASPRIVNMCACHE)
+	if (HASPRIVNMCACHE(Lf))
+	    goto print_nma;
+#endif	/* defined(HASPRIVNMCACHE) */
+
 	if (Lf->fsdir || Lf->fsdev) {
 
 	/*
@@ -779,12 +1324,12 @@ printuid(u, ty)
 	 * Define the UID cache, if necessary.
 	 */
 	    if (!uc) {
-		if ((uc = (struct uidcache **)calloc(UIDCACHEL,
-						sizeof(struct uidcache *)))
-		== (struct uidcache **)NULL) {
+		if (!(uc = (struct uidcache **)calloc(UIDCACHEL,
+						sizeof(struct uidcache *))))
+		{
 		    (void) fprintf(stderr,
 			"%s: no space for %d byte UID cache hash buckets\n",
-			Pn, UIDCACHEL * (sizeof(struct uidcache *)));
+			Pn, (int)(UIDCACHEL * (sizeof(struct uidcache *))));
 		    Exit(1);
 		}
 		if (CkPasswd) {
@@ -800,7 +1345,7 @@ printuid(u, ty)
 		if (sbs.st_mtime != sb.st_mtime || sbs.st_ctime != sb.st_ctime)
 		{
 		    for (i = 0; i < UIDCACHEL; i++) {
-			if ((up = uc[i]) != (struct uidcache *)NULL) {
+			if (!(up = uc[i])) {
 			    do {
 				upn = up->next;
 				(void) free((FREE_P *)up);
@@ -828,21 +1373,21 @@ printuid(u, ty)
 	 *
 	 * Look up the login name from the UID for a new cache entry.
 	 */
-	    if ((pw = getpwuid((uid_t)u)) == (struct passwd *)NULL) {
+	    if (!(pw = getpwuid((uid_t)u))) {
 		if (!Fwarn) {
-		    (void) fprintf(stderr, "%s: no pwd entry for UID %d\n",
-			Pn, u);
+		    (void) fprintf(stderr, "%s: no pwd entry for UID %lu\n",
+			Pn, (unsigned long)u);
 		}
 	    } else {
 
 	    /*
 	     * Allocate and fill a new cache entry.  Link it to its hash bucket.
 	     */
-		if ((upn = (struct uidcache *)malloc(sizeof(struct uidcache)))
-		== (struct uidcache *)NULL) {
+		if (!(upn = (struct uidcache *)malloc(sizeof(struct uidcache))))
+		{
 		    (void) fprintf(stderr,
 			"%s: no space for UID cache entry for: %lu, %s)\n",
-			(unsigned long)u, pw->pw_name);
+			Pn, (unsigned long)u, pw->pw_name);
 		    Exit(1);
 		}
 		(void) strncpy(upn->nm, pw->pw_name, LNML);
@@ -858,7 +1403,7 @@ printuid(u, ty)
 /*
  * Produce a numeric conversion of the UID.
  */
-	(void) sprintf(user, "%*u", USERPRTL, u);
+	(void) sprintf(user, "%*lu", USERPRTL, (unsigned long)u);
 	if (ty)
 	    *ty = 1;
 	return(user);
@@ -1167,3 +1712,37 @@ printunkaf(fam)
 	(void) sprintf(Namech, "no further information on %sAF_%s", p, s);
 	return;
 }
+
+
+/*
+ * update_portmap() - update a portmap entry with its port number or service
+ *		      name
+ */
+
+static void
+update_portmap(pt, pn)
+	struct porttab *pt;		/* porttab entry */
+	char *pn;			/* port name */
+{
+	MALLOC_S al, nl;
+	char *cp;
+
+	if (pt->ss)
+	    return;
+	if (!(al = strlen(pn))) {
+	    pt->ss = 1;
+	    return;
+	}
+	nl = al + pt->nl + 2;
+	if (!(cp = (char *)malloc(nl + 1))) {
+	    (void) fprintf(stderr,
+		"%s: can't allocate %d bytes for portmap name: %s[%s]\n",
+		Pn, (int)(nl + 1), pn, pt->name);
+	    Exit(1);
+	}
+	(void) sprintf(cp, "%s[%s]", pn, pt->name);
+	(void) free((FREE_P *)pt->name);
+	pt->name = cp;
+	pt->nl = nl;
+	pt->ss = 1;
+}
